@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
+ *   Copyright (C) 2005-2006 by Albert Astals Cid <aacid@kde.org>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -11,22 +11,24 @@
 #define HIGHSCOREDIALOG_H
 
 #include <qpair.h>
-#include <qvaluelist.h>
+#include <qlist.h>
 
-#include <kdialogbase.h>
+#include <kdialog.h>
+
+class QSvgRenderer;
 
 class myTabWidget;
 
-class highScoreDialog : private KDialogBase
+class highScoreDialog : private KDialog
 {
 	public:
-		highScoreDialog(QWidget *parent);
+		highScoreDialog(QWidget *parent, QSvgRenderer *renderer);
 		
 		bool scoreGoodEnough(int level, int score);
 		void addScore(int level, int score, const QString &name);
 		void showLevel(int level);
 	
-		QValueList< QPair<int, QString> > m_scores[3];
+		QList< QPair<int, QString> > m_scores[3];
 		myTabWidget *m_tw;
 };
 
