@@ -1,5 +1,5 @@
 /*************************************************************************************
- *  Copyright (C) 2008 by Aleix Pol <aleixpol@gmail.com>                             *
+ *  Copyright (C) 2008 by Aleix Pol <aleixpol@kde.org>                               *
  *                                                                                   *
  *  This program is free software; you can redistribute it and/or                    *
  *  modify it under the terms of the GNU General Public License                      *
@@ -21,10 +21,13 @@
 
 #include "expressionwriter.h"
 
+namespace Analitza
+{
+
 /**
  *	This class represents the string expression writer.
  *
- *	@author Aleix Pol <aleixpol@gmail.com>
+ *	@author Aleix Pol <aleixpol@kde.org>  
  */
 
 class HtmlExpressionWriter : public ExpressionWriter
@@ -37,11 +40,17 @@ class HtmlExpressionWriter : public ExpressionWriter
 		virtual QString accept(const Container* var);
 		virtual QString accept(const Operator* var);
 		virtual QString accept(const Vector* var);
+		virtual QString accept(const List* l);
+		virtual QString accept(const Apply* a);
 		
 		QString result() const { return m_result; }
 		
 	private:
+		template <class T>
+			static QStringList allValues(T it, const T& itEnd, ExpressionWriter* writer);
 		QString m_result;
 };
 
+}
+
 #endif
