@@ -91,12 +91,3 @@ bool List::operator==(const List& v) const
 	}
 	return eq;
 }
-
-bool List::decorate(const QMap< QString, Object** >& scope)
-{
-	bool ret=false;
-	foreach(Object* o, m_elements) {
-		ret |= o->decorate(scope);
-	}
-	return ret;
-}
