@@ -22,6 +22,10 @@
 #include <QClipboard>
 #include <QApplication>
 #include <QScrollBar>
+#include <QTemporaryFile>
+#include <QTimer>
+#include <QTextDocument>
+#include <QDir>
 
 #include <KLocale>
 #include <KStandardAction>
@@ -30,27 +34,39 @@
 #include <KAction>
 #include <kio/copyjob.h>
 #include <kio/netaccess.h>
-
-#include "explexer.h"
-#include "expressionparser.h"
-#include "variables.h"
-#include "expression.h"
-#include <qtimer.h>
-#include <qtextdocument.h>
 #include <kio/job.h>
-#include <qtemporaryfile.h>
-#include <QDir>
+
+#include <analitza/explexer.h>
+#include <analitza/expressionparser.h>
+#include <analitza/variables.h>
+#include <analitza/expression.h>
 
 ConsoleHtml::ConsoleHtml(QWidget *parent) : KHTMLPart(parent), m_mode(Evaluation)
 {
-	QPalette p=qApp->palette();
-	
 	setJScriptEnabled(false);
 	setJavaEnabled(false);
 	setMetaRefreshEnabled(false);
 	setPluginsEnabled(false);
 	setOnlyLocalReferences(true);
 	
+	view()->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+	
+	connect(this, SIGNAL(popupMenu(const QString &, const QPoint &)), this, SLOT(context(const QString &, const QPoint &)));
+	connect(browserExtension(), SIGNAL(openUrlRequest(KUrl, KParts::OpenUrlArguments, KParts::BrowserArguments)), SLOT(openClickedUrl(KUrl)));
+	connect(view()->verticalScrollBar(), SIGNAL(rangeChanged(int,int)), SLOT(scrollDown(int, int)));
+	
+	QMetaObject::invokeMethod(this, "initialize", Qt::QueuedConnection);
+}
+
+ConsoleHtml::~ConsoleHtml()
+{
+	qDeleteAll(m_options);
+}
+
+void ConsoleHtml::initialize()
+{
+	QPalette p=qApp->palette();
+	
 	m_css ="<style type=\"text/css\">\n";
 	m_css +=QString("\thtml { background-color: %1; }\n").arg(p.color(QPalette::Active, QPalette::Base).name());
 	m_css +="\t.error { border-style: solid; border-width: 1px; border-color: #ff3b21; background-color: #ffe9c4; padding:7px;}\n";
@@ -66,25 +82,27 @@ ConsoleHtml::ConsoleHtml(QWidget *parent) : KHTMLPart(parent), m_mode(Evaluation
 	m_css +="\t.keyword { color: #000064; }\n";
 	m_css +="\t.func { color: #008600; }\n";
 	m_css +="\t.result { padding-left: 10%; }\n";
+	m_css +="\t.options { font-size: small; text-align:right }\n";
 	m_css +="\tli { padding-left: 12px; padding-bottom: 4px; list-style-position: inside; }";
 	m_css +="</style>\n";
 	
-	begin();
-	write("<html>\n<head>"+m_css+"</head></html>");
-	end();
-	
-	connect(this, SIGNAL(popupMenu(const QString &, const QPoint &)), this, SLOT(context(const QString &, const QPoint &)));
+// 	begin();
+// 	write("<html>\n<head>"+m_css+"</head></html>");
+// 	end();
 }
 
-ConsoleHtml::~ConsoleHtml() {}
-
-/*QString printable(const QString &s)
+void ConsoleHtml::openClickedUrl(const KUrl& url)
 {
-	QString s1(s);
-	s1.replace("<", "&lt;");
-	s1.replace(">", "&gt;");
-	return s1;
-}*/
+	QString id =url.queryItem("id");
+	QString exp=url.queryItem("func");
+	
+	qDebug() << "open!!" << id;
+	foreach(InlineOptions* opt, m_options) {
+		if(opt->id() == id) {
+			opt->triggerOption(Analitza::Expression(exp, false));
+		}
+	}
+}
 
 bool ConsoleHtml::addOperation(const Analitza::Expression& e, const QString& input)
 {
@@ -100,9 +118,34 @@ bool ConsoleHtml::addOperation(const Analitza::Expression& e, const QString& inp
 		}
 	}
 	
+	QString options;
 	if(a.isCorrect()) {
 		result = res.toHtml();
 		
+		Analitza::Expression lambdaexp = a.dependenciesToLambda();
+		QStringList bvars = lambdaexp.bvarList();
+		
+		Analitza::ExpressionType functype;
+		{
+			Analitza::Analyzer b;
+			b.setExpression(lambdaexp);
+			
+			functype = b.type();
+		}
+		
+		foreach(InlineOptions* opt, m_options) {
+			if(opt->matchesExpression(lambdaexp, functype)) {
+				KUrl url("/query");
+				url.addQueryItem("id", opt->id());
+				url.addQueryItem("func", lambdaexp.toString());
+				
+				options += i18n(" <a href='%1'>%2</a>", url.url(), opt->caption());
+			}
+		}
+		
+		if(!options.isEmpty())
+			options = "<div class='options'>"+i18n("Options: %1", options)+"</div>";
+		
 		a.insertVariable("ans", res);
 		m_script += e; //Script won't have the errors
 		newEntry = QString("%1<br />=<span class='result'>%2</span>").arg(e.toHtml()).arg(result);
@@ -110,7 +153,7 @@ bool ConsoleHtml::addOperation(const Analitza::Expression& e, const QString& inp
 		m_htmlLog += i18n("<ul class='error'>Error: <b>%1</b><li>%2</li></ul>", Qt::escape(input), a.errors().join("</li>\n<li>"));
 	}
 	
-	updateView(newEntry);
+	updateView(newEntry, options);
 	
 	return a.isCorrect();
 }
@@ -166,7 +209,7 @@ bool ConsoleHtml::loadScript(const KUrl& path)
 	
 	if(!correct) {
 		m_htmlLog += i18n("<ul class='error'>Error: Could not load %1</ul>", path.prettyUrl());
-		updateView(QString());
+		updateView(QString(), QString());
 	}
 	
 	return correct;
@@ -183,9 +226,8 @@ bool ConsoleHtml::saveScript(const KUrl & path) const
 	
 	if(correct) {
 		QTextStream out(&file);
-		QList<Analitza::Expression>::const_iterator it = m_script.begin();
-		for(; it!=m_script.end(); ++it)
-			out << it->toString() << endl;
+		foreach(const Analitza::Expression& exp, m_script)
+			out << exp.toString() << endl;
 	}
 	
 	if(!path.isLocalFile()) {
@@ -220,7 +262,7 @@ bool ConsoleHtml::saveLog(const KUrl& path) const
 	return correct;
 }
 
-void ConsoleHtml::updateView(const QString& newEntry)
+void ConsoleHtml::updateView(const QString& newEntry, const QString& options)
 {
 	//FIXME: Check that the output html is not correct
 	begin();
@@ -228,24 +270,23 @@ void ConsoleHtml::updateView(const QString& newEntry)
 	write("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\">\n<head>\n\t<title> :) </title>\n");
 	write(m_css);
 	write("</head>\n<body>");
-	foreach(const QString &entry, m_htmlLog) {
+	foreach(const QString &entry, m_htmlLog)
 		write("<p class='normal'>"+entry+"</p>");
-	}
+	
 	if(!newEntry.isEmpty()) {
 		m_htmlLog += newEntry;
+		write(options);
 		write("<p class='last'>"+newEntry+"</p>");
 	}
 	write("</body></html>");
 	end();
 	
 	emit changed();
-	
-	QTimer::singleShot(0, this, SLOT(scrollDown()));
 }
 
-void ConsoleHtml::scrollDown()
+void ConsoleHtml::scrollDown(int min, int max)
 {
-	view()->verticalScrollBar()->setValue(view()->verticalScrollBar()->maximum());
+	view()->verticalScrollBar()->setValue(max);
 }
 
 void ConsoleHtml::copy() const
@@ -272,7 +313,7 @@ void ConsoleHtml::clear()
 {
 	m_script.clear();
 	m_htmlLog.clear();
-	updateView(QString());
+	updateView(QString(), QString());
 }
 
 void ConsoleHtml::modifyVariable(const QString& name, const Analitza::Expression& exp)
