@@ -18,17 +18,9 @@
 
 #include "kalgebra.h"
 #include "varedit.h"
-#include "functionedit.h"
 #include "consolehtml.h"
-#include "expressionedit.h"
-#include "graph2d.h"
-#include "variables.h"
-#include "variablesmodel.h"
 #include "dictionary.h"
-#include "functionsmodel.h"
-#include "functionsview.h"
 #include "askname.h"
-#include "value.h"
 #include "variablesdelegate.h"
 #include "viewportwidget.h"
 #ifdef HAVE_OPENGL
@@ -36,6 +28,16 @@
 #endif
 
 #include <analitzagui/operatorsmodel.h>
+#include <analitzagui/functionfactory.h>
+
+#include <analitzagui/functionedit.h>
+#include <analitzagui/functionsmodel.h>
+#include <analitzagui/functionsview.h>
+#include <analitzagui/expressionedit.h>
+#include <analitzagui/graph2d.h>
+#include <analitzagui/variablesmodel.h>
+#include <analitza/variables.h>
+#include <analitza/value.h>
 
 #include <QVBoxLayout>
 #include <QHeaderView>
@@ -54,13 +56,60 @@
 #include <KProcess>
 #include <KRecentFilesAction>
 #include <KApplication>
+#include <kabstractfilewidget.h>
+
+class Add2DOption : public InlineOptions
+{
+	public:
+		Add2DOption(KAlgebra* c)
+		: m_kalgebra(c)
+		{}
+		
+		virtual QString id() const { return "add2d"; }
+		virtual bool matchesExpression(const Analitza::Expression& exp, const Analitza::ExpressionType& functype) const
+		{
+			bool cont = FunctionFactory::self()->contains(exp.bvarList());
+				
+			return cont && functype.canReduceTo(FunctionFactory::self()->type(exp.bvarList()));
+		}
+
+		virtual QString caption() const { return i18n("Plot 2D"); }
+
+		virtual void triggerOption(const Analitza::Expression& exp) { m_kalgebra->add2D(exp); }
+	
+	private:
+		KAlgebra* m_kalgebra;
+};
+
+#ifdef HAVE_OPENGL
+class Add3DOption : public InlineOptions
+{
+	public:
+		Add3DOption(KAlgebra* c)
+		: m_kalgebra(c)
+		{}
+		
+		virtual QString id() const { return "add3d"; }
+		virtual bool matchesExpression(const Analitza::Expression& exp, const Analitza::ExpressionType& functype) const
+		{
+			return Graph3D::checkExpression(exp.bvarList(), functype);
+		}
+
+		virtual QString caption() const { return i18n("Plot 3D"); }
+
+		virtual void triggerOption(const Analitza::Expression& exp) { m_kalgebra->add3D(exp); }
+	
+	private:
+		KAlgebra* m_kalgebra;
+};
+#endif
 
 KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 {
 	resize(900, 500);
 	
-	QTabWidget *tabs = new QTabWidget(this);
-	this->setCentralWidget(tabs);
+	m_tabs = new QTabWidget(this);
+	this->setCentralWidget(m_tabs);
 	
 	setStatusBar(new KStatusBar(this));
 	setMenuBar(new KMenuBar(this));
@@ -74,7 +123,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
     menuBar()->addMenu("|")->setEnabled(false);
 	
 	///////Console
-	QWidget *console = new QWidget(tabs);
+	QWidget *console = new QWidget(m_tabs);
 	QVBoxLayout *c_layo = new QVBoxLayout(console);
 	c_results = new ConsoleHtml(console);
 	c_results->view()->setFocusPolicy(Qt::NoFocus);
@@ -97,7 +146,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	c_exp->setExamples(QStringList() << "square:=x->x**2" << "fib:=n->piecewise { eq(n,0)?0, eq(n,1)?1, ?fib(n-1)+fib(n-2) }");
 	c_dock_vars->setWidget(c_variables);
 	
-	tabs->addTab(console, i18n("&Console"));
+	m_tabs->addTab(console, i18n("&Console"));
 	console->setLayout(c_layo);
 	c_layo->addWidget(c_results->view());
 	c_layo->addWidget(c_exp);
@@ -108,7 +157,6 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	connect(c_results, SIGNAL(changed()), c_exp, SLOT(updateCompleter()));
 	connect(c_results, SIGNAL(paste(QString)), c_exp, SLOT(insertText(QString)));
 	connect(c_variables, SIGNAL(clicked(QModelIndex)), this, SLOT(edit_var(QModelIndex)));
-	
 	////////menu
 	c_menu = menuBar()->addMenu(i18n("C&onsole"));
 	
@@ -122,6 +170,16 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 						this, SLOT(saveScript()), Qt::CTRL+Qt::Key_G);
 	c_menu->addAction(KIcon("document-save"), i18nc("@item:inmenu", "&Export Log..."),
 						this, SLOT(saveLog()), QKeySequence::Save);
+	c_menu->addSeparator()->setText(i18n("Execution Mode"));
+	QActionGroup *execGroup = new QActionGroup(c_menu);
+	QAction* calc = c_menu->addAction(i18nc("@item:inmenu", "Calculate"), this, SLOT(consoleCalculate()));
+	QAction* eval = c_menu->addAction(i18nc("@item:inmenu", "Evaluate"), this, SLOT(consoleEvaluate()));
+	
+	calc->setCheckable(true);
+	eval->setCheckable(true);
+	eval->setChecked(true);
+	execGroup->addAction(calc);
+	execGroup->addAction(eval);
 	c_menu->addSeparator();
 	c_menu->addAction(KStandardAction::clear(c_results, SLOT(clear()), this));
 	initializeRecentScripts();
@@ -131,7 +189,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	//////2D Graph
 	b_funcsModel=new FunctionsModel(this);
 	
-	m_graph2d = new Graph2D(b_funcsModel, tabs);
+	m_graph2d = new Graph2D(b_funcsModel, m_tabs);
 	
 	b_dock_funcs = new QDockWidget(i18n("Functions"), this);
 	b_tools = new QTabWidget(b_dock_funcs);
@@ -170,7 +228,8 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	
 	b_dock_funcs->setWidget(b_tools);
 	b_dock_funcs->setFeatures(QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetMovable);
-	tabs->addTab(m_graph2d, i18n("&2D Graph"));
+	m_tabs->addTab(m_graph2d, i18n("&2D Graph"));
+	c_results->addOptionsObserver(new Add2DOption(this));
 	
 	connect(b_varsModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(valueChanged()));
 	connect(b_funcs, SIGNAL(doubleClicked(const QModelIndex &)), this, SLOT(edit_func(const QModelIndex &)));
@@ -218,7 +277,7 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	
 	/////3DGraph
 #ifdef HAVE_OPENGL
-	QWidget *tridim = new QWidget(tabs);
+	QWidget *tridim = new QWidget(m_tabs);
 	QVBoxLayout *t_layo = new QVBoxLayout(tridim);
 	t_exp = new ExpressionEdit(tridim);
 	t_exp->setExamples(QStringList() << "sin x+sin y" << "(x,y)->x" << "x*y");
@@ -226,12 +285,13 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	m_graph3d = new Graph3D(tridim);
 	
 	tridim->setLayout(t_layo);
-	tabs->addTab(tridim, i18n("&3D Graph"));
+	m_tabs->addTab(tridim, i18n("&3D Graph"));
 	t_layo->addWidget(m_graph3d);
 	t_layo->addWidget(t_exp);
 	
 	connect(t_exp,  SIGNAL(returnPressed()), this, SLOT(new_func3d()));
 	connect(m_graph3d, SIGNAL(status(const QString &)), this, SLOT(changeStatusBar(const QString &)));
+	c_results->addOptionsObserver(new Add3DOption(this));
 	
 	////////menu
 	t_menu = menuBar()->addMenu(i18n("3D &Graph"));
@@ -263,8 +323,8 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	d_dock = new QDockWidget(i18n("Operations"), this);
 	d_dock->setFeatures(QDockWidget::DockWidgetFloatable | QDockWidget::DockWidgetMovable);
 	this->addDockWidget(Qt::RightDockWidgetArea, d_dock);
-	Dictionary *dic = new Dictionary(tabs);
-	tabs->addTab(dic, i18n("&Dictionary"));
+	Dictionary *dic = new Dictionary(m_tabs);
+	m_tabs->addTab(dic, i18n("&Dictionary"));
 	
 	QWidget *w=new QWidget;
 	QLayout *leftLayo=new QVBoxLayout(w);
@@ -288,12 +348,12 @@ KAlgebra::KAlgebra(QWidget *parent) : KMainWindow(parent)
 	//Ego's reminder
 	menuBar()->addMenu(helpMenu());
 	
-	connect(b_funcsModel, SIGNAL(functionModified(QString, Expression)),
-			c_results, SLOT(modifyVariable(QString, Expression)));
+	connect(b_funcsModel, SIGNAL(functionModified(QString, Analitza::Expression)),
+			c_results, SLOT(modifyVariable(QString, Analitza::Expression)));
 	connect(b_funcsModel, SIGNAL(functionRemoved(QString)),
 			c_results, SLOT(removeVariable(QString)));
 	
-	connect(tabs, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));
+	connect(m_tabs, SIGNAL(currentChanged(int)), this, SLOT(tabChanged(int)));
 	tabChanged(0);
 }
 
@@ -324,9 +384,19 @@ void KAlgebra::newInstance()
 	KProcess::startDetached(QApplication::applicationFilePath());
 }
 
+void KAlgebra::add2D(const Analitza::Expression& exp)
+{
+	qDebug() << "adding" << exp.toString();
+	
+	Function f(b_funcsModel->freeId(), exp, c_results->analitza()->variables(), QPen(Qt::blue), 6, 0);
+	b_funcsModel->addFunction(f);
+	
+	m_tabs->setCurrentIndex(1);
+}
+
 void KAlgebra::new_func()
 {
-	function f=b_funced->createFunction();
+	Function f=b_funced->createFunction();
 	
 	if(b_funced->editing()) {
 		b_funcsModel->editFunction(f.name(), f);
@@ -413,7 +483,7 @@ void KAlgebra::loadScript(const KUrl& path)
 
 void KAlgebra::saveScript()
 {
-	KUrl path = KFileDialog::getSaveUrl(KUrl(), "*.kal|"+i18n("Script (*.kal)"), this);
+	KUrl path = KFileDialog::getSaveUrl(KUrl(), "*.kal|"+i18n("Script (*.kal)"), this, QString(), KFileDialog::ConfirmOverwrite);
 	bool loaded=false;
 	if(!path.isEmpty())
 		loaded=c_results->saveScript(path);
@@ -424,7 +494,7 @@ void KAlgebra::saveScript()
 
 void KAlgebra::saveLog()
 {
-	KUrl path = KFileDialog::getSaveUrl(KUrl(), "*.html|"+i18n("HTML File (*.html)"),  this);
+	KUrl path = KFileDialog::getSaveUrl(KUrl(), "*.html|"+i18n("HTML File (*.html)"), this, QString(), KFileDialog::ConfirmOverwrite);
 	if(!path.isEmpty())
 		c_results->saveLog(path);
 }
@@ -466,7 +536,7 @@ void KAlgebra::set_solid()
 void KAlgebra::save3DGraph()
 {
 #ifdef HAVE_OPENGL
-	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|PNG File"), this);
+	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|PNG File"), this, QString(), KFileDialog::ConfirmOverwrite);
 	if(!path.isEmpty())
 		m_graph3d->toPixmap().save(path, "PNG");
 #endif
@@ -491,9 +561,19 @@ void KAlgebra::toggleKeepAspect()
 
 void KAlgebra::saveGraph()
 {
-	QString path = KFileDialog::getSaveFileName(KUrl(), i18n("*.png|Image File\n*.svg|SVG File"), this);
-	if(!path.isEmpty())
-		m_graph2d->toImage(path);
+	KFileDialog dialog(KUrl(), i18n("*.png|Image File\n*.svg|SVG File"), this);
+	dialog.setOperationMode(KFileDialog::Saving);
+	dialog.setConfirmOverwrite(true);
+	
+	if(dialog.exec()) {
+		QString filter = dialog.fileWidget()->currentFilter();
+		QString filename = dialog.selectedFile();
+		
+		Graph2D::Format f=Graph2D::PNG;
+		if(filename.endsWith(".svg") || (!filename.endsWith(".png") && filter.mid(2, 3)=="svg"))
+			f=Graph2D::SVG;
+		m_graph2d->toImage(filename, f);
+	}
 }
 
 void KAlgebra::tabChanged(int n)
@@ -504,8 +584,9 @@ void KAlgebra::tabChanged(int n)
 	
 	c_menu->setEnabled(false);
 	b_menu->setEnabled(false);
+#ifdef HAVE_OPENGL
 	t_menu->setEnabled(false);
-	
+#endif
 	m_status->clear();
 	
 	switch(n) {
@@ -551,6 +632,16 @@ void KAlgebra::updateInformation()
 	c_variables->header()->resizeSections(QHeaderView::ResizeToContents);
 }
 
+void KAlgebra::consoleCalculate()
+{
+	c_results->setMode(ConsoleHtml::Calculation);
+}
+
+void KAlgebra::consoleEvaluate()
+{
+	c_results->setMode(ConsoleHtml::Evaluation);
+}
+
 void KAlgebra::valueChanged()
 {
 	//FIXME: Should only repaint the affected ones.
@@ -572,6 +663,14 @@ void KAlgebra::varsContextMenu(const QPoint& p)
 	}
 }
 
+void KAlgebra::add3D(const Analitza::Expression& exp)
+{
+#ifdef HAVE_OPENGL
+	m_graph3d->setFunc(exp);
+	m_tabs->setCurrentIndex(2);
+#endif
+}
+
 void KAlgebra::dictionaryFilterChanged(const QString&)
 {
 	if(d_list->model()->rowCount()==1)
