@@ -27,7 +27,6 @@
 #include <kmessagebox.h>
 #include <knumvalidator.h>
 #include <klineedit.h>
-#include <KStandardDirs>
 
 /* these includes are needed for Qt support */
 #include <qapplication.h>
@@ -37,8 +36,6 @@
 
 //Added by qt3to4:
 #include <QGridLayout>
-#include <QHBoxLayout>
-#include <QVBoxLayout>
 
 #include "settingsclass.h"
 
