@@ -465,7 +465,7 @@ static const ArgsParser::spec argsspecParabolaBDP[] =
   { AbstractLineImp::stype(), I18N_NOOP( "Construct a parabola with this directrix" ),
     I18N_NOOP( "Select the directrix of the new parabola..." ), false },
   { PointImp::stype(), I18N_NOOP( "Construct a parabola with this focus" ),
-    I18N_NOOP( "Select the focus of the new parabola..." ), true }
+    I18N_NOOP( "Select the focus of the new parabola..." ), false }
 };
 
 KIG_INSTANTIATE_OBJECT_TYPE_INSTANCE( ParabolaBDPType )
@@ -496,7 +496,7 @@ ObjectImp* ParabolaBDPType::calc( const LineData& l, const Coordinate& c ) const
   Coordinate fa = c - l.a;
   ret.pdimen = fa.y*ldir.x - fa.x*ldir.y;
   ConicImpPolar* r = new ConicImpPolar( ret );
-  kdDebug() << k_funcinfo << r->conicTypeString() << endl;
+  kDebug() << r->conicTypeString();
   return r;
 }
 
@@ -684,6 +684,6 @@ void ConicRadicalType::executeAction( int i, ObjectHolder&, ObjectTypeCalcer& t,
   zeroindexo->setImp( new IntImp( newzeroindex ) );
   KigCommand* kc = new KigCommand( d, "Switch Conic Radical Lines" );
   mon.finish( kc );
-  d.history()->addCommand( kc );
+  d.history()->push( kc );
 }
 
