@@ -15,8 +15,8 @@
 #ifndef HTMESH_H
 #define HTMESH_H
 
+#include <cstdio>
 #include "typedef.h"
-#include <stdio.h>
 
 class SpatialIndex;
 class RangeConvex;
@@ -94,7 +94,14 @@ public:
          * the results by supplying an optional integer bufNum parameter.
          */
 
-        /* @short finds the trixels that cover the specified circle
+        /**
+         *@short finds the trixels that cover the specified circle
+         *@param ra Central ra in degrees
+         *@param dec Central dec in degrees
+         *@param radius Radius of the circle in degrees
+         *@note You will need to supply unprecessed (ra, dec) in most
+         * situations. Please see SkyMesh::aperture()'s code before
+         * messing with this method.
          */
         void intersect(double ra, double dec, double radius,
                        BufNum bufNum=0); 
