@@ -17,7 +17,6 @@
 
 #include "deepskycomponent.h"
 
-#include <QPainter>
 #include <QDir>
 #include <QFile>
 
@@ -33,23 +32,18 @@
 #include "skylabeler.h"
 #include "Options.h"
 #include "skymesh.h"
+#include "skypainter.h"
+#include "projections/projector.h"
 
 
-DeepSkyComponent::DeepSkyComponent( SkyComponent *parent )
-        : SkyComponent(parent)
+DeepSkyComponent::DeepSkyComponent( SkyComposite *parent ) :
+    SkyComponent(parent)
 {
-    m_DeepSkyList = QList<DeepSkyObject*>();
-    m_MessierList = QList<DeepSkyObject*>();
-    m_NGCList = QList<DeepSkyObject*>();
-    m_ICList = QList<DeepSkyObject*>();
-    m_OtherList = QList<DeepSkyObject*>();
-
     m_skyMesh = SkyMesh::Instance();
-
     // Add labels
-    for ( int i = 0; i <= MAX_LINENUMBER_MAG; i++ )
+    for( int i = 0; i <= MAX_LINENUMBER_MAG; i++ )
         m_labelList[ i ] = new LabelList;
-
+    loadData();
 }
 
 DeepSkyComponent::~DeepSkyComponent()
@@ -66,7 +60,7 @@ void DeepSkyComponent::update( KSNumbers* )
 {}
 
 
-void DeepSkyComponent::init()
+void DeepSkyComponent::loadData()
 {
     KStarsData* data = KStarsData::Instance();
     //Check whether we need to concatenate a plit NGC/IC catalog
@@ -114,10 +108,10 @@ void DeepSkyComponent::init()
 
         //B magnitude
         ss = line.mid( 23, 4 );
-    if (ss == "    " ) { mag = 99.9f; } else { mag = ss.toFloat(); }
+        if (ss == "    " ) { mag = 99.9f; } else { mag = ss.toFloat(); }
 
         //object type
-        type = line.mid( 29, 1 ).toInt();
+        type = line.mid( 28, 2 ).toInt();
 
         //major and minor axes
         ss = line.mid( 31, 5 );
@@ -155,7 +149,7 @@ void DeepSkyComponent::init()
         r.setH( rah, ram, int(ras) );
         dms d( dd, dm, ds );
 
-    if ( sgn == "-" ) { d.setD( -1.0*d.Degrees() ); }
+        if ( sgn == "-" ) { d.setD( -1.0*d.Degrees() ); }
 
         bool hasName = true;
         QString snum;
@@ -201,6 +195,8 @@ void DeepSkyComponent::init()
         //Assign object to general DeepSkyObjects list,
         //and a secondary list based on its catalog.
         m_DeepSkyList.append( o );
+        appendIndex( o, &m_DeepSkyIndex, trixel );
+
         if ( o->isCatalogM()) {
             m_MessierList.append( o );
             appendIndex( o, &m_MessierIndex, trixel );
@@ -290,7 +286,7 @@ void DeepSkyComponent::appendIndex( DeepSkyObject *o, DeepSkyIndex* dsIndex, Tri
 }
 
 
-void DeepSkyComponent::draw( QPainter& psky )
+void DeepSkyComponent::draw( SkyPainter *skyp )
 {
     if ( ! selected() ) return;
 
@@ -299,42 +295,42 @@ void DeepSkyComponent::draw( QPainter& psky )
     drawFlag = Options::showMessier() &&
                ! ( Options::hideOnSlew() && Options::hideMessier() && SkyMap::IsSlewing() );
 
-    drawDeepSkyCatalog( psky, drawFlag, &m_MessierIndex, "MessColor" );
+    drawDeepSkyCatalog( skyp, drawFlag, &m_MessierIndex, "MessColor", Options::showMessierImages() );
 
     drawFlag = Options::showNGC() &&
                ! ( Options::hideOnSlew() && Options::hideNGC() && SkyMap::IsSlewing() );
 
-    drawDeepSkyCatalog( psky, drawFlag,     &m_NGCIndex,     "NGCColor" );
+    drawDeepSkyCatalog( skyp, drawFlag,     &m_NGCIndex,     "NGCColor" );
 
     drawFlag = Options::showIC() &&
                ! ( Options::hideOnSlew() && Options::hideIC() && SkyMap::IsSlewing() );
 
-    drawDeepSkyCatalog( psky, drawFlag,      &m_ICIndex,      "ICColor" );
+    drawDeepSkyCatalog( skyp, drawFlag,      &m_ICIndex,      "ICColor" );
 
     drawFlag = Options::showOther() &&
                ! ( Options::hideOnSlew() && Options::hideOther() && SkyMap::IsSlewing() );
 
-    drawDeepSkyCatalog( psky, drawFlag,   &m_OtherIndex,   "NGCColor" );
+    drawDeepSkyCatalog( skyp, drawFlag,   &m_OtherIndex,   "NGCColor" );
 }
 
-void DeepSkyComponent::drawDeepSkyCatalog( QPainter& psky, bool drawObject,
-        DeepSkyIndex* dsIndex, const QString& colorString)
+void DeepSkyComponent::drawDeepSkyCatalog( SkyPainter *skyp, bool drawObject,
+                                           DeepSkyIndex* dsIndex, const QString& colorString, bool drawImage)
 {
-    bool drawImage =  Options::showMessierImages();
     if ( ! ( drawObject || drawImage ) ) return;
 
     SkyMap *map = SkyMap::Instance();
+    const Projector *proj = map->projector();
     KStarsData *data = KStarsData::Instance();
 
     UpdateID updateID = data->updateID();
     UpdateID updateNumID = data->updateNumID();
 
-    psky.setPen( data->colorScheme()->colorNamed( colorString ) );
-    psky.setBrush( Qt::NoBrush );
+    skyp->setPen( data->colorScheme()->colorNamed( colorString ) );
+    skyp->setBrush( Qt::NoBrush );
     QColor color        = data->colorScheme()->colorNamed( colorString );
     QColor colorExtra = data->colorScheme()->colorNamed( "HSTColor" );
 
-    m_hideLabels =  ( map->isSlewing() && Options::hideLabels() ) ||
+    m_hideLabels =  ( map->isSlewing() && Options::hideOnSlew() ) ||
                     ! ( Options::showDeepSkyMagnitudes() || Options::showDeepSkyNames() );
 
 
@@ -348,7 +344,7 @@ void DeepSkyComponent::drawDeepSkyCatalog( QPainter& psky, bool drawObject,
         maglim -= (Options::magLimitDrawDeepSky() - Options::magLimitDrawDeepSkyZoomOut() )*(0.75*lgmax - lgz)/(0.75*lgmax - lgmin);
     m_zoomMagLimit = maglim;
 
-    double labelMagLim = Options::deepSkyLabelDensity() / 3.0;
+    double labelMagLim = Options::deepSkyLabelDensity();
     labelMagLim += ( Options::magLimitDrawDeepSky() - labelMagLim ) * ( lgz - lgmin) / (lgmax - lgmin );
     if ( labelMagLim > Options::magLimitDrawDeepSky() ) labelMagLim = Options::magLimitDrawDeepSky();
 
@@ -375,56 +371,20 @@ void DeepSkyComponent::drawDeepSkyCatalog( QPainter& psky, bool drawObject,
                 obj->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
             }
 
-            if ( ! map->checkVisibility( obj ) ) continue;
-
             float mag = obj->mag();
-            float size = map->scale() * obj->a() * dms::PI * Options::zoomFactor() / 10800.0;
+            float size = obj->a() * dms::PI * Options::zoomFactor() / 10800.0;
 
             //only draw objects if flags set, it's bigger than 1 pixel (unless
             //zoom > 2000.), and it's brighter than maglim (unless mag is
             //undefined (=99.9)
             if ( (size > 1.0 || Options::zoomFactor() > 2000.) &&
-                 ( mag < (float)maglim || obj->isCatalogIC() ) ) {
-
-                QPointF o = map->toScreen( obj );
-                if ( ! map->onScreen( o ) ) continue;
-
-                double PositionAngle = map->findPA( obj, o.x(), o.y() );
-
-                //Draw Image
-                if ( drawImage && Options::zoomFactor() > 5.*MINZOOM ) {
-                    obj->drawImage( psky, o.x(), o.y(), PositionAngle, Options::zoomFactor() );
-                }
-
-                //Draw Symbol
-                if ( drawObject ) {
-                    //change color if extra images are available
-                    // most objects don't have those, so we only change colors temporarily
-                    // for the few exceptions. Changing color is expensive!!!
-                    bool bColorChanged = false;
-                    if ( obj->isCatalogM() && obj->ImageList().count() > 1 ) {
-                        psky.setPen( colorExtra );
-                        bColorChanged = true;
-                    } else if ( (!obj->isCatalogM()) && obj->ImageList().count() ) {
-                        psky.setPen( colorExtra );
-                        bColorChanged = true;
-                    }
-
-                    obj->drawSymbol( psky, o.x(), o.y(), PositionAngle, Options::zoomFactor() );
-
-                    if ( bColorChanged ) {
-                        psky.setPen( color );
-                    }
-                }
-
-                if ( !( drawObject || drawImage )  && ( m_hideLabels || mag > labelMagLim ) ) continue;
-            
-                addLabel( o, obj );
-            }
-            else { //Object failed checkVisible(); delete it's Image pointer, if it exists.
-                if ( obj->image() ) {
-                    obj->deleteImage();
-                }
+                 ( mag < (float)maglim || obj->isCatalogIC() ) )
+            {
+    
+                bool drawn = skyp->drawDeepSkyObject(obj, drawImage);
+                if ( drawn  && !( m_hideLabels || mag > labelMagLim ) )
+                    addLabel( proj->toScreen(obj), obj );
+                    //FIXME: find a better way to do above
             }
         }
     }
@@ -438,11 +398,12 @@ void DeepSkyComponent::addLabel( const QPointF& p, DeepSkyObject *obj )
     m_labelList[ idx ]->append( SkyLabel( p, obj ) );
 }
 
-void DeepSkyComponent::drawLabels( QPainter& psky )
+void DeepSkyComponent::drawLabels()
 {
     if ( m_hideLabels ) return;
 
-    psky.setPen( QColor( KStarsData::Instance()->colorScheme()->colorNamed( "DSNameColor" ) ) );
+    SkyLabeler *labeler = SkyLabeler::Instance();
+    labeler->setPen( QColor( KStarsData::Instance()->colorScheme()->colorNamed( "DSNameColor" ) ) );
 
     int max = int( m_zoomMagLimit * 10.0 );
     if ( max < 0 ) max = 0;
@@ -451,7 +412,7 @@ void DeepSkyComponent::drawLabels( QPainter& psky )
     for ( int i = 0; i <= max; i++ ) {
         LabelList* list = m_labelList[ i ];
         for ( int j = 0; j < list->size(); j++ ) {
-            list->at(j).obj->drawNameLabel( psky, list->at(j).o );
+            labeler->drawNameLabel(list->at(j).obj, list->at(j).o);
         }
         list->clear();
     }
@@ -464,6 +425,21 @@ SkyObject* DeepSkyComponent::findByName( const QString &name ) {
     return nameHash[ name.toLower() ];
 }
 
+void DeepSkyComponent::objectsInArea( QList<SkyObject*>& list, const SkyRegion& region ) 
+{
+    for( SkyRegion::const_iterator it = region.constBegin(); it != region.constEnd(); it++ )
+    {
+        Trixel trixel = it.key();
+        if( m_DeepSkyIndex.contains( trixel ) )
+        {
+            DeepSkyList* dsoList = m_DeepSkyIndex.value(trixel);
+            for( DeepSkyList::iterator dsit = dsoList->begin(); dsit != dsoList->end(); dsit++ )
+                list.append( *dsit );
+        }
+    }
+}
+
+
 //we multiply each catalog's smallest angular distance by the
 //following factors before selecting the final nearest object:
 // IC catalog = 0.8
@@ -581,44 +557,17 @@ SkyObject* DeepSkyComponent::objectNearest( SkyPoint *p, double &maxrad ) {
     return oBest;
 }
 
-void DeepSkyComponent::clear() {
-    while ( ! m_MessierList.isEmpty() ) {
-        SkyObject *o = m_MessierList.takeFirst();
-        int i = objectNames(o->type()).indexOf( o->name() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-        i = objectNames(o->type()).indexOf( o->longname() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-
-        delete o;
-    }
-
-    while ( ! m_NGCList.isEmpty() ) {
-        SkyObject *o = m_NGCList.takeFirst();
-        int i = objectNames(o->type()).indexOf( o->name() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-        i = objectNames(o->type()).indexOf( o->longname() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-
-        delete o;
-    }
-
-    while ( ! m_ICList.isEmpty() ) {
-        SkyObject *o = m_ICList.takeFirst();
-        int i = objectNames(o->type()).indexOf( o->name() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-        i = objectNames(o->type()).indexOf( o->longname() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-
-        delete o;
-    }
-
-    while ( ! m_OtherList.isEmpty() ) {
-        SkyObject *o = m_OtherList.takeFirst();
-        int i = objectNames(o->type()).indexOf( o->name() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-        i = objectNames(o->type()).indexOf( o->longname() );
-        if ( i >= 0 ) objectNames(o->type()).removeAt( i );
-
+void DeepSkyComponent::clearList(QList<DeepSkyObject*>& list) {
+    while( !list.isEmpty() ) {
+        SkyObject *o = list.takeFirst();
+        removeFromNames( o );
         delete o;
     }
 }
+
+void DeepSkyComponent::clear() {
+    clearList( m_MessierList );
+    clearList( m_NGCList );
+    clearList( m_ICList );
+    clearList( m_OtherList );
+}
