@@ -17,8 +17,6 @@
 
 #include "ecliptic.h"
 
-#include <QPainter>
-
 #include "ksnumbers.h"
 #include "kstarsdata.h"
 #include "skymap.h"
@@ -26,72 +24,32 @@
 #include "dms.h"
 #include "Options.h"
 #include "linelist.h"
+#include "skylabeler.h"
 
-Ecliptic::Ecliptic(SkyComponent *parent ) :
+#include "skypainter.h"
+
+Ecliptic::Ecliptic(SkyComposite *parent ) :
         LineListIndex( parent, i18n("Ecliptic") ),
-        m_label( LineListIndex::name() )
-{}
-
-
-bool Ecliptic::selected()
-{
-    return Options::showEcliptic();
-}
-
-void Ecliptic::draw( QPainter &psky )
-{
-    if ( ! selected() ) return;
-
-    KStarsData *data = KStarsData::Instance();
-    QColor color( data->colorScheme()->colorNamed( "EclColor" ) );
-    psky.setPen( QPen( QBrush( color ), 1, Qt::SolidLine ) );
-
-    m_label.reset( psky );
-
-    if ( ! skyMesh()->isZoomedIn() ) {
-        drawLines( psky );
-    }
-    else {
-        drawAllLines( psky );
-    }
-    m_label.draw( psky );
-}
-
-
-void Ecliptic::drawLabel( QPainter& psky )
-{
-    if( !selected() )
-        return;
-
-    KStarsData *data = KStarsData::Instance();
-    QColor color( data->colorScheme()->colorNamed( "EclColor" ) );
-    psky.setPen( QPen( QBrush( color ), 1, Qt::SolidLine ) );
-
-    m_label.draw( psky );
-}
-
-
-void Ecliptic::init()
+        m_label( name() )
 {
     KStarsData *data = KStarsData::Instance();
     KSNumbers num( data->ut().djd() );
     dms elat(0.0), elng(0.0);
 
-    double eps    =   0.1;
-    double minRa  =   0.0;
-    double maxRa  =  23.0;
-    double dRa    =   2.0;
-    double dRa2   =  2. / 5.;
-    double ra, ra2;
+    const double eps    =   0.1;
+    const double minRa  =   0.0;
+    const double maxRa  =  23.0;
+    const double dRa    =   2.0;
+    const double dRa2   =  2. / 5.;
 
-    for ( ra = minRa; ra < maxRa; ra += dRa ) {
+    for(double ra = minRa; ra < maxRa; ra += dRa ) {
         LineList* lineList = new LineList();
-        for ( ra2 = ra; ra2 <= ra + dRa + eps; ra2 += dRa2 ) {
+        for(double ra2 = ra; ra2 <= ra + dRa + eps; ra2 += dRa2 ) {
             elng.setH( ra2 );
             SkyPoint* o = new SkyPoint();
             o->setFromEcliptic( num.obliquity(), &elng, &elat );
-            o->setRA0( o->ra()->Hours() );
-            o->setDec0( o->dec()->Degrees() );
+            o->setRA0( o->ra().Hours() );
+            o->setDec0( o->dec().Degrees() );
             o->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
             lineList->append( o );
         }
@@ -99,3 +57,21 @@ void Ecliptic::init()
     }
 }
 
+bool Ecliptic::selected()
+{
+    return Options::showEcliptic();
+}
+
+void Ecliptic::draw( SkyPainter *skyp )
+{
+    if ( ! selected() ) return;
+
+    KStarsData *data = KStarsData::Instance();
+    QColor color( data->colorScheme()->colorNamed( "EclColor" ) );
+    skyp->setPen( QPen( QBrush( color ), 1, Qt::SolidLine ) );
+
+    m_label.reset();
+    drawLines( skyp );
+    SkyLabeler::Instance()->setPen( QPen( QBrush( color ), 1, Qt::SolidLine ) );
+    m_label.draw();
+}
