@@ -43,7 +43,9 @@ class MilkyWay;
 class SolarSystemComposite;
 class StarComponent;
 class DeepStarComponent;
-class SatelliteComposite;
+//class SatelliteComposite;
+class TargetListComponent;
+class TargetListComponent;
 
 class DeepSkyObject;
 class KSPlanetBase;
@@ -108,7 +110,7 @@ public:
     	*@short Delegate draw requests to all sub components
     	*@p psky Reference to the QPainter on which to paint
     	*/
-    virtual void draw( QPainter& psky );
+    virtual void draw( SkyPainter *skyp );
 
     /**
       *@return the object nearest a given point in the sky.
@@ -205,8 +207,9 @@ private:
     SolarSystemComposite        *m_SolarSystem;
     SkyComposite                *m_CustomCatalogs;
     StarComponent               *m_Stars;
-    SatelliteComposite          *m_Satellites;
+    //SatelliteComposite          *m_Satellites;
     FlagComponent               *m_Flags;
+    TargetListComponent         *m_ObservingList;
 
     SkyMesh*                m_skyMesh;
     SkyLabeler*             m_skyLabeler;
