@@ -17,9 +17,6 @@
 
 //This file contains Event handlers for the SkyMap class.
 
-#include <stdlib.h>
-#include <math.h> //using fabs()
-
 #include <QCursor>
 #include <QPainter>
 #include <QMouseEvent>
@@ -29,25 +26,30 @@
 #include <QKeySequence>
 #include <QPaintEvent>
 
-#include <kiconloader.h>
 #include <kstatusbar.h>
-#include <kurl.h>
 #include <kio/job.h>
 
 #include "skymap.h"
+#include "skyqpainter.h"
+#include "skyglpainter.h"
 #include "Options.h"
 #include "kstars.h"
 #include "kstarsdata.h"
 #include "ksutils.h"
 #include "simclock.h"
-#include "infoboxes.h"
 #include "kspopupmenu.h"
-#include "skyobjects/ksmoon.h"
+#include "skyobjects/ksplanetbase.h"
+#include "widgets/infoboxwidget.h"
+
+#include "projections/projector.h"
+
+#include "skycomponents/skymapcomposite.h"
 
 // TODO: Remove if debug key binding is removed
 #include "skycomponents/skylabeler.h"
 #include "skycomponents/starcomponent.h"
 
+
 void SkyMap::resizeEvent( QResizeEvent * )
 {
     computeSkymap = true; // skymap must be new computed
@@ -55,15 +57,17 @@ void SkyMap::resizeEvent( QResizeEvent * )
     //FIXME: No equivalent for this line in Qt4 ??
     //	if ( testWState( Qt::WState_AutoMask ) ) updateMask();
 
-    // avoid phantom positions of infoboxes
-    if ( ks && ( isVisible() || width() == ks->width() || height() == ks->height() ) ) {
-        infoBoxes()->resize( width(), height() );
-    }
-    *sky = sky->scaled( width(), height() );
-    *sky2 = sky2->scaled( width(), height() );
+    // Resize the widget that draws the sky map.
+    // FIXME: The resize event doesn't pass to children. Any better way of doing this?
+    m_SkyMapDraw->resize( size() );
+
+    // Resize infoboxes container.
+    // FIXME: this is not really pretty. Maybe there are some better way to this???
+    m_iboxes->resize( size() );
 }
 
 void SkyMap::keyPressEvent( QKeyEvent *e ) {
+    KStars* kstars = KStars::Instance();
     QString s;
     bool arrowKeyPressed( false );
     bool shiftPressed( false );
@@ -83,11 +87,11 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
     switch ( e->key() ) {
     case Qt::Key_Left :
         if ( Options::useAltAz() ) {
-            focus()->setAz( dms( focus()->az()->Degrees() - step * MINZOOM/Options::zoomFactor() ).reduce() );
-            focus()->HorizontalToEquatorial( data->LST, data->geo()->lat() );
+            focus()->setAz( dms( focus()->az().Degrees() - step * MINZOOM/Options::zoomFactor() ).reduce() );
+            focus()->HorizontalToEquatorial( data->lst(), data->geo()->lat() );
         } else {
-            focus()->setRA( focus()->ra()->Hours() + 0.05*step * MINZOOM/Options::zoomFactor() );
-            focus()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+            focus()->setRA( focus()->ra().Hours() + 0.05*step * MINZOOM/Options::zoomFactor() );
+            focus()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         }
 
         arrowKeyPressed = true;
@@ -97,11 +101,11 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
 
     case Qt::Key_Right :
         if ( Options::useAltAz() ) {
-            focus()->setAz( dms( focus()->az()->Degrees() + step * MINZOOM/Options::zoomFactor() ).reduce() );
-            focus()->HorizontalToEquatorial( data->LST, data->geo()->lat() );
+            focus()->setAz( dms( focus()->az().Degrees() + step * MINZOOM/Options::zoomFactor() ).reduce() );
+            focus()->HorizontalToEquatorial( data->lst(), data->geo()->lat() );
         } else {
-            focus()->setRA( focus()->ra()->Hours() - 0.05*step * MINZOOM/Options::zoomFactor() );
-            focus()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+            focus()->setRA( focus()->ra().Hours() - 0.05*step * MINZOOM/Options::zoomFactor() );
+            focus()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         }
 
         arrowKeyPressed = true;
@@ -111,13 +115,13 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
 
     case Qt::Key_Up :
         if ( Options::useAltAz() ) {
-            focus()->setAlt( focus()->alt()->Degrees() + step * MINZOOM/Options::zoomFactor() );
-            if ( focus()->alt()->Degrees() > 90.0 ) focus()->setAlt( 90.0 );
-            focus()->HorizontalToEquatorial( data->LST, data->geo()->lat() );
+            focus()->setAlt( focus()->alt().Degrees() + step * MINZOOM/Options::zoomFactor() );
+            if ( focus()->alt().Degrees() > 90.0 ) focus()->setAlt( 90.0 );
+            focus()->HorizontalToEquatorial( data->lst(), data->geo()->lat() );
         } else {
-            focus()->setDec( focus()->dec()->Degrees() + step * MINZOOM/Options::zoomFactor() );
-            if (focus()->dec()->Degrees() > 90.0) focus()->setDec( 90.0 );
-            focus()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+            focus()->setDec( focus()->dec().Degrees() + step * MINZOOM/Options::zoomFactor() );
+            if (focus()->dec().Degrees() > 90.0) focus()->setDec( 90.0 );
+            focus()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         }
 
         arrowKeyPressed = true;
@@ -127,13 +131,13 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
 
     case Qt::Key_Down:
         if ( Options::useAltAz() ) {
-            focus()->setAlt( focus()->alt()->Degrees() - step * MINZOOM/Options::zoomFactor() );
-            if ( focus()->alt()->Degrees() < -90.0 ) focus()->setAlt( -90.0 );
-            focus()->HorizontalToEquatorial(data->LST, data->geo()->lat() );
+            focus()->setAlt( focus()->alt().Degrees() - step * MINZOOM/Options::zoomFactor() );
+            if ( focus()->alt().Degrees() < -90.0 ) focus()->setAlt( -90.0 );
+            focus()->HorizontalToEquatorial(data->lst(), data->geo()->lat() );
         } else {
-            focus()->setDec( focus()->dec()->Degrees() - step * MINZOOM/Options::zoomFactor() );
-            if (focus()->dec()->Degrees() < -90.0) focus()->setDec( -90.0 );
-            focus()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+            focus()->setDec( focus()->dec().Degrees() - step * MINZOOM/Options::zoomFactor() );
+            if (focus()->dec().Degrees() < -90.0) focus()->setDec( -90.0 );
+            focus()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         }
 
         arrowKeyPressed = true;
@@ -143,14 +147,12 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
 
     case Qt::Key_Plus:   //Zoom in
     case Qt::Key_Equal:
-        if ( ks ) 
-            zoomInOrMagStep( e->modifiers() );
+        zoomInOrMagStep( e->modifiers() );
         break;
 
     case Qt::Key_Minus:  //Zoom out
     case Qt::Key_Underscore:
-        if (  ks ) 
-            zoomOutOrMagStep( e->modifiers() );
+        zoomOutOrMagStep( e->modifiers() );
         break;
 
     case Qt::Key_0: //center on Sun
@@ -214,31 +216,20 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
         break;
 
     case Qt::Key_BracketLeft:   // Begin measuring angular distance
-        if ( !isAngleMode() ) {
+        if( !angularDistanceMode )
             slotBeginAngularDistance();
-        }
-
         break;
-
-    case Qt::Key_BracketRight:  // End measuring angular distance
-        if ( isAngleMode() ) {
-            slotEndAngularDistance();
-        }
-
-        break;
-
     case Qt::Key_Escape:        // Cancel angular distance measurement
-        if  (isAngleMode() ) {
+        if( angularDistanceMode )
             slotCancelAngularDistance();
-        }
         break;
-
     case Qt::Key_Comma:  //advance one step backward in time
     case Qt::Key_Less:
-        if ( data->clock()->isActive() ) data->clock()->stop();
-        data->clock()->setScale( -1.0 * data->clock()->scale() ); //temporarily need negative time step
+        if ( data->clock()->isActive() )
+            data->clock()->stop();
+        data->clock()->setClockScale( -1.0 * data->clock()->scale() ); //temporarily need negative time step
         data->clock()->manualTick( true );
-        data->clock()->setScale( -1.0 * data->clock()->scale() ); //reset original sign of time step
+        data->clock()->setClockScale( -1.0 * data->clock()->scale() ); //reset original sign of time step
         update();
         qApp->processEvents();
         break;
@@ -292,7 +283,7 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
         }
 
         if ( clickedObject() ) {
-            ks->observingList()->slotAddObject();
+            kstars->observingList()->slotAddObject();
         }
 
         if ( orig ) {
@@ -330,8 +321,9 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
             setClickedObject( focusObject() );
         }
 
-        if ( clickedObject() && clickedObject()->isSolarSystem() ) {
-            if ( ((KSPlanetBase*)clickedObject())->hasTrail() )
+        KSPlanetBase* planet = dynamic_cast<KSPlanetBase*>( clickedObject() );
+        if( planet ) {
+            if( planet->hasTrail() )
                 slotRemovePlanetTrail();
             else
                 slotAddPlanetTrail();
@@ -343,101 +335,26 @@ void SkyMap::keyPressEvent( QKeyEvent *e ) {
         break;
     }
 
-    //DEBUG_REFRACT
-    case Qt::Key_Q:
+    case Qt::Key_R:
         {
-            for ( double alt=-0.5; alt<30.5; alt+=1.0 ) {
-                dms a( alt );
-                dms b( refract( &a, true ) ); //find apparent alt from true alt
-                dms c( refract( &b, false ) );
-
-                kDebug() << a.toDMSString() << b.toDMSString() << c.toDMSString();
-            }
+            // Toggle relativistic corrections
+            Options::setUseRelativistic( ! Options::useRelativistic() );
+            kDebug() << "Relativistc corrections: " << Options::useRelativistic();
+            forceUpdate();
             break;
         }
-
-        //TIMING
-        // *** Uncomment and insert timing test code here ***
-    case Qt::Key_X:
-        {
-            QTime t;
-            t.start();
-            foreach ( SkyObject *star, data->skyComposite()->stars() )
-            toScreen( star );
-            kDebug() << QString("toScreen() for all stars took %1 ms").arg(t.elapsed());
-            t.start();
-            foreach ( SkyObject *star, data->skyComposite()->stars() )
-            toScreenQuaternion( star );
-            kDebug() << QString("toScreenQuaternion() for all stars took %1 ms").arg(t.elapsed());
-            break;
-        }
-        //END_TIMING
-
     case Qt::Key_A:
         Options::setUseAntialias( ! Options::useAntialias() );
         kDebug() << "Use Antialiasing: " << Options::useAntialias();
         forceUpdate();
         break;
-
-        //Test code: create a SkyLine
-        /**
-        case Qt::Key_V:
-        kDebug() << "Create a skyline: ";
-        SkyLine sl( SkyPoint( 12.34, 33.50 ), SkyPoint( 14.00, 40.00 ) );
-        kDebug() << "  " << sl.points().size() << " :: "
-        << sl.point(0)->ra()->toHMSString() << " : " 
-        << sl.point(1)->ra()->toHMSString() << endl;
-
-        SkyPoint p( 11.75, 30.25 );
-        sl.setPoint( 1, &p );
-        kDebug() << "  " << sl.points().size() << " :: "
-        << sl.point(0)->ra()->toHMSString() << " : " 
-        << sl.point(1)->ra()->toHMSString() << endl;
-            break;
-        **/
-        /*
-    case Qt::Key_B:    // print useful debug info about memory allocation for stars
-        data->skyComposite()->getStarComponent()->printDebugInfo();
-        break;
-    case Qt::Key_F:    // verify the integrity of StarBlockLists
-        data->skyComposite()->getStarComponent()->verifySBLIntegrity();
-        break;
-    case Qt::Key_G:    // print Cache structure
-        StarComponent::m_StarBlockFactory.printStructure();
-        break;
-    case Qt::Key_H: {  // Frugal memory mode
-        StarComponent::veryFrugalMem = false;
-        if( StarComponent::frugalMem = !(StarComponent::frugalMem) )
-            kDebug() << "Switched to frugal memory mode";
-        else
-            kDebug() << "Switched to generous memory mode";
-        break;
+    default:
+        // We don't want to do anything in this case. Key is unknown
+        return;
     }
-    case Qt::Key_I: {  // Very frugal memory mode
-        StarComponent::frugalMem = false;
-        if( StarComponent::veryFrugalMem = !(StarComponent::veryFrugalMem) )
-            kDebug() << "Switched to very frugal memory mode";
-        else
-            kDebug() << "Switched to generous memory mode";
-        
-        break;
-    }
-        */
-
-    }
-
-    setOldFocus( focus() );
-    oldfocus()->setAz( focus()->az()->Degrees() );
-    oldfocus()->setAlt( focus()->alt()->Degrees() );
-
-    double dHA = data->LST->Hours() - focus()->ra()->Hours();
-    while ( dHA < 0.0 ) dHA += 24.0;
-    data->HourAngle->setH( dHA );
 
     if ( arrowKeyPressed ) {
-        infoBoxes()->focusObjChanged( i18n( "nothing" ) );
         stopTracking();
-
         if ( scrollCount > 10 ) {
             setDestination( focus() );
             scrollCount = 0;
@@ -460,7 +377,10 @@ void SkyMap::slotJobResult( KJob *job ) {
 }
 
 void SkyMap::stopTracking() {
-    if ( ks && Options::isTracking() ) ks->slotTrack();
+    KStars* kstars = KStars::Instance();
+    emit positionChanged( focus() );
+    if( kstars && Options::isTracking() )
+        kstars->slotTrack();
 }
 
 void SkyMap::keyReleaseEvent( QKeyEvent *e ) {
@@ -478,11 +398,11 @@ void SkyMap::keyReleaseEvent( QKeyEvent *e ) {
         scrollCount = 0;
 
         if ( Options::useAltAz() )
-            setDestinationAltAz( focus()->alt()->Degrees(), focus()->az()->Degrees() );
+            setDestinationAltAz( focus()->alt(), focus()->az() );
         else
             setDestination( focus() );
 
-        showFocusCoords( true );
+        showFocusCoords();
         forceUpdate();  // Need a full update to draw faint objects that are not drawn while slewing.
         break;
     }
@@ -502,12 +422,6 @@ void SkyMap::mouseMoveEvent( QMouseEvent *e ) {
         HoverTimer.start( HOVER_INTERVAL );
     }
 
-    //Are we dragging an infoBox?
-    if ( infoBoxes()->dragBox( e ) ) {
-        update();
-        return;
-    }
-
     //Are we defining a ZoomRect?
     if ( ZoomRect.center().x() > 0 && ZoomRect.center().y() > 0 ) {
         //cancel operation if the user let go of CTRL
@@ -535,106 +449,74 @@ void SkyMap::mouseMoveEvent( QMouseEvent *e ) {
         }
     }
 
-    if ( unusablePoint( e->pos() ) ) return;  // break if point is unusable
+    if ( projector()->unusablePoint( e->pos() ) ) return;  // break if point is unusable
 
     //determine RA, Dec of mouse pointer
-    setMousePoint( fromScreen( e->pos(), data->LST, data->geo()->lat() ) );
-    mousePoint()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+    setMousePoint( projector()->fromScreen( e->pos(), data->lst(), data->geo()->lat() ) );
+    mousePoint()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
 
     double dyPix = 0.5*height() - e->y();
     if ( midMouseButtonDown ) { //zoom according to y-offset
         float yoff = dyPix - y0;
-
         if (yoff > 10 ) {
             y0 = dyPix;
-            if ( ks ) ks->slotZoomIn();
+            slotZoomIn();
         }
         if (yoff < -10 ) {
             y0 = dyPix;
-            if ( ks ) ks->slotZoomOut();
+            slotZoomOut();
         }
     }
 
     if ( mouseButtonDown ) {
         // set the mouseMoveCursor and set slewing=true, if they are not set yet
-        if (!mouseMoveCursor) setMouseMoveCursor();
-        if (!slewing) {
+        if( !mouseMoveCursor )
+            setMouseMoveCursor();
+        if( !slewing ) {
             slewing = true;
-            infoBoxes()->focusObjChanged( i18n( "nothing" ) );
             stopTracking(); //toggle tracking off
         }
 
         //Update focus such that the sky coords at mouse cursor remain approximately constant
         if ( Options::useAltAz() ) {
-            mousePoint()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
-            clickedPoint()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
-            dms dAz = mousePoint()->az()->Degrees() - clickedPoint()->az()->Degrees();
-            dms dAlt = mousePoint()->alt()->Degrees() - clickedPoint()->alt()->Degrees();
-            focus()->setAz( focus()->az()->Degrees() - dAz.Degrees() ); //move focus in opposite direction
-            focus()->setAlt( focus()->alt()->Degrees() - dAlt.Degrees() );
-
-            if ( focus()->alt()->Degrees() >90.0 ) focus()->setAlt( 90.0 );
-            if ( focus()->alt()->Degrees() <-90.0 ) focus()->setAlt( -90.0 );
-            focus()->setAz( focus()->az()->reduce() );
-            focus()->HorizontalToEquatorial( data->LST, data->geo()->lat() );
+            mousePoint()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
+            clickedPoint()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
+            dms dAz  = mousePoint()->az()  - clickedPoint()->az();
+            dms dAlt = mousePoint()->alt() - clickedPoint()->alt();
+            focus()->setAz( focus()->az().Degrees() - dAz.Degrees() ); //move focus in opposite direction
+            focus()->setAz( focus()->az().reduce() );
+            focus()->setAlt(
+                KSUtils::clamp( focus()->alt().Degrees() - dAlt.Degrees() , -90.0 , 90.0 ) );
+            focus()->HorizontalToEquatorial( data->lst(), data->geo()->lat() );
         } else {
-            dms dRA = mousePoint()->ra()->Degrees() - clickedPoint()->ra()->Degrees();
-            dms dDec = mousePoint()->dec()->Degrees() - clickedPoint()->dec()->Degrees();
-            focus()->setRA( focus()->ra()->Hours() - dRA.Hours() ); //move focus in opposite direction
-            focus()->setDec( focus()->dec()->Degrees() - dDec.Degrees() );
-
-            if ( focus()->dec()->Degrees() >90.0 ) focus()->setDec( 90.0 );
-            if ( focus()->dec()->Degrees() <-90.0 ) focus()->setDec( -90.0 );
-            focus()->setRA( focus()->ra()->reduce() );
-            focus()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+            dms dRA  = mousePoint()->ra()  - clickedPoint()->ra();
+            dms dDec = mousePoint()->dec() - clickedPoint()->dec();
+            focus()->setRA( focus()->ra().Hours() - dRA.Hours() ); //move focus in opposite direction
+            focus()->setRA( focus()->ra().reduce() );
+            focus()->setDec(
+                KSUtils::clamp( focus()->dec().Degrees() - dDec.Degrees() , -90.0 , 90.0 ) );
+            focus()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         }
 
         ++scrollCount;
         if ( scrollCount > 4 ) {
-            showFocusCoords( true );
+            showFocusCoords();
             scrollCount = 0;
         }
 
-        setOldFocus( focus() );
-
-        double dHA = data->LST->Hours() - focus()->ra()->Hours();
-        while ( dHA < 0.0 ) dHA += 24.0;
-        data->HourAngle->setH( dHA );
-
         //redetermine RA, Dec of mouse pointer, using new focus
-        setMousePoint( fromScreen( e->pos(), data->LST, data->geo()->lat() ) );
-        mousePoint()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+        setMousePoint( projector()->fromScreen( e->pos(), data->lst(), data->geo()->lat() ) );
+        mousePoint()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         setClickedPoint( mousePoint() );
 
         forceUpdate();  // must be new computed
 
     } else { //mouse button not down
-        if ( ks ) {
-            QString sX, sY, s;
-
-            if ( Options::showAltAzField() ) {
-                sX = mousePoint()->az()->toDMSString(true);  //true: force +/- symbol
-                dms a( mousePoint()->alt()->Degrees() );
-                if ( Options::useAltAz() && Options::useRefraction() )
-                    a = refract( mousePoint()->alt(), true ); //true: compute apparent alt from true alt
-                sY = a.toDMSString(true); //true: force +/- symbol
-    
-                s = sX + ",  " + sY;
-                ks->statusBar()->changeItem( s, 1 );
-            }
-
-            if ( Options::showRADecField() ) {
-                sX = mousePoint()->ra()->toHMSString();
-                sY = mousePoint()->dec()->toDMSString(true); //true: force +/- symbol
-                s = sX + ",  " + sY;
-                ks->statusBar()->changeItem( s, 2 );
-            }
-        }
+        emit mousePointChanged( mousePoint() );
     }
 }
 
 void SkyMap::wheelEvent( QWheelEvent *e ) {
-    if ( ! ks ) return;
     if ( e->delta() > 0 ) 
         zoomInOrMagStep ( e->modifiers() );
     else
@@ -642,28 +524,22 @@ void SkyMap::wheelEvent( QWheelEvent *e ) {
 }
 
 void SkyMap::mouseReleaseEvent( QMouseEvent * ) {
-    if ( infoBoxes()->unGrabBox() ) {
-        update();
-        return;
-    }
-
     if ( ZoomRect.isValid() ) {
         //Zoom in on center of Zoom Circle, by a factor equal to the ratio
         //of the sky pixmap's width to the Zoom Circle's diameter
         float factor = float(width()) / float(ZoomRect.width());
 
-        infoBoxes()->focusObjChanged( i18n( "nothing" ) );
         stopTracking();
 
-        SkyPoint newcenter = fromScreen( ZoomRect.center(), data->LST, data->geo()->lat() );
+        SkyPoint newcenter = projector()->fromScreen( ZoomRect.center(), data->lst(), data->geo()->lat() );
 
         setFocus( &newcenter );
         setDestination( &newcenter );
-        ks->zoom( Options::zoomFactor() * factor );
-
         setDefaultMouseCursor();
+
+        setZoomFactor( Options::zoomFactor() * factor );
+
         ZoomRect = QRect(); //invalidate ZoomRect
-        forceUpdate();
     } else {
         setDefaultMouseCursor();
         ZoomRect = QRect(); //just in case user Ctrl+clicked + released w/o dragging...
@@ -676,12 +552,10 @@ void SkyMap::mouseReleaseEvent( QMouseEvent * ) {
             slewing = false;
 
             if ( Options::useAltAz() )
-                setDestinationAltAz( focus()->alt()->Degrees(), focus()->az()->Degrees() );
+                setDestinationAltAz( focus()->alt(), focus()->az() );
             else
                 setDestination( focus() );
         }
-
-        setOldFocus( focus() );
         forceUpdate();	// is needed because after moving the sky not all stars are shown
     }
     if ( midMouseButtonDown ) midMouseButtonDown = false;  // if middle button was pressed unset here
@@ -690,11 +564,7 @@ void SkyMap::mouseReleaseEvent( QMouseEvent * ) {
 }
 
 void SkyMap::mousePressEvent( QMouseEvent *e ) {
-    //did we Grab an infoBox?
-    if ( e->button() == Qt::LeftButton && infoBoxes()->grabBox( e ) ) {
-        update(); //refresh without redrawing skymap
-        return;
-    }
+    KStars* kstars = KStars::Instance();
 
     if ( ( e->modifiers() & Qt::ControlModifier ) && (e->button() == Qt::LeftButton) ) {
         ZoomRect.moveCenter( e->pos() );
@@ -704,10 +574,11 @@ void SkyMap::mousePressEvent( QMouseEvent *e ) {
     }
 
     // if button is down and cursor is not moved set the move cursor after 500 ms
-    QTimer t;
-    t.singleShot (500, this, SLOT (setMouseMoveCursor()));
+    QTimer::singleShot(500, this, SLOT (setMouseMoveCursor()));
 
-    if ( unusablePoint( e->pos() ) ) return;  // break if point is unusable
+    // break if point is unusable
+    if ( projector()->unusablePoint( e->pos() ) )
+        return;
 
     if ( !midMouseButtonDown && e->button() == Qt::MidButton ) {
         y0 = 0.5*height() - e->y();  //record y pixel coordinate for middle-button zooming
@@ -721,155 +592,60 @@ void SkyMap::mousePressEvent( QMouseEvent *e ) {
         }
 
         //determine RA, Dec of mouse pointer
-        setMousePoint( fromScreen( e->pos(), data->LST, data->geo()->lat() ) );
-        mousePoint()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+        setMousePoint( projector()->fromScreen( e->pos(), data->lst(), data->geo()->lat() ) );
+        mousePoint()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
         setClickedPoint( mousePoint() );
-        clickedPoint()->EquatorialToHorizontal( data->LST, data->geo()->lat() );
+        clickedPoint()->EquatorialToHorizontal( data->lst(), data->geo()->lat() );
 
         //Find object nearest to clickedPoint()
         double maxrad = 1000.0/Options::zoomFactor();
-        setClickedObject( data->skyComposite()->objectNearest( clickedPoint(), maxrad ) );
+        SkyObject* obj = data->skyComposite()->objectNearest( clickedPoint(), maxrad );
+        setClickedObject( obj );
+        if( obj )
+            setClickedPoint(  obj );
 
-        if ( clickedObject() ) {
-            setClickedPoint( clickedObject() );
-
-            if ( e->button() == Qt::RightButton ) {
-                clickedObject()->showPopupMenu( pmenu, QCursor::pos() );
+        switch( e->button() ) {
+        case Qt::LeftButton:
+            if( kstars ) {
+                QString name;
+                if( clickedObject() )
+                    name = clickedObject()->translatedLongName();
+                else
+                    name = i18n( "Empty sky" );
+                kstars->statusBar()->changeItem(name, 0 );
             }
-
-            if ( ks && e->button() == Qt::LeftButton ) {
-                ks->statusBar()->changeItem( clickedObject()->translatedLongName(), 0 );
-            }
-        } else {
-            //Empty sky selected.  If left-click, display "nothing" in the status bar.
-            //If right-click, open "empty" popup menu.
-            setClickedObject( NULL );
-
-            switch (e->button()) {
-            case Qt::LeftButton:
-                if ( ks ) ks->statusBar()->changeItem( i18n( "Empty sky" ), 0 );
-                break;
-            case Qt::RightButton:
-                {
-                    SkyObject *nullObj = new SkyObject( SkyObject::TYPE_UNKNOWN, clickedPoint()->ra()->Hours(), clickedPoint()->dec()->Degrees() );
-                    pmenu->createEmptyMenu( nullObj );
-                    delete nullObj;
-
-                    pmenu->popup(  QCursor::pos() );
-                    break;
+            break;
+        case Qt::RightButton:
+            if( angularDistanceMode ) {
+                // Compute angular distance.
+                slotEndAngularDistance();
+            } else {
+                // Show popup menu
+                if( clickedObject() ) {
+                    clickedObject()->showPopupMenu( pmenu, QCursor::pos() );
+                } else {
+                    pmenu->createEmptyMenu( clickedPoint() );
+                    pmenu->popup( QCursor::pos() );
                 }
-
-            default:
-                break;
             }
+            break;
+        default: ;
         }
     }
 }
 
 void SkyMap::mouseDoubleClickEvent( QMouseEvent *e ) {
-    //Was the event inside an infoBox?  If so, shade the box.
-    if ( e->button() == Qt::LeftButton ) {
-        if ( infoBoxes()->shadeBox( e ) ) {
-            update();
-            return;
-        }
-
-        if ( unusablePoint( e->pos() ) ) return;  // break if point is unusable
-
-        if (mouseButtonDown ) mouseButtonDown = false;
-        if ( e->x() != width()/2 || e->y() != height()/2 ) slotCenter();
+    if ( e->button() == Qt::LeftButton && !projector()->unusablePoint( e->pos() ) ) {
+        mouseButtonDown = false;
+        if( e->x() != width()/2 || e->y() != height()/2 )
+            slotCenter();
     }
 }
 
-void SkyMap::paintEvent( QPaintEvent * )
-{
-    //If computeSkymap is false, then we just refresh the window using the stored sky pixmap
-    //and draw the "overlays" on top.  This lets us update the overlay information rapidly
-    //without needing to recompute the entire skymap.
-    //use update() to trigger this "short" paint event; to force a full "recompute"
-    //of the skymap, use forceUpdate().
-    if (!computeSkymap)
-    {
-        *sky2 = *sky;
-        drawOverlays( sky2 );
-        QPainter p;
-        p.begin( this );
-        p.drawImage( 0, 0, sky2->toImage() );
-        p.end();
-        return ; // exit because the pixmap is repainted and that's all what we want
-    }
-
-    //TIMING
-    // 	QTime t;
-    // 	t.start();
-
-    QPainter psky;
-    setMapGeometry();
-
-    //FIXME: What to do about the visibility logic?
-    // 	//checkSlewing combines the slewing flag (which is true when the display is actually in motion),
-    // 	//the hideOnSlew option (which is true if slewing should hide objects),
-    // 	//and clockSlewing (which is true if the timescale exceeds Options::slewTimeScale)
-    // 	bool checkSlewing = ( ( slewing || ( clockSlewing && data->clock()->isActive() ) )
-    // 				&& Options::hideOnSlew() );
-    //
-    // 	//shortcuts to inform whether to draw different objects
-    // 	bool drawPlanets( Options::showSolarSystem() && !(checkSlewing && Options::hidePlanets() ) );
-    // 	bool drawMW( Options::showMilkyWay() && !(checkSlewing && Options::hideMilkyWay() ) );
-    // 	bool drawCNames( Options::showCNames() && !(checkSlewing && Options::hideCNames() ) );
-    // 	bool drawCLines( Options::showCLines() &&!(checkSlewing && Options::hideCLines() ) );
-    // 	bool drawCBounds( Options::showCBounds() &&!(checkSlewing && Options::hideCBounds() ) );
-    // 	bool drawGrid( Options::showGrid() && !(checkSlewing && Options::hideGrid() ) );
-
-    psky.begin( sky );
-    psky.setRenderHint(QPainter::Antialiasing, (!slewing && Options::useAntialias()) );
-
-    psky.fillRect( 0, 0, width(), height(), QBrush( data->colorScheme()->colorNamed( "SkyColor" ) ) );
-
-    //TIMING
-//     QTime t2;
-//     t2.start();
-
-    //Draw all sky elements
-    data->skyComposite()->draw( psky );
-
-    //TIMING
-//     kDebug() << QString("SkyMapComposite::draw() took %1 ms").arg(t2.elapsed());
-
-    //Finish up
-    psky.end();
-
-    //TIMING
-    //	t2.start();
-
-    *sky2 = *sky;
-    drawOverlays( sky2 );
-
-    //TIMING
-    //	kDebug() << QString("drawOverlays() took %1 ms").arg(t2.elapsed());
-
-    //TIMING
-    //	t2.start();
-
-    QPainter psky2;
-    psky2.begin( this );
-    psky2.drawPixmap( 0, 0, *sky2 );
-    psky2.end();
-
-    //TIMING
-    //	kDebug() << QString("drawImage() took %1 ms").arg(t2.elapsed());
-
-    computeSkymap = false;	// use forceUpdate() to compute new skymap else old pixmap will be shown
-
-    //TIMING
-    //	kDebug() << QString("Skymap draw took %1 ms").arg(t.elapsed());
-}
-
 double SkyMap::zoomFactor( const int modifier ) {
     double factor = ( modifier & Qt::ControlModifier) ? DZOOM : 2.0; 
     if ( modifier & Qt::ShiftModifier ) 
         factor = sqrt( factor );
-
     return factor;
 }
 
@@ -877,7 +653,7 @@ void SkyMap::zoomInOrMagStep( const int modifier ) {
     if ( modifier & Qt::AltModifier )
         incMagLimit( modifier );
     else
-        ks->zoomIn( zoomFactor( modifier ) );
+        setZoomFactor( Options::zoomFactor() * zoomFactor( modifier ) );
 }
 
     
@@ -885,27 +661,18 @@ void SkyMap::zoomOutOrMagStep( const int modifier ) {
     if ( modifier & Qt::AltModifier )
         decMagLimit( modifier );
     else
-        ks->zoomOut( zoomFactor (modifier ) );
-}
-
-void SkyMap::zoomIn( const int modifier) {
-    ks->zoomIn( zoomFactor( modifier) );
-}
-
-void SkyMap::zoomOut( const int modifier) {
-    ks->zoomOut( zoomFactor( modifier ) );
+        setZoomFactor( Options::zoomFactor() / zoomFactor (modifier ) );
 }
 
 double SkyMap::magFactor( const int modifier ) {
     double factor = ( modifier & Qt::ControlModifier) ? 0.2 : 1.0; 
     if ( modifier & Qt::ShiftModifier ) 
         factor /= 2.0;
-
     return factor;
 }
 
 void SkyMap::incMagLimit( const int modifier ) {
-    double limit = 2.222 * log10( (float)Options::starDensity() ) + 0.35;
+    double limit = 2.222 * log10(static_cast<double>( Options::starDensity() )) + 0.35;
     limit += magFactor( modifier );
     if ( limit > 7.94 ) limit = 7.94;
     Options::setStarDensity( pow( 10, ( limit - 0.35 ) / 2.222) );
@@ -914,7 +681,7 @@ void SkyMap::incMagLimit( const int modifier ) {
 }
 
 void SkyMap::decMagLimit( const int modifier ) {
-    double limit = 2.222 * log10( (float)Options::starDensity() ) + 0.35;
+    double limit = 2.222 * log10(static_cast<double>( Options::starDensity() )) + 0.35;
     limit -= magFactor( modifier );
     if ( limit < 3.55 ) limit = 3.55;
     Options::setStarDensity( pow( 10, ( limit - 0.35 ) / 2.222) );
