@@ -149,8 +149,8 @@ bool KSComet::findGeocentricPosition( const KSNumbers *num, const KSPlanetBase *
 
     //xe, ye, ze are the Earth's heliocentric cartesian coords
     double cosBe, sinBe, cosLe, sinLe;
-    Earth->ecLong()->SinCos( sinLe, cosLe );
-    Earth->ecLat()->SinCos( sinBe, cosBe );
+    Earth->ecLong().SinCos( sinLe, cosLe );
+    Earth->ecLat().SinCos( sinBe, cosBe );
 
     double xe = Earth->rsun() * cosBe * cosLe;
     double ye = Earth->rsun() * cosBe * sinLe;
