@@ -29,12 +29,11 @@
 #include "kssun.h"
 #include "kstarsdata.h"
 #include "Options.h"
+#include "skycomponents/skymapcomposite.h"
 
 void SkyPoint::set( const dms& r, const dms& d ) {
-    RA0.set( r );
-    Dec0.set( d );
-    RA.set( r );
-    Dec.set( d );
+    RA  = RA0  = r;
+    Dec = Dec0 = d;
     //Quaternion
     syncQuaternion();
 }
@@ -62,10 +61,10 @@ void SkyPoint::EquatorialToHorizontal( const dms *LST, const dms *lat ) {
     double sindec, cosdec, sinlat, coslat, sinHA, cosHA;
     double sinAlt, cosAlt;
 
-    dms HourAngle = LST->Degrees() - ra()->Degrees();
+    dms HourAngle = (*LST) - ra();
 
     lat->SinCos( sinlat, coslat );
-    dec()->SinCos( sindec, cosdec );
+    dec().SinCos( sindec, cosdec );
     HourAngle.SinCos( sinHA, cosHA );
 
     sinAlt = sindec*sinlat + cosdec*coslat*cosHA;
@@ -109,7 +108,7 @@ void SkyPoint::HorizontalToEquatorial( const dms *LST, const dms *lat ) {
     double sinDec, cosDec;
 
     lat->SinCos( sinlat, coslat );
-    alt()->SinCos( sinAlt, cosAlt );
+    alt().SinCos( sinAlt, cosAlt );
     Az.SinCos( sinAz,  cosAz );
 
     sinDec = sinAlt*sinlat + cosAlt*coslat*cosAz;
@@ -142,8 +141,8 @@ void SkyPoint::HorizontalToEquatorial( const dms *LST, const dms *lat ) {
 
 void SkyPoint::findEcliptic( const dms *Obliquity, dms &EcLong, dms &EcLat ) {
     double sinRA, cosRA, sinOb, cosOb, sinDec, cosDec, tanDec;
-    ra()->SinCos( sinRA, cosRA );
-    dec()->SinCos( sinDec, cosDec );
+    ra().SinCos( sinRA, cosRA );
+    dec().SinCos( sinDec, cosDec );
     Obliquity->SinCos( sinOb, cosOb );
 
     tanDec = sinDec/cosDec;                    // FIXME: -jbb div by zero?
@@ -232,8 +231,8 @@ SkyPoint SkyPoint::moveAway( SkyPoint &from, double dist ){
     double dst = fabs( dist * dms::DegToRad / 3600.0 ); // In radian
     
     // Compute the bearing angle w.r.t. the RA axis ("latitude")
-    dms dRA( ra()->Degrees() - from.ra()->Degrees() );
-    dms dDec( dec()->Degrees() - from.dec()->Degrees() );
+    dms dRA( ra().Degrees() - from.ra().Degrees() );
+    dms dDec( dec().Degrees() - from.dec().Degrees() );
     double bearing = atan2( dRA.sin() / dRA.cos(), dDec.sin() ); // Do not use dRA = PI / 2!!
     //double bearing = atan2( dDec.radians() , dRA.radians() );
     
@@ -241,12 +240,12 @@ SkyPoint SkyPoint::moveAway( SkyPoint &from, double dist ){
     dist = fabs( dist ); // in radian
 
 
-    lat1.setRadians( asin( dec()->sin() * cos( dst ) +
-                           dec()->cos() * sin( dst ) * cos( dir0 ) ) );
-    dtheta.setRadians( atan2( sin( dir0 ) * sin( dst ) * dec()->cos(),
-                              cos( dst ) - dec()->sin() * lat1.sin() ) );
+    lat1.setRadians( asin( dec().sin() * cos( dst ) +
+                           dec().cos() * sin( dst ) * cos( dir0 ) ) );
+    dtheta.setRadians( atan2( sin( dir0 ) * sin( dst ) * dec().cos(),
+                              cos( dst ) - dec().sin() * lat1.sin() ) );
     
-    dms finalRA( ra()->Degrees() + dtheta.Degrees() );
+    dms finalRA( ra().Degrees() + dtheta.Degrees() );
     return SkyPoint( finalRA, lat1 );
 }
 
@@ -259,24 +258,24 @@ bool SkyPoint::bendlight() {
     // radii, where the effect is only about 0.06".  Assuming
     // min. sun-earth distance is 200 solar radii.
 
-    static KSSun* sun;
+    static KSSun* thesun;
     const dms maxAngle( 1.75 * ( 30.0 / 200.0) / dms::DegToRad );
 
-    if( !sun ) // Hope we don't destroy the sun before the program ends!
-        sun = (KSSun*) KStarsData::Instance()->skyComposite()->findByName( "Sun" );
+    if( !thesun ) // Hope we don't destroy the sun before the program ends!
+        thesun = (KSSun*) KStarsData::Instance()->skyComposite()->findByName( "Sun" );
     //    kDebug() << "bendlight says maximum correctable angle is " << maxAngle.Degrees();
 
     // TODO: We can make this code more efficient
-    SkyPoint sp( sun->ra0(), sun->dec0() );
+    SkyPoint sp = *thesun;
     //    kDebug() << "The unaberrated sun is " << sp.angularDistanceTo( sun ).Degrees() << "deg away";
     if( fabs( angularDistanceTo( &sp ).radians() ) <= maxAngle.radians() ) {
         // We correct for GR effects
-        double corr_sec = 1.75 * sun->physicalSize() / ( sun->rearth() * AU_KM * angularDistanceTo( &sp ).sin() );
+        double corr_sec = 1.75 * thesun->physicalSize() / ( thesun->rearth() * AU_KM * angularDistanceTo( &sp ).sin() );
         Q_ASSERT( corr_sec > 0 );
         
         sp = moveAway( sp, corr_sec );
-        setRA( *sp.ra() );
-        setDec( *sp.dec() );
+        setRA(  sp.ra() );
+        setDec( sp.dec() );
         return true;
     }
     return false;
@@ -567,8 +566,8 @@ void SkyPoint::addEterms(void) {
 
     SkyPoint spd = Eterms();
 
-    RA.setD( RA.Degrees() + spd.ra()->Degrees() );
-    Dec.setD( Dec.Degrees() + spd.dec()->Degrees() );
+    RA.setD( RA.Degrees() + spd.ra().Degrees() );
+    Dec.setD( Dec.Degrees() + spd.dec().Degrees() );
 
 }
 
@@ -576,14 +575,14 @@ void SkyPoint::subtractEterms(void) {
 
     SkyPoint spd = Eterms();
 
-    RA.setD( RA.Degrees() - spd.ra()->Degrees() );
-    Dec.setD( Dec.Degrees() - spd.dec()->Degrees() );
+    RA.setD( RA.Degrees() - spd.ra().Degrees() );
+    Dec.setD( Dec.Degrees() - spd.dec().Degrees() );
 }
 
-dms SkyPoint::angularDistanceTo(const SkyPoint *sp) {
+dms SkyPoint::angularDistanceTo(const SkyPoint *sp) const {
 
-    double dalpha = ra()->radians() - sp->ra()->radians() ;
-    double ddelta = dec()->radians() - sp->dec()->radians() ;
+    double dalpha = ra().radians() - sp->ra().radians() ;
+    double ddelta = dec().radians() - sp->dec().radians() ;
 
     double sa = sin(dalpha/2.);
     double sd = sin(ddelta/2.);
@@ -591,7 +590,7 @@ dms SkyPoint::angularDistanceTo(const SkyPoint *sp) {
     double hava = sa*sa;
     double havd = sd*sd;
 
-    double aux = havd + cos (dec()->radians()) * cos(sp->dec()->radians())
+    double aux = havd + cos (dec().radians()) * cos(sp->dec().radians())
                  * hava;
 
     dms angDist;
@@ -624,8 +623,8 @@ double SkyPoint::vRSun(long double jd0) {
 
     aux.precessFromAnyEpoch(jd0, J2000);
 
-    aux.ra()->SinCos( sinRA, cosRA );
-    aux.dec()->SinCos( sinDec, cosDec );
+    aux.ra().SinCos( sinRA, cosRA );
+    aux.dec().SinCos( sinDec, cosDec );
 
     /* Computation is done performing the scalar product of a unitary vector
     in the direction of the source with the vector velocity of Sun, both being in the
@@ -672,8 +671,8 @@ double SkyPoint::vREarth(long double jd0)
 
     aux.precessFromAnyEpoch(jd0, J2000);
 
-    aux.ra()->SinCos( sinRA, cosRA );
-    aux.dec()->SinCos( sinDec, cosDec );
+    aux.ra().SinCos( sinRA, cosRA );
+    aux.dec().SinCos( sinDec, cosDec );
 
     /* vEarth is referred to the J2000 equinox, hence we need that
     the source coordinates are also in the same reference system.
@@ -718,8 +717,44 @@ double SkyPoint::vTopocentric(double vgeo, double vsite[3])
 }
 
 bool SkyPoint::checkCircumpolar( const dms *gLat ) {
-    return fabs(dec()->Degrees())  >  (90 - fabs(gLat->Degrees()));
+    return fabs(dec().Degrees())  >  (90 - fabs(gLat->Degrees()));
 }
 
+dms SkyPoint::altRefracted() const {
+    if( Options::useRefraction() )
+        return refract(Alt);
+    else
+        return Alt;
+}
 
+double SkyPoint::refractionCorr(double alt) {
+    return 1.02 / tan(dms::DegToRad * ( alt + 10.3/(alt + 5.11) )) / 60;
+}
+// Critical height. Below this height formula produce meaningless
+// results and correction value is just interpolated
 
+dms SkyPoint::refract(dms h) {
+    const double alt = h.Degrees();
+    const double corrCrit = SkyPoint::refractionCorr( SkyPoint::altCrit );
+
+    if( alt > SkyPoint::altCrit )
+        return dms( alt + SkyPoint::refractionCorr(alt) );
+    else
+        return dms( alt + corrCrit * (alt + 90) / (SkyPoint::altCrit + 90) );
+}
+
+// Found uncorrected value by solving equation. This is OK since
+// unrefract is never called in loops.
+//
+// Convergence is quite fast just a few iterations.
+dms SkyPoint::unrefract(dms h) {
+    const double alt = h.Degrees();
+
+    double h0 = alt;
+    double h1 = alt - refractionCorr(h0);
+    while( fabs(h1 - h0) > 1e-4 ) {
+        h0 = h1;
+        h1 = alt - refractionCorr(h0);
+    }
+    return dms(h1);
+}
