@@ -52,6 +52,7 @@
 #include "dialogs/locationdialog.h"
 #include "skyobjects/skyobject.h"
 #include "skyobjects/starobject.h"
+#include "skycomponents/skymapcomposite.h"
 #include "skymap.h"
 #include "dialogs/detaildialog.h"
 #include "dialogs/finddialog.h"
@@ -61,9 +62,9 @@
 #include "imageviewer.h"
 #include "thumbnailpicker.h"
 #include "obslistpopupmenu.h"
-#include "comast/log.h"
-#include "comast/comast.h"
-#include "comast/execute.h"
+#include "oal/log.h"
+#include "oal/oal.h"
+#include "oal/execute.h"
 
 #include <config-kstars.h>
 
@@ -96,6 +97,7 @@ ObservingList::ObservingList( KStars *_ks )
     setCaption( i18n( "Observing List" ) );
     setButtons( KDialog::Close );
     dt = KStarsDateTime::currentDateTime();
+    setFocusPolicy(Qt::StrongFocus);
     geo = ks->data()->geo();
     sessionView = false;
     nativeSave = true;
@@ -135,13 +137,10 @@ ObservingList::ObservingList( KStars *_ks )
     ui->SetLocation->setText( geo -> fullName() );
     ui->ImagePreview->installEventFilter( this );
     ui->TableView->viewport()->installEventFilter( this );
+    ui->TableView->installEventFilter( this );
     ui->SessionView->viewport()->installEventFilter( this );
-    QFile file;
-    if ( KSUtils::openDataFile( file, "noimage.png" ) ) {
-       file.close();
-       ui->ImagePreview->showPreview( KUrl( file.fileName() ) );
-    } else
-        ui->ImagePreview->hide();
+    ui->SessionView->installEventFilter( this );
+    // setDefaultImage();
     //Connections
     connect( this, SIGNAL( closeClicked() ), this, SLOT( slotClose() ) );
     connect( ui->TableView, SIGNAL( doubleClicked( const QModelIndex& ) ),
@@ -173,7 +172,7 @@ ObservingList::ObservingList( KStars *_ks )
     connect( ui->SaveImage, SIGNAL( clicked() ),
              this, SLOT( slotSaveImage() ) );
     connect( ui->DeleteImage, SIGNAL( clicked() ),
-             this, SLOT( slotDeleteImage() ) );
+             this, SLOT( slotDeleteCurrentImage() ) );
     connect( ui->GoogleImage, SIGNAL( clicked() ),
              this, SLOT( slotGoogleImage() ) );
     connect( ui->SetTime, SIGNAL( clicked() ),
@@ -181,9 +180,9 @@ ObservingList::ObservingList( KStars *_ks )
     connect( ui->tabWidget, SIGNAL( currentChanged(int) ),
              this, SLOT( slotChangeTab(int) ) );
     connect( ui->saveImages, SIGNAL( clicked() ),
-             this, SLOT( slotSaveImages() ) );
-    connect( ui->DeleteImages, SIGNAL( clicked() ),
-             this, SLOT( slotDeleteImages() ) );
+             this, SLOT( slotSaveAllImages() ) );
+    connect( ui->DeleteAllImages, SIGNAL( clicked() ),
+             this, SLOT( slotDeleteAllImages() ) );
     connect( ui->OALExport, SIGNAL( clicked() ),
              this, SLOT( slotOALExport() ) );  
     //Add icons to Push Buttons
@@ -206,7 +205,7 @@ ObservingList::ObservingList( KStars *_ks )
 
     slotLoadWishList(); //Load the wishlist from disk if present
     m_CurrentObject = 0;
-    setSaveImages();
+    setSaveImagesButton();
     //Hide the MiniButton until I can figure out how to resize the Dialog!
 //    ui->MiniButton->hide();
 }
@@ -252,18 +251,24 @@ void ObservingList::slotAddObject( SkyObject *obj, bool session, bool update ) {
         m_ObservingList.append( obj );
         m_CurrentObject = obj;
         QList<QStandardItem*> itemList;
-        if(obj->name() == "star" )
-            itemList << new QStandardItem( obj->translatedName() ) 
-                    << new QStandardItem( obj->ra0()->toHMSString() ) 
-                    << new QStandardItem( obj->dec0()->toDMSString() ) 
-                    << new QStandardItem( smag )
-                    << new QStandardItem( obj->typeName() );
-        else
-            itemList << new QStandardItem( obj->translatedName() ) 
-                    << new QStandardItem( p.ra()->toHMSString() ) 
-                    << new QStandardItem( p.dec()->toDMSString() ) 
-                    << new QStandardItem( smag )
-                    << new QStandardItem( obj->typeName() );
+
+        QString ra, dec;
+
+        if(obj->name() == "star" ) {
+            ra = obj->ra0().toHMSString();
+            dec = obj->dec0().toDMSString();
+        }
+        else {
+            ra = p.ra().toHMSString();
+            dec = p.dec().toDMSString();
+        }
+
+        itemList << new QStandardItem( obj->translatedName() )
+                << new QStandardItem( ra )
+                << new QStandardItem( dec )
+                << new QStandardItem( smag )
+                << new QStandardItem( obj->typeName() );
+
         m_Model->appendRow( itemList );
         //Note addition in statusbar
         ks->statusBar()->changeItem( i18n( "Added %1 to observing list.", obj->name() ), 0 );
@@ -277,24 +282,30 @@ void ObservingList::slotAddObject( SkyObject *obj, bool session, bool update ) {
         dms lst(geo->GSTtoLST( dt.gst() ));
         p.EquatorialToHorizontal( &lst, geo->lat() );
         QList<QStandardItem*> itemList;
-        if(obj->name() == "star" )
-            itemList << new QStandardItem( obj->translatedName() ) 
-                    << new QStandardItem( obj->ra0()->toHMSString() ) 
-                    << new QStandardItem( obj->dec0()->toDMSString() ) 
-                    << new QStandardItem( smag )
-                    << new QStandardItem( obj->typeName() )
-                    << new QStandardItem( "--"  )
-                    << new QStandardItem( "--"  )
-                    << new QStandardItem( "--"  );
-        else
-            itemList << new QStandardItem( obj->translatedName() ) 
-                    << new QStandardItem( p.ra()->toHMSString() ) 
-                    << new QStandardItem( p.dec()->toDMSString() ) 
-                    << new QStandardItem( smag )
-                    << new QStandardItem( obj->typeName() )
-                    << new QStandardItem( TimeHash.value( obj->name(), obj->transitTime( dt, geo ) ).toString( "HH:mm" ) )
-                    << new QStandardItem( p.alt()->toDMSString() )
-                    << new QStandardItem( p.az()->toDMSString() );
+
+        QString ra, dec, time = "--", alt = "--", az = "--";
+
+        if(obj->name() == "star" ) {
+            ra = obj->ra0().toHMSString();
+            dec = obj->dec0().toDMSString();
+        }
+        else {
+            ra = p.ra().toHMSString();
+            dec = p.dec().toDMSString();
+            time = TimeHash.value( obj->name(), obj->transitTime( dt, geo ) ).toString( "HH:mm" );
+            alt = p.alt().toDMSString();
+            az = p.az().toDMSString();
+        }
+
+        itemList << new QStandardItem( obj->translatedName() )
+                << new QStandardItem( ra )
+                << new QStandardItem( dec )
+                << new QStandardItem( smag )
+                << new QStandardItem( obj->typeName() )
+                << new QStandardItem( time )
+                << new QStandardItem( alt )
+                << new QStandardItem( az );
+
         m_Session->appendRow( itemList );
         //Adding an object should trigger the modified flag
         if ( ! isModified ) isModified = true;
@@ -302,7 +313,7 @@ void ObservingList::slotAddObject( SkyObject *obj, bool session, bool update ) {
         //Note addition in statusbar
         ks->statusBar()->changeItem( i18n( "Added %1 to session list.", obj->name() ), 0 );
     }
-    setSaveImages();
+    setSaveImagesButton();
 }
 
 void ObservingList::slotRemoveObject( SkyObject *o, bool session, bool update ) {
@@ -312,64 +323,51 @@ void ObservingList::slotRemoveObject( SkyObject *o, bool session, bool update )
         else if( sessionView ) //else if is needed as clickedObject should not be removed from the session list.
             session = true;
     }
-    if( ! session ) {
-        int k = obsList().indexOf( o );
-        if ( o == LogObject ) saveCurrentUserLog();
-        //Remove row from the TableView model
-        bool found(false);
-        if ( o->name() == "star" ) {
-            //Find object in table by RA and Dec
-            for ( int irow = 0; irow < m_Model->rowCount(); ++irow ) {
-                QString ra = m_Model->item(irow, 1)->text();
-                QString dc = m_Model->item(irow, 2)->text();
-                if ( o->ra0()->toHMSString() == ra && o->dec0()->toDMSString() == dc ) {
-                    m_Model->removeRow(irow);
-                    found = true;
-                    break;
-                }
-            }
-        } else { // name is not "star"
-            //Find object in table by name
-            for ( int irow = 0; irow < m_Model->rowCount(); ++irow ) {
-                QString name = m_Model->item(irow, 0)->text();
-                if ( o->translatedName() == name ) {
-                    m_Model->removeRow(irow);
-                    found = true;
-                    break;
-                }
+
+    QStandardItemModel *currentModel;
+
+    int k;
+    if (! session) {
+        currentModel = m_Model;
+        k = obsList().indexOf( o );
+    }
+    else {
+        currentModel = m_Session;
+        k = sessionList().indexOf( o );
+    }
+
+    if ( o == LogObject ) saveCurrentUserLog();
+    //Remove row from the TableView model
+    bool found(false);
+    if ( o->name() == "star" ) {
+        //Find object in table by RA and Dec
+        for ( int irow = 0; irow < currentModel->rowCount(); ++irow ) {
+            QString ra = currentModel->item(irow, 1)->text();
+            QString dc = currentModel->item(irow, 2)->text();
+            if ( o->ra0().toHMSString() == ra && o->dec0().toDMSString() == dc ) {
+                currentModel->removeRow(irow);
+                found = true;
+                break;
             }
         }
+    } else { // name is not "star"
+        //Find object in table by name
+        for ( int irow = 0; irow < currentModel->rowCount(); ++irow ) {
+            QString name = currentModel->item(irow, 0)->text();
+            if ( o->translatedName() == name ) {
+                currentModel->removeRow(irow);
+                found = true;
+                break;
+            }
+        }
+    }
+
+    if( ! session ) {
         obsList().removeAt(k);
         ui->View->removeAllPlotObjects();
         ui->TableView->resizeColumnsToContents();
         if( ! update ) slotSaveList();
     } else {
-        int k = sessionList().indexOf( o );
-        if ( o == LogObject ) saveCurrentUserLog();
-        //Remove row from the Session View model
-        bool found(false);
-        if ( o->name() == "star" ) {
-            //Find object in table by RA and Dec
-            for ( int irow = 0; irow < m_Model->rowCount(); ++irow ) {
-                QString ra = m_Session->item(irow, 1)->text();
-                QString dc = m_Session->item(irow, 2)->text();
-                if ( o->ra0()->toHMSString() == ra && o->dec0()->toDMSString() == dc ) {
-                    m_Session->removeRow(irow);
-                    found = true;
-                    break;
-                }
-            }
-        } else { // name is not "star"
-            //Find object in table by name
-            for ( int irow = 0; irow < m_Session->rowCount(); ++irow ) {
-                QString name = m_Session->item(irow, 0)->text();
-                if ( o->translatedName() == name ) {
-                    m_Session->removeRow(irow);
-                    found = true;
-                    break;
-                }
-            }
-        }
         if( ! update )
             TimeHash.remove( o->name() );
         sessionList().removeAt(k);//Remove from the session list
@@ -379,66 +377,72 @@ void ObservingList::slotRemoveObject( SkyObject *o, bool session, bool update )
     }
 }
 
+void ObservingList::removeObject(SkyObject *o, bool sessionView) {
+    if ( sessionView )
+        slotRemoveObject( o, true );
+    else
+        slotRemoveObject( o );
+}
+
 void ObservingList::slotRemoveSelectedObjects() {
-    if( sessionView )
-    {
-        //Find each object by name in the session list, and remove it
-        //Go backwards so item alignment doesn't get screwed up as rows are removed.
-        for ( int irow = m_Session->rowCount()-1; irow >= 0; --irow ) {
-            if ( ui->SessionView->selectionModel()->isRowSelected( irow, QModelIndex() ) ) {
-                QModelIndex mSortIndex = m_SortModelSession->index( irow, 0 );
-                QModelIndex mIndex = m_SortModelSession->mapToSource( mSortIndex );
-                int irow = mIndex.row();
-                QString ra = m_Session->item(irow, 1)->text();
-                QString dc = m_Session->item(irow, 2)->text();
-                foreach ( SkyObject *o, sessionList() ) {
-                    //Stars named "star" must be matched by coordinates
-                    if ( o->name() == "star" ) {
-                        if ( o->ra0()->toHMSString() == ra && o->dec0()->toDMSString() == dc ) {
-                            slotRemoveObject( o, true );
-                            break;
-                        }
-    
-                    } else if ( o->translatedName() == mIndex.data().toString() ) {
-                        slotRemoveObject( o, true );
+    QStandardItemModel *currentModel;
+//    ObservingListUI local_ui;
+    if ( sessionView )    
+        currentModel = m_Session;
+    else
+        currentModel = m_Model;
+
+    //Find each object by name in the session list, and remove it
+    //Go backwards so item alignment doesn't get screwed up as rows are removed.
+    for ( int irow = currentModel->rowCount()-1; irow >= 0; --irow ) {
+        bool rowSelected;
+        if ( sessionView )
+            rowSelected = ui->SessionView->selectionModel()->isRowSelected( irow, QModelIndex() );
+        else
+            rowSelected = ui->TableView->selectionModel()->isRowSelected( irow, QModelIndex() );
+
+        if ( rowSelected ) {
+            QList<SkyObject*> currList;
+            QModelIndex mSortIndex, mIndex;
+             if ( sessionView ) {
+                mSortIndex = m_SortModelSession->index( irow, 0 );
+                mIndex = m_SortModelSession->mapToSource( mSortIndex );
+                currList = sessionList();
+            } else {
+                mSortIndex = m_SortModel->index( irow, 0 );
+                mIndex = m_SortModel->mapToSource( mSortIndex );
+                currList = obsList();
+            }
+
+            int irow = mIndex.row();
+            QString ra = currentModel->item(irow, 1)->text();
+            QString dc = currentModel->item(irow, 2)->text();
+
+            foreach ( SkyObject *o, currList ) {
+                //Stars named "star" must be matched by coordinates
+                if ( o->name() == "star" ) {
+                    if ( o->ra0().toHMSString() == ra && o->dec0().toDMSString() == dc ) {
+                        removeObject(o, sessionView);
+                        break;
+                    }
+                } else {
+                    if ( o->translatedName() == mIndex.data().toString() ) {
+                        removeObject(o, sessionView);
                         break;
                     }
                 }
             }
         }
+    }
+
+    if (sessionView) {
         //we've removed all selected objects, so clear the selection
         ui->SessionView->selectionModel()->clear();
         //Update the lists in the Execute window as well
         ks->getExecute()->init();
-    } else {
-         //Find each object by name in the observing list, and remove it
-         //Go backwards so item alignment doesn't get screwed up as rows are removed.
-         for ( int irow = m_Model->rowCount()-1; irow >= 0; --irow ) {
-             if ( ui->TableView->selectionModel()->isRowSelected( irow, QModelIndex() ) ) {
-                 QModelIndex mSortIndex = m_SortModel->index( irow, 0 );
-                 QModelIndex mIndex = m_SortModel->mapToSource( mSortIndex );
-                 int irow = mIndex.row();
-                 QString ra = m_Model->item(irow, 1)->text();
-                 QString dc = m_Model->item(irow, 2)->text(); 
-                 foreach ( SkyObject *o, obsList() ) {
-                     //Stars named "star" must be matched by coordinates
-                     if ( o->name() == "star" ) {
-                         if ( o->ra0()->toHMSString() == ra && o->dec0()->toDMSString() == dc ) {
-                             slotRemoveObject( o );
-                             break;
-                         }
-     
-                     } else if ( o->translatedName() == mIndex.data().toString() ) {
-                         slotRemoveObject( o );
-                         break;
-                     }
-                 }
-             }
-         }
-        //we've removed all selected objects, so clear the selection
-        ui->TableView->selectionModel()->clear();
     }
-    setSaveImages();
+
+    setSaveImagesButton();
     ui->ImagePreview->setCursor( Qt::ArrowCursor );
 }
 
@@ -449,56 +453,46 @@ void ObservingList::slotNewSelection() {
     showScope = false;
     ui->ImagePreview->clearPreview();
     ui->ImagePreview->setCursor( Qt::ArrowCursor );
-    QFile file;
-    if ( KSUtils::openDataFile( file, "noimage.png" ) ) {
-       file.close();
-       ui->ImagePreview->showPreview( KUrl( file.fileName() ) );
-    } else
-        ui->ImagePreview->hide();
     QModelIndexList selectedItems;
     QString newName;
     SkyObject *o;
     ui->SaveImage->setEnabled( false );
     ui->DeleteImage->setEnabled( false );
-    if( sessionView ) {
+
+    QStandardItemModel *currentModel;
+    QList<SkyObject*> currList;
+
+    if ( sessionView ) {
         selectedItems = m_SortModelSession->mapSelectionToSource( ui->SessionView->selectionModel()->selection() ).indexes();
-        //When one object is selected
-        if ( selectedItems.size() == m_Session->columnCount() ) {
-            newName = selectedItems[0].data().toString();
-            singleSelection = true;
-            //Find the selected object in the SessionList,
-            //then break the loop.  Now SessionList.current()
-            //points to the new selected object (until now it was the previous object)
-            foreach ( o, sessionList() ) {
-                if ( o->translatedName() == newName ) {
-                    found = true;
-                    break;
-                }
-            }
-        }
+        currentModel = m_Session;
+        currList = sessionList();
     } else {
         selectedItems = m_SortModel->mapSelectionToSource( ui->TableView->selectionModel()->selection() ).indexes();
-        //When one object is selected
-        if ( selectedItems.size() == m_Model->columnCount() ) {
-            newName = selectedItems[0].data().toString();
-            singleSelection = true;
-            //Find the selected object in the obsList,
-            //then break the loop.  Now obsList.current()
-            //points to the new selected object (until now it was the previous object)
-            foreach ( o, obsList() ) {
-                if ( o->translatedName() == newName ) {
-                    found = true;
+        currentModel = m_Model;
+        currList = obsList();
+    }
+
+    if ( selectedItems.size() == currentModel->columnCount() ) {
+        newName = selectedItems[0].data().toString();
+        singleSelection = true;
+        //Find the selected object in the SessionList,
+        //then break the loop.  Now SessionList.current()
+        //points to the new selected object (until now it was the previous object)
+        foreach ( o, currList ) {
+            if ( o->translatedName() == newName ) {
+                found = true;
                 break;
-                }
             }
         }
     }
+
     if( singleSelection ) {
         //Enable buttons
         ui->ImagePreview->setCursor( Qt::PointingHandCursor );
         #ifdef HAVE_INDI_H
             showScope = true;
         #endif
+        setDefaultImage();
         if ( found ) {
             m_CurrentObject = o;
             QPoint pos(0,0);
@@ -548,41 +542,52 @@ void ObservingList::slotNewSelection() {
         } else {
             kDebug() << i18n( "Object %1 not found in list.", newName );
         } 
-    } else if ( selectedItems.size() == 0 ) {//Nothing selected
-        //Disable buttons
-        noSelection = true;
-        ui->NotesLabel->setText( i18n( "Select an object to record notes on it here:" ) );
-        ui->NotesLabel->setEnabled( false );
-        ui->NotesEdit->setEnabled( false );
-        m_CurrentObject = 0;
-        ui->TimeEdit->setEnabled( false );
-        ui->SetTime->setEnabled( false );
-        ui->GoogleImage->setEnabled( false );
-        //Clear the user log text box.
-        saveCurrentUserLog();
-        ui->NotesEdit->setPlainText("");
-        //Clear the plot in the AVTPlotwidget
-        ui->View->removeAllPlotObjects();
-    } else { //more than one object selected.
-        ui->NotesLabel->setText( i18n( "Select a single object to record notes on it here:" ) );
-        ui->NotesLabel->setEnabled( false );
-        ui->NotesEdit->setEnabled( false );
-        ui->TimeEdit->setEnabled( false );
-        ui->SetTime->setEnabled( false );
-        ui->GoogleImage->setEnabled( false );
-        m_CurrentObject = 0;
-        //Clear the plot in the AVTPlotwidget
-        ui->View->removeAllPlotObjects();
-        //Clear the user log text box.
-        saveCurrentUserLog();
-        ui->NotesEdit->setPlainText("");
+    } else {
+        if ( selectedItems.size() == 0 ) {//Nothing selected
+            //Disable buttons
+            noSelection = true;
+            ui->NotesLabel->setText( i18n( "Select an object to record notes on it here:" ) );
+            ui->NotesLabel->setEnabled( false );
+            ui->NotesEdit->setEnabled( false );
+            m_CurrentObject = 0;
+            ui->TimeEdit->setEnabled( false );
+            ui->SetTime->setEnabled( false );
+            ui->GoogleImage->setEnabled( false );
+            //Clear the user log text box.
+            saveCurrentUserLog();
+            ui->NotesEdit->setPlainText("");
+            //Clear the plot in the AVTPlotwidget
+            ui->View->removeAllPlotObjects();
+        } else { //more than one object selected.
+            ui->NotesLabel->setText( i18n( "Select a single object to record notes on it here:" ) );
+            ui->NotesLabel->setEnabled( false );
+            ui->NotesEdit->setEnabled( false );
+            ui->TimeEdit->setEnabled( false );
+            ui->SetTime->setEnabled( false );
+            ui->GoogleImage->setEnabled( false );
+            m_CurrentObject = 0;
+            //Clear the plot in the AVTPlotwidget
+            ui->View->removeAllPlotObjects();
+            //Clear the user log text box.
+            saveCurrentUserLog();
+            ui->NotesEdit->setPlainText("");
+        }
     }
 }
 
 void ObservingList::slotCenterObject() {
-    ks->map()->setClickedObject( currentObject() );
-    ks->map()->setClickedPoint( currentObject() );
-    ks->map()->slotCenter();
+    QModelIndexList selectedItems;
+    if (sessionView) {
+        selectedItems = ui->SessionView->selectionModel()->selectedRows();
+    } else {
+        selectedItems = ui->TableView->selectionModel()->selectedRows();
+    }
+
+    if (selectedItems.size() == 1) {
+        ks->map()->setClickedObject( currentObject() );
+        ks->map()->setClickedPoint( currentObject() );
+        ks->map()->slotCenter();
+    }
 }
 
 void ObservingList::slotSlewToObject()
@@ -590,11 +595,7 @@ void ObservingList::slotSlewToObject()
 #ifdef HAVE_INDI_H
 
     INDI_D *indidev(NULL);
-    INDI_P *prop(NULL), *onset(NULL);
-    INDI_E *RAEle(NULL), *DecEle(NULL), *AzEle(NULL), *AltEle(NULL), *ConnectEle(NULL), *nameEle(NULL);
-    bool useJ2000( false);
-    int selectedCoord(0);
-    SkyPoint sp;
+    INDI_E *ConnectEle(NULL);
 
     // Find the first device with EQUATORIAL_EOD_COORD or EQUATORIAL_COORD and with SLEW element
     // i.e. the first telescope we find!
@@ -605,23 +606,6 @@ void ObservingList::slotSlewToObject()
         for (int j=0; j < imenu->managers.at(i)->indi_dev.size(); j++)
         {
             indidev = imenu->managers.at(i)->indi_dev.at(j);
-            indidev->stdDev->currentObject = NULL;
-            prop = indidev->findProp("EQUATORIAL_EOD_COORD");
-            if (prop == NULL)
-            {
-                prop = indidev->findProp("EQUATORIAL_COORD");
-                if (prop == NULL)
-                {
-                    prop = indidev->findProp("HORIZONTAL_COORD");
-                    if (prop == NULL)
-                        continue;
-                    else
-                        selectedCoord = 1;      /* Select horizontal */
-                }
-                else
-                    useJ2000 = true;
-            }
-
             ConnectEle = indidev->findElem("CONNECT");
             if (!ConnectEle) continue;
 
@@ -631,88 +615,15 @@ void ObservingList::slotSlewToObject()
                 return;
             }
 
-            switch (selectedCoord)
-            {
-                // Equatorial
-            case 0:
-                if (prop->perm == PP_RO) continue;
-                RAEle  = prop->findElement("RA");
-                if (!RAEle) continue;
-                DecEle = prop->findElement("DEC");
-                if (!DecEle) continue;
-                break;
+        if (!indidev->stdDev->slew_scope(static_cast<SkyPoint *> (currentObject())))
+        continue;
 
-                // Horizontal
-            case 1:
-                if (prop->perm == PP_RO) continue;
-                AzEle = prop->findElement("AZ");
-                if (!AzEle) continue;
-                AltEle = prop->findElement("ALT");
-                if (!AltEle) continue;
-                break;
-            }
-
-            onset = indidev->findProp("ON_COORD_SET");
-            if (!onset) continue;
-
-            onset->activateSwitch("SLEW");
-
-            indidev->stdDev->currentObject = currentObject();
-
-            /* Send object name if available */
-            if (indidev->stdDev->currentObject)
-            {
-                nameEle = indidev->findElem("OBJECT_NAME");
-                if (nameEle && nameEle->pp->perm != PP_RO)
-                {
-                    nameEle->write_w->setText(indidev->stdDev->currentObject->name());
-                    nameEle->pp->newText();
-                }
-            }
-
-            switch (selectedCoord)
-            {
-            case 0:
-                if (indidev->stdDev->currentObject)
-                    sp.set (indidev->stdDev->currentObject->ra(), indidev->stdDev->currentObject->dec());
-                else
-                    sp.set (ks->map()->clickedPoint()->ra(), ks->map()->clickedPoint()->dec());
-
-                if (useJ2000)
-                    sp.apparentCoord(ks->data()->ut().djd(), (long double) J2000);
-
-                RAEle->write_w->setText(QString("%1:%2:%3").arg(sp.ra()->hour()).arg(sp.ra()->minute()).arg(sp.ra()->second()));
-                DecEle->write_w->setText(QString("%1:%2:%3").arg(sp.dec()->degree()).arg(sp.dec()->arcmin()).arg(sp.dec()->arcsec()));
-
-                break;
-
-            case 1:
-                if (indidev->stdDev->currentObject)
-                {
-                    sp.setAz(*indidev->stdDev->currentObject->az());
-                    sp.setAlt(*indidev->stdDev->currentObject->alt());
-                }
-                else
-                {
-                    sp.setAz(*ks->map()->clickedPoint()->az());
-                    sp.setAlt(*ks->map()->clickedPoint()->alt());
-                }
-
-                AzEle->write_w->setText(QString("%1:%2:%3").arg(sp.az()->degree()).arg(sp.az()->arcmin()).arg(sp.az()->arcsec()));
-                AltEle->write_w->setText(QString("%1:%2:%3").arg(sp.alt()->degree()).arg(sp.alt()->arcmin()).arg(sp.alt()->arcsec()));
-
-                break;
-            }
-
-            prop->newText();
-
-            return;
+        return;
         }
     }
 
     // We didn't find any telescopes
     KMessageBox::sorry(0, i18n("KStars did not find any active telescopes."));
-
 #endif
 }
 
@@ -722,7 +633,7 @@ void ObservingList::slotDetails() {
     if ( currentObject() ) {
         QPointer<DetailDialog> dd = new DetailDialog( currentObject(), ks->data()->lt(), geo, ks );
         dd->exec();
-    	delete dd;
+        delete dd;
     }
 }
 
@@ -772,7 +683,7 @@ void ObservingList::slotAVT() {
                 foreach ( SkyObject *o, sessionList() ) {
                     //Stars named "star" must be matched by coordinates
                     if ( o->name() == "star" ) {
-                        if ( o->ra0()->toHMSString() == ra && o->dec0()->toDMSString() == dc ) {
+                        if ( o->ra0().toHMSString() == ra && o->dec0().toDMSString() == dc ) {
                             avt->processObject( o );
                             break;
                         }
@@ -785,7 +696,7 @@ void ObservingList::slotAVT() {
             }
         }
         avt->exec();
-	    delete avt;
+        delete avt;
     } else {
         selectedItems = m_SortModel->mapSelectionToSource( ui->TableView->selectionModel()->selection() ).indexes();
         if ( selectedItems.size() ) {
@@ -796,7 +707,7 @@ void ObservingList::slotAVT() {
                         avt->processObject( o );
             }
             avt->exec();
-	        delete avt;
+            delete avt;
         }
     }       
 }
@@ -857,14 +768,14 @@ void ObservingList::slotOpenList() {
         QTextStream istream( &f );
         QString input;
         input = istream.readAll();
-        Comast::Log logObject;
+        OAL::Log logObject;
         logObject.readBegin( input );
         //Set the New TimeHash
         TimeHash = logObject.timeHash();
         geo = logObject.geoLocation();
         dt = logObject.dateTime();
         foreach( SkyObject *o, *( logObject.targetList() ) )
-            slotAddObject( o, true );
+        slotAddObject( o, true );
         //Update the location and user set times from file
         slotUpdate();
         //Newly-opened list should not trigger isModified flag
@@ -906,7 +817,7 @@ void ObservingList::slotSaveList() {
     QTextStream ostream( &f );
     foreach ( SkyObject* o, obsList() ) {
         if ( o->name() == "star" ) {
-            ostream << o->name() << "  " << o->ra0()->Hours() << "  " << o->dec0()->Degrees() << endl;
+            ostream << o->name() << "  " << o->ra0().Hours() << "  " << o->dec0().Degrees() << endl;
         } else if ( o->type() == SkyObject::STAR ) {
             StarObject *s = (StarObject*)o;
             if ( s->name() == s->gname() ) 
@@ -966,7 +877,7 @@ void ObservingList::slotSaveSession() {
     return;
     }
     QTextStream ostream( &f );
-    Comast::Log log;
+    OAL::Log log;
     ostream<< log.writeLog( nativeSave );
     f.close();
     isModified = false;//We've saved the session, so reset the modified flag.
@@ -1016,7 +927,7 @@ double ObservingList::findAltitude( SkyPoint *p, double hour ) {
     ut= ut.addSecs( hour*3600.0 );
     dms LST = geo->GSTtoLST( ut.gst() );
     p->EquatorialToHorizontal( &LST, geo->lat() );
-    return p->alt()->Degrees();
+    return p->alt().Degrees();
 }
 
 void ObservingList::slotToggleSize() {
@@ -1088,7 +999,7 @@ void ObservingList::slotChangeTab(int index) {
     } else {
         sessionView = false;
     }
-    setSaveImages();
+    setSaveImagesButton();
     ui->WizardButton->setEnabled( ! sessionView );//wizard adds only to the Wish List
     ui->OALExport->setEnabled( sessionView );
     //Clear the selection in the Tables
@@ -1172,15 +1083,15 @@ void ObservingList::downloadReady() {
 
 void ObservingList::setCurrentImage( SkyObject *o, bool temp  ) {
     QString RAString, DecString, RA, Dec;
-    RAString = RAString.sprintf( "%02d+%02d+%02d", o->ra0()->hour(), o->ra0()->minute(), o->ra0()->second() );
+    RAString = RAString.sprintf( "%02d+%02d+%02d", o->ra0().hour(), o->ra0().minute(), o->ra0().second() );
     decsgn = '+';
-    if ( o->dec0()->Degrees() < 0.0 ) decsgn = '-';
-    int dd = abs( o->dec0()->degree() );
-    int dm = abs( o->dec0()->arcmin() );
-    int ds = abs( o->dec0()->arcsec() );
+    if ( o->dec0().Degrees() < 0.0 ) decsgn = '-';
+    int dd = abs( o->dec0().degree() );
+    int dm = abs( o->dec0().arcmin() );
+    int ds = abs( o->dec0().arcsec() );
     DecString = DecString.sprintf( "%c%02d+%02d+%02d", decsgn, dd, dm, ds );
-    RA = RA.sprintf( "ra=%f", o->ra0()->Degrees() );
-    Dec = Dec.sprintf( "&dec=%f", o->dec0()->Degrees() );
+    RA = RA.sprintf( "ra=%f", o->ra0().Degrees() );
+    Dec = Dec.sprintf( "&dec=%f", o->dec0().Degrees() );
     if( temp )
         CurrentImage = "Temp_Image_" +  o->name().remove(' ');
     else
@@ -1203,7 +1114,7 @@ void ObservingList::setCurrentImage( SkyObject *o, bool temp  ) {
     SDSSUrl = UrlPrefix + RA + Dec + UrlSuffix;
 }
 
-void ObservingList::slotSaveImages() {
+void ObservingList::slotSaveAllImages() {
     ui->GoogleImage->setEnabled( false );
     ui->SaveImage->setEnabled( false );
     ui->DeleteImage->setEnabled( false );
@@ -1212,22 +1123,18 @@ void ObservingList::slotSaveImages() {
     ui->TableView->clearSelection();
     ui->SessionView->clearSelection();
 
-    if( sessionView ) {
-        foreach( SkyObject *o, sessionList() ) {
-            setCurrentImage( o );
-            QString img( CurrentImagePath  );
-            KUrl url( SDSSUrl );
-            if( ! o->isSolarSystem() )//TODO find a way for adding support for solar system images
-                saveImage( url, img );
-        }
-    } else {
-        foreach( SkyObject *o, obsList() ) {
-            setCurrentImage( o );
-            QString img( CurrentImagePath  );
-            KUrl url( SDSSUrl );
-            if( ! o->isSolarSystem() )//TODO find a way for adding support for solar system images
-                saveImage( url, img );
-        }
+    QList<SkyObject*> currList;
+    if ( sessionView )
+        currList = sessionList();
+    else
+        currList = obsList();
+
+    foreach( SkyObject *o, currList ) {
+        setCurrentImage( o );
+        QString img( CurrentImagePath  );
+        KUrl url( SDSSUrl );
+        if( ! o->isSolarSystem() )//TODO find a way for adding support for solar system images
+            saveImage( url, img );
     }
 }
 
@@ -1264,7 +1171,9 @@ void ObservingList::slotImageViewer() {
         iv->show();
 }
 
-void ObservingList::slotDeleteImages() {
+void ObservingList::slotDeleteAllImages() {
+    if( KMessageBox::warningYesNo( 0, i18n( "This will delete all saved images. Are you sure you want to do this?" ), i18n( "Delete All Images" ) ) == KMessageBox::No )
+        return;
     ui->ImagePreview->setCursor( Qt::ArrowCursor );
     ui->GoogleImage->setEnabled( false );
     ui->SaveImage->setEnabled( false );
@@ -1277,6 +1186,7 @@ void ObservingList::slotDeleteImages() {
     QDirIterator iterator( KStandardDirs::locateLocal( "appdata", "" ) );
     while( iterator.hasNext() )
     {
+        // TODO: Probably, there should be a different directory for cached images in the observing list.
         if( iterator.fileName().contains( "Image" ) && ( ! iterator.fileName().contains( "dat" ) ) && ( ! iterator.fileName().contains( "obslist" ) ) ) {
             QFile file( iterator.filePath() );
             file.remove();
@@ -1285,7 +1195,7 @@ void ObservingList::slotDeleteImages() {
     }
 }
 
-void ObservingList::setSaveImages() {
+void ObservingList::setSaveImagesButton() {
     ui->saveImages->setEnabled( false );
     if( sessionView ) {
         if( ! sessionList().isEmpty() )
@@ -1326,6 +1236,7 @@ bool ObservingList::eventFilter( QObject *obj, QEvent *event ) {
             }
         }
     }
+
     if( obj == ui->SessionView->viewport() && ! noSelection ) {
         if( event->type() == QEvent::MouseButtonRelease ) {
             QMouseEvent *mouseEvent = static_cast<QMouseEvent* >(event);
@@ -1340,6 +1251,19 @@ bool ObservingList::eventFilter( QObject *obj, QEvent *event ) {
             }
         }
     }
+
+    if( obj == ui->TableView || obj == ui->SessionView)
+    {
+        if (event->type() == QEvent::KeyPress)
+        {
+                QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+                if (keyEvent->key() == Qt::Key_Delete)
+                    slotRemoveSelectedObjects();
+                return true;
+         } else
+                return false;
+    }
+
     return false;
 }
 
@@ -1360,7 +1284,7 @@ void ObservingList::slotGoogleImage() {
     delete tp;
 }
 
-void ObservingList::slotDeleteImage() {
+void ObservingList::slotDeleteCurrentImage() {
     QFile::remove( CurrentImagePath );
     QFile::remove( CurrentTempPath );
     slotNewSelection();
@@ -1416,4 +1340,13 @@ void ObservingList::selectObject( SkyObject *o ) {
     }
 }
 
+void ObservingList::setDefaultImage() {
+    QFile file;
+    if ( KSUtils::openDataFile( file, "noimage.png" ) ) {
+       file.close();
+       ui->ImagePreview->showPreview( KUrl( file.fileName() ) );
+    } else
+        ui->ImagePreview->hide();
+}
+
 #include "observinglist.moc"
