@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -23,11 +23,21 @@ class EquirectScanlineTextureMapper : public AbstractScanlineTextureMapper
     Q_OBJECT
 
  public:
-    EquirectScanlineTextureMapper( GeoSceneTexture *textureLayer, StackedTileLoader *tileLoader,
-                                   QObject *parent = 0 );
-    void mapTexture( ViewParams *viewParams );
+    EquirectScanlineTextureMapper( StackedTileLoader *tileLoader, QObject *parent = 0 );
+
+    virtual void mapTexture( GeoPainter *painter,
+                             ViewParams *viewParams,
+                             const QRect &dirtyRect,
+                             TextureColorizer *texColorizer );
+
+    virtual void setRepaintNeeded();
 
  private:
+    void mapTexture( ViewParams *viewParams,
+                     TextureColorizer *texColorizer );
+
+ private:
+    bool   m_repaintNeeded;
     float  m_oldCenterLon;
     int    m_oldYPaintedTop;
 };
