@@ -1,17 +1,17 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2008 Torsten Rahn <tackat@kde.org>"
-// Copyright 2007      Inge Wallin  <ingwa@kde.org>"
+// Copyright 2006-2008 Torsten Rahn <tackat@kde.org>
+// Copyright 2007      Inge Wallin  <ingwa@kde.org>
+// Copyright 2009      Jens-Michael Hoffmann <jensmh@gmx.de>
 //
 
-
-#ifndef MARBLEMAP_H
-#define MARBLEMAP_H
+#ifndef MARBLE_MARBLEMAP_H
+#define MARBLE_MARBLEMAP_H
 
 
 /** @file
@@ -24,12 +24,14 @@
 
 #include <QtCore/QSize>
 #include <QtCore/QString>
+#include <QtCore/QModelIndex>
 #include <QtCore/QObject>
 #include <QtGui/QFont>
 #include <QtGui/QPixmap>
 
 #include "marble_export.h"
 #include "GeoDataCoordinates.h"       // In geodata/data/
+#include "GeoDataLookAt.h"
 #include "Quaternion.h"
 #include "global.h"             // types needed in all of marble.
 
@@ -37,8 +39,6 @@
 class QAbstractItemModel;
 class QModelIndex;
 class QItemSelectionModel;
-class QStyleOptionGraphicsItem;
-class QUrl;
 
 namespace Marble
 {
@@ -49,15 +49,20 @@ namespace Marble
 class MarbleMapPrivate;
 
 // Marble
+class GeoDataLatLonAltBox;
 class MarbleModel;
-class ViewParams;
-class MeasureTool;
+class ViewportParams;
 class SunLocator;
-class GpxFileModel;
 class FileViewModel;
 class GeoPainter;
+class LayerInterface;
 class RenderPlugin;
+class AbstractDataPlugin;
+class AbstractDataPluginItem;
 class AbstractFloatItem;
+class MeasureTool;
+class TextureLayer;
+class TileCoordsPyramid;
 
 /**
  * @short A class that can paint a view of the earth.
@@ -123,69 +128,64 @@ class MARBLE_EXPORT MarbleMap : public QObject
     /**
      * @brief Return the model that this view shows.
      */
-    MarbleModel  *model() const;
+    MarbleModel *model() const;
 
     // Accessors to internal objects;
-    ViewParams   *viewParams();
-    MeasureTool  *measureTool();
+    ViewportParams *viewport();
 
-    void  setSize(int width, int height);
-    void  setSize(QSize size);
-    QSize size()   const;
-    int   width()  const;
-    int   height() const;
+    void setMapQuality( MapQuality quality );
+    MapQuality mapQuality() const;
+
+    void setSize( int width, int height );
+    void setSize( const QSize& size );
+    QSize size() const;
+    int width() const;
+    int height() const;
 
     /**
      * @brief  Return the radius of the globe in pixels.
      */
-    int         radius() const;
+    int radius() const;
 
     /**
      * @brief  Set the radius of the globe in pixels.
      * @param  radius  The new globe radius value in pixels.
      */
-    void        setRadius(int radius);
+    void setRadius( int radius );
 
     /**
      * @brief Return the current zoom level.
      */
-    int         zoom() const;
+    int zoom() const;
+
+    int tileZoomLevel() const;
 
     /**
-     * @brief Return the current distance.
+     * @brief Return the current distance. Convenience function calling distance(radius())
+     * @see distance(qreal) radius
      */
-    qreal      distance() const;
+    qreal distance() const;
 
     /**
      * @brief  Set the distance of the observer to the globe in km.
      * @param  distance  The new distance in km.
      */
-    void        setDistance( qreal distance );
+    void setDistance( qreal distance );
 
     /**
      * @brief Return the current distance string.
      */
-    QString     distanceString() const;
+    QString distanceString() const;
 
     /**
      * @brief return the minimum zoom value for the current map theme.
      */
-    int         minimumZoom() const;
+    int minimumZoom() const;
 
     /**
      * @brief return the minimum zoom value for the current map theme.
      */
-    int         maximumZoom() const;
-
-    /**
-     * @brief return if the map needs to be updated.
-     */
-    bool needsUpdate() const;
-
-    /**
-     * @brief Mark the map as needing an update.
-     */
-    void setNeedsUpdate();
+    int maximumZoom() const;
 
     /**
      * @brief Get the screen coordinates corresponding to geographical coordinates in the map.
@@ -197,13 +197,7 @@ class MARBLE_EXPORT MarbleMap : public QObject
      *         @c false if the geographical coordinates are not visible on the screen
      */
     bool screenCoordinates( qreal lon, qreal lat,
-                            qreal& x, qreal& y );
-
-    /**
-     * @brief Get the Y coordinate of the North Pole.
-     * @return the pixel address of the geographical north pole.
-     */
-    int northPoleY();
+                            qreal& x, qreal& y ) const;
 
     /**
      * @brief Get the earth coordinates corresponding to a pixel in the map.
@@ -216,17 +210,19 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     bool geoCoordinates( int x, int y,
                          qreal& lon, qreal& lat,
-                         GeoDataCoordinates::Unit = GeoDataCoordinates::Degree );
+                         GeoDataCoordinates::Unit = GeoDataCoordinates::Degree ) const;
 
     /**
      * @brief Return the longitude of the center point.
+     * @return The longitude of the center point in degree.
      */
-    qreal  centerLongitude() const;
+    qreal centerLongitude() const;
 
     /**
      * @brief Return the latitude of the center point.
+     * @return The latitude of the center point in degree.
      */
-    qreal  centerLatitude()  const;
+    qreal centerLatitude() const;
 
     /**
      * @brief returns the model for all the placemarks on the globe.
@@ -239,35 +235,34 @@ class MARBLE_EXPORT MarbleMap : public QObject
     QItemSelectionModel *placemarkSelectionModel() const;
 
     /**
-     * @brief  Return how much the map will move if one of the move slots are called.
-     * @return The move step.
-     */
-    qreal  moveStep();
-
-    /**
-     * @brief  Add a GeoDataPlacemark file to the model.
+     * @brief  Add a GeoData file to the model. Supported file types are .pnt, .gpx and .kml
      * @param  filename  the filename of the file containing the Placemarks.
+     * @deprecated This method has been renamed addGeoDataFile
      */
-    void addPlacemarkFile( const QString &filename );
+    MARBLE_DEPRECATED( void addPlacemarkFile( const QString &filename ) );
 
     /**
-     * @brief  Add GeoDataPlacemark data as string to the model.
+     * @brief  Add GeoData data as string to the model.
      * @param  data  the string containing the Placemarks.
      * @param key  the string needed to identify the data
+     * @deprecated This method has been renamed addGeoDataString
      */
-    void addPlacemarkData( const QString& data, const QString& key = "data" );
-    
+    MARBLE_DEPRECATED( void addPlacemarkData( const QString& data, const QString& key = "data" ) );
+
     /**
      * @brief  remove data or files from the model.
      * @param key  either the filename or the string used to identify the data in addPlacemarkFile and addPlacemarkData
+     * @deprecated This method has been renamed removeGeoData
      */
-    void removePlacemarkKey( const QString& key );
+    MARBLE_DEPRECATED( void removePlacemarkKey( const QString& key ) );
+
+    QVector<QModelIndex> whichFeatureAt( const QPoint& ) const;
 
     /**
      * @brief  Return the quaternion that specifies the rotation of the globe.
      * @return The quaternion that describes the rotation of the globe.
      */
-    Quaternion  planetAxis() const;
+    Quaternion planetAxis() const;
 
     /**
      * @brief  Return a QPixmap with the current contents of the map.
@@ -278,133 +273,135 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * @brief  Return the property value by name.
      * @return The property value (usually: visibility).
      */
-    bool  propertyValue( const QString& name) const;
+    bool propertyValue( const QString& name ) const;
 
     /**
      * @brief  Return whether the overview map is visible.
      * @return The overview map visibility.
      */
-    bool  showOverviewMap() const;
+    bool showOverviewMap() const;
 
     /**
      * @brief  Return whether the scale bar is visible.
      * @return The scale bar visibility.
      */
-    bool  showScaleBar() const;
+    bool showScaleBar() const;
 
     /**
      * @brief  Return whether the compass bar is visible.
      * @return The compass visibility.
      */
-    bool  showCompass() const;
+    bool showCompass() const;
 
     /**
      * @brief  Return whether the cloud cover is visible.
      * @return The cloud cover visibility.
      */
-    bool  showClouds() const;
+    bool showClouds() const;
 
     /**
      * @brief  Return whether the atmospheric glow is visible.
      * @return The cloud cover visibility.
      */
-    bool  showAtmosphere() const;
+    bool showAtmosphere() const;
 
     /**
      * @brief  Return whether the crosshairs are visible.
      * @return The crosshairs' visibility.
      */
-    bool  showCrosshairs() const;
+    bool showCrosshairs() const;
 
     /**
      * @brief  Return whether the coordinate grid is visible.
      * @return The coordinate grid visibility.
      */
-    bool  showGrid() const;
+    bool showGrid() const;
 
     /**
      * @brief  Return whether the place marks are visible.
      * @return The place mark visibility.
      */
-    bool  showPlaces() const;
+    bool showPlaces() const;
 
     /**
      * @brief  Return whether the city place marks are visible.
      * @return The city place mark visibility.
      */
-    bool  showCities() const;
+    bool showCities() const;
 
     /**
      * @brief  Return whether the terrain place marks are visible.
      * @return The terrain place mark visibility.
      */
-    bool  showTerrain() const;
+    bool showTerrain() const;
 
     /**
      * @brief  Return whether other places are visible.
      * @return The visibility of other places.
      */
-    bool  showOtherPlaces() const;
+    bool showOtherPlaces() const;
 
     /**
      * @brief  Return whether the relief is visible.
      * @return The relief visibility.
      */
-    bool  showRelief() const;
+    bool showRelief() const;
 
     /**
      * @brief  Return whether the elevation model is visible.
      * @return The elevation model visibility.
      */
-    bool  showElevationModel() const;
+    bool showElevationModel() const;
 
     /**
      * @brief  Return whether the ice layer is visible.
      * @return The ice layer visibility.
      */
-    bool  showIceLayer() const;
+    bool showIceLayer() const;
 
     /**
      * @brief  Return whether the borders are visible.
      * @return The border visibility.
      */
-    bool  showBorders() const;
+    bool showBorders() const;
 
     /**
      * @brief  Return whether the rivers are visible.
      * @return The rivers' visibility.
      */
-    bool  showRivers() const;
+    bool showRivers() const;
 
     /**
      * @brief  Return whether the lakes are visible.
      * @return The lakes' visibility.
      */
-    bool  showLakes() const;
+    bool showLakes() const;
 
     /**
      * @brief Return whether Gps Data is visible.
      * @return The Gps Data's visibility.
      */
-    bool  showGps() const;
+    bool showGps() const;
 
     /**
      * @brief  Return whether the frame rate gets displayed.
      * @return the frame rates visibility
      */
-    bool  showFrameRate() const;
+    bool showFrameRate() const;
+
+    bool showBackground() const;
 
     /**
      * @brief  Returns the limit in kilobytes of the persistent (on hard disc) tile cache.
      * @return the limit of persistent tile cache in kilobytes.
      */
-    quint64  persistentTileCacheLimit() const;
+    quint64 persistentTileCacheLimit() const;
 
     /**
      * @brief  Returns the limit in kilobytes of the volatile (in RAM) tile cache.
      * @return the limit of volatile tile cache in kilobytes.
      */
-    quint64  volatileTileCacheLimit() const;
+    quint64 volatileTileCacheLimit() const;
 
     /**
      * @brief  Return the sun locator object.
@@ -412,9 +409,71 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     SunLocator* sunLocator();
 
+    /**
+     * @brief Returns a list of all RenderPlugins in the model, this includes float items
+     * @return the list of RenderPlugins
+     */
     QList<RenderPlugin *> renderPlugins() const;
     QList<AbstractFloatItem *> floatItems() const;
 
+    /**
+     * @brief Returns a list of all FloatItems in the model
+     * @return the list of the floatItems
+     */
+    AbstractFloatItem * floatItem( const QString &nameId ) const;
+
+    /**
+     * @brief Returns a list of all DataPlugins on the layer
+     * @return the list of DataPlugins
+     */
+    QList<AbstractDataPlugin *> dataPlugins()  const;
+
+    /**
+     * @brief Returns all widgets of dataPlugins on the position curpos
+     */
+    QList<AbstractDataPluginItem *> whichItemAt( const QPoint& curpos ) const;
+
+    /**
+     * @brief Add a layer to be included in rendering.
+     */
+    void addLayer( LayerInterface *layer );
+
+    /**
+     * @brief Remove a layer from being included in rendering.
+     */
+    void removeLayer( LayerInterface *layer );
+
+    /**
+      * @brief Move camera to the given position. This can change
+      * both the zoom value and the position
+      */
+    void flyTo( const GeoDataLookAt &lookAt );
+
+    /**
+      * @brief Return the current camera position
+      */
+    GeoDataLookAt lookAt() const;
+
+    /**
+      * @brief Return the globe radius (pixel) for the given distance (km)
+      */
+    qreal radiusFromDistance( qreal distance ) const;
+
+    /**
+      * @brief Return the distance (km) at the given globe radius (pixel)
+      */
+    qreal distanceFromRadius( qreal radius ) const;
+
+    /**
+      * Returns the zoom value (no unit) corresponding to the given camera distance (km)
+      */
+    qreal zoomFromDistance( qreal distance ) const;
+
+    /**
+      * Returns the distance (km) corresponding to the given zoom value
+      */
+    qreal distanceFromZoom( qreal zoom ) const;
+
  public Q_SLOTS:
 
     void updateSun();
@@ -425,7 +484,7 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * @param painter  The painter to use.
      * @param dirtyRect the rectangle that actually needs repainting.
      */
-    void  paint( GeoPainter &painter, QRect &dirtyRect);
+    void paint( GeoPainter &painter, QRect &dirtyRect );
 
     /**
      * @brief  Zoom the view to a certain zoomlevel
@@ -435,22 +494,13 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * interpretation.  A zoom value around 1000 lets the viewer see
      * all of the earth in the default window.
      */
-    void  zoomView(int zoom);
+    void zoomView( int zoom );
 
     /**
      * @brief  Zoom the view by a certain step
      * @param  zoomStep  the difference between the old zoom and the new
      */
-    void  zoomViewBy(int zoomStep);
-
-    /**
-     * @brief  Zoom in by the amount zoomStep.
-     */
-    void  zoomIn();
-    /**
-     * @brief  Zoom out by the amount zoomStep.
-     */
-    void  zoomOut();
+    void zoomViewBy( int zoomStep );
 
     /**
      * @brief  Rotate the view by the two angles phi and theta.
@@ -463,13 +513,13 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * of (lon, lat), otherwise the resulting angle will be the sum of
      * the previous position and the two offsets.
      */
-    void  rotateBy( const qreal &deltaLon, const qreal &deltaLat );
+    void rotateBy( const qreal &deltaLon, const qreal &deltaLat );
 
     /**
      * @brief  Rotate the view by the angle specified by a Quaternion.
      * @param  incRot a quaternion specifying the rotation
      */
-    void  rotateBy( const Quaternion& incRot );
+    void rotateBy( const Quaternion& incRot );
 
     /**
      * @brief  Center the view on a geographical point
@@ -478,23 +528,23 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * @param  lon  an angle parallel to the longitude lines
      *              +180(W) - -180(E)
      */
-    void  centerOn( const qreal &lon, const qreal &lat );
+    void centerOn( const qreal lon, const qreal lat );
 
     /**
      * @brief  Center the view on a point
      * @param  index  an index for a QModel, indicating a city
      */
-    void  centerOn( const QModelIndex& index );
+    void centerOn( const QModelIndex& index );
 
     /**
      * @brief  Set the latitude for the center point
-     * @param  lat  the new value for the latitude
+     * @param  lat  the new value for the latitude in degree
      */
     void setCenterLatitude( qreal lat );
 
     /**
      * @brief  Set the longitude for the center point
-     * @param  lon  the new value for the longitude
+     * @param  lon  the new value for the longitude in degree
      */
     void setCenterLongitude( qreal lon );
 
@@ -504,56 +554,13 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * @return @c Equirectangular   a flat map
      * @return @c Mercator          another flat map
      */
-    Projection  projection() const;
+    Projection projection() const;
 
     /**
      * @brief  Set the Projection used for the map
      * @param  projection projection type (e.g. Spherical, Equirectangular, Mercator)
      */
-    void  setProjection( Projection projection );
-
-    /**
-     * @brief  get the home point
-     * @param  lon  the longitude of the home point.
-     * @param  lat  the latitude of the home point.
-     * @param  zoom the default zoom level of the home point.
-     */
-    void  home(qreal &lon, qreal &lat, int& zoom);
-    /**
-     * @brief  Set the home point
-     * @param  lon  the longitude of the new home point.
-     * @param  lat  the latitude of the new home point.
-     * @param  zoom the default zoom level for the new home point.
-     */
-    void  setHome(qreal lon, qreal lat, int zoom = 1050);
-    /**
-     * @brief  Set the home point
-     * @param  homePoint  the new home point.
-     * @param  zoom       the default zoom level for the new home point.
-     */
-    void  setHome(const GeoDataCoordinates& homePoint, int zoom = 1050);
-
-    /**
-     * @brief  Move left by the moveStep.
-     */
-    void  moveLeft();
-    /**
-     * @brief  Move right by the moveStep.
-     */
-    void  moveRight();
-    /**
-     * @brief  Move up by the moveStep.
-     */
-    void  moveUp();
-    /**
-     * @brief  Move down by the moveStep.
-     */
-    void  moveDown();
-
-    /**
-     * @brief Center the view on the default start point with the default zoom.
-     */
-    void  goHome();
+    void setProjection( Projection projection );
 
     /**
      * @brief Get the ID of the current map theme
@@ -574,7 +581,7 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * Example: 
      *    maptheme = "earth/bluemarble/bluemarble.dgml" 
      */
-    void  setMapThemeId( const QString& maptheme );
+    void setMapThemeId( const QString& maptheme );
 
     /**
      * @brief  Sets the value of a map theme property
@@ -707,32 +714,37 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     void setShowFrameRate( bool visible );
 
-    /**
-     * @brief Set the current Gps position
-     * @param lat the new latitude value
-     * @param lon the new longitude value
-     */
-    void changeCurrentPosition( qreal lon, qreal lat );
+    void setShowBackground( bool visible );
 
      /**
      * @brief used to notify about the position of the mouse click
       */
     void notifyMouseClick( int x, int y );
 
-    /**
-     * @brief updates the gps tracking point by polling
-     */
-    void updateGps();
-
     /**
      * @brief Opens a gpx file for viewing on the Marble Map
+     * @deprecated Please use addGeoDataFile instead
      */
-    void openGpxFile( QString &filename );
+    void openGpxFile( const QString &filename );
 
     /**
-     * @brief Return a QAbstractItemModel containing GPX files.
+     * @brief  Add a GeoData file to the model. Supported file types are .pnt, .gpx and .kml
+     * @param  filename  the filename of the file containing the data to be loaded.
      */
-    GpxFileModel  *gpxFileModel();
+    void addGeoDataFile( const QString &filename );
+
+    /**
+     * @brief  Add GeoData data as string to the model.
+     * @param  data  the string containing the Placemarks.
+     * @param key  the string needed to identify the data
+     */
+    void addGeoDataString( const QString& data, const QString& key = "data" );
+
+    /**
+     * @brief  remove data or files from the model.
+     * @param key  either the filename or the string used to identify the data in addGeoDataFile and addGeoDataString
+     */
+    void removeGeoData( const QString& key );
 
     /**
      * @brief Return a QAbstractItemModel containing files.
@@ -753,18 +765,22 @@ class MARBLE_EXPORT MarbleMap : public QObject
      */
     void setVolatileTileCacheLimit( quint64 kiloBytes );
 
-    /**
-     * @brief Update the map because the model changed.
-     */
-    void updateChangedMap();
-
     bool mapCoversViewport();
 
-    Marble::AngleUnit defaultAngleUnit() const;
-    void setDefaultAngleUnit( Marble::AngleUnit angleUnit );
+    AngleUnit defaultAngleUnit() const;
+    void setDefaultAngleUnit( AngleUnit angleUnit );
 
     QFont defaultFont() const;
     void setDefaultFont( const QFont& font );
+    
+    /**
+     * @brief Reload the currently displayed map by reloading texture tiles
+     *        from the Internet. In the future this should be extended to all
+     *        kinds of data which is used in the map.
+     */
+    void reload() const;
+
+    void downloadRegion( QString const & sourceDir, QVector<TileCoordsPyramid> const & );
 
  Q_SIGNALS:
     /**
@@ -772,24 +788,47 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * @param zoom  The new zoom value.
      * @see  zoomView()
      */
-    void  zoomChanged( int zoom );
-    void  distanceChanged( const QString& distanceString );
+    void zoomChanged( int zoom );
+    void distanceChanged( const QString& distanceString );
+
+    void tileLevelChanged( int level );
 
     /**
      * @brief Signal that the theme has changed
      * @param theme  Name of the new theme.
      */
-    void  themeChanged( QString theme );
+    void themeChanged( const QString& theme );
 
-    void  projectionChanged( Projection );
+    void projectionChanged( Projection );
 
-    void  mouseMoveGeoPosition( QString );
+    void mouseMoveGeoPosition( const QString& );
 
-    void  mouseClickGeoPosition( qreal lon, qreal lat, GeoDataCoordinates::Unit );
+    void mouseClickGeoPosition( qreal lon, qreal lat, GeoDataCoordinates::Unit );
 
-    void  timeout();
+    void framesPerSecond( qreal fps );
 
-    void  framesPerSecond( qreal fps );
+    /**
+     * This signal is emitted when the repaint of the view was requested.
+     * If available with the @p dirtyRegion which is the region the view will change in.
+     * If dirtyRegion.isEmpty() returns true, the whole viewport has to be repainted.
+     */
+    void repaintNeeded( const QRegion& dirtyRegion = QRegion() );
+
+    /**
+     * This signal is emitted when the visible region of the map changes. This typically happens
+     * when the user moves the map around or zooms.
+     */
+    void visibleLatLonAltBoxChanged( const GeoDataLatLonAltBox& visibleLatLonAltBox );
+
+    /**
+     * @brief This signal is emit when the settings of a plugin changed.
+     */
+    void pluginSettingsChanged();
+
+    /**
+     * @brief Signal that a render item has been initialized
+     */
+    void renderPluginInitialized( RenderPlugin *renderPlugin );
 
  protected:
 
@@ -798,13 +837,17 @@ class MARBLE_EXPORT MarbleMap : public QObject
      * @brief the globe and before all other layers have been rendered.
      * @param painter 
      */
-    virtual void  customPaint( GeoPainter *painter );
+    virtual void customPaint( GeoPainter *painter );
 
  private:
     Q_DISABLE_COPY( MarbleMap )
-    MarbleMapPrivate  * const d;
+    MarbleMapPrivate * const d;
+
+    TextureLayer *textureLayer();
+
+    MeasureTool *measureTool();
 };
 
 }
 
-#endif // MARBLEMAP_H
+#endif
