@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -19,6 +19,7 @@
 
 // Marble
 #include "MarbleDebug.h"
+#include "GeoDataExtendedData.h"
 #include "GeoDataStyle.h"       // In geodata/data/
 
 using namespace Marble;
@@ -37,7 +38,7 @@ class MarblePlacemarkModel::Private
     }
 
     int m_size;
-    QVector<GeoDataPlacemark>     *m_placemarkContainer;
+    QVector<GeoDataPlacemark*>     *m_placemarkContainer;
 };
 
 
@@ -55,7 +56,7 @@ MarblePlacemarkModel::~MarblePlacemarkModel()
     delete d;
 }
 
-void MarblePlacemarkModel::setPlacemarkContainer( QVector<GeoDataPlacemark> *container )
+void MarblePlacemarkModel::setPlacemarkContainer( QVector<GeoDataPlacemark*> *container )
 {
     d->m_placemarkContainer = container;
 }
@@ -85,35 +86,41 @@ QVariant MarblePlacemarkModel::data( const QModelIndex &index, int role ) const
         return QVariant();
 
     if ( role == Qt::DisplayRole ) {
-        return d->m_placemarkContainer->at( index.row() ).name();
+        return d->m_placemarkContainer->at( index.row() )->name();
     } else if ( role == Qt::DecorationRole ) {
-          return qVariantFromValue( d->m_placemarkContainer->at( index.row() ).style()->iconStyle().icon() );
+          return qVariantFromValue( d->m_placemarkContainer->at( index.row() )->style()->iconStyle().icon() );
     } else if ( role == PopularityIndexRole ) {
-        return d->m_placemarkContainer->at( index.row() ).popularityIndex();
+        return d->m_placemarkContainer->at( index.row() )->popularityIndex();
     } else if ( role == VisualCategoryRole ) {
-        return d->m_placemarkContainer->at( index.row() ).visualCategory();
+        return d->m_placemarkContainer->at( index.row() )->visualCategory();
     } else if ( role == AreaRole ) {
-        return d->m_placemarkContainer->at( index.row() ).area();
+        return d->m_placemarkContainer->at( index.row() )->area();
     } else if ( role == PopulationRole ) {
-        return d->m_placemarkContainer->at( index.row() ).population();
+        return d->m_placemarkContainer->at( index.row() )->population();
     } else if ( role == CountryCodeRole ) {
-        return d->m_placemarkContainer->at( index.row() ).countryCode();
+        return d->m_placemarkContainer->at( index.row() )->countryCode();
+    } else if ( role == StateRole ) {
+        return d->m_placemarkContainer->at( index.row() )->state();
     } else if ( role == PopularityRole ) {
-        return d->m_placemarkContainer->at( index.row() ).popularity();
+        return d->m_placemarkContainer->at( index.row() )->popularity();
     } else if ( role == DescriptionRole ) {
-        return d->m_placemarkContainer->at( index.row() ).description();
+        return d->m_placemarkContainer->at( index.row() )->description();
     } else if ( role == Qt::ToolTipRole ) {
-        return d->m_placemarkContainer->at( index.row() ).description();
+        return d->m_placemarkContainer->at( index.row() )->description();
     } else if ( role == GeoTypeRole ) {
-        return d->m_placemarkContainer->at( index.row() ).role();
+        return d->m_placemarkContainer->at( index.row() )->role();
     } else if ( role == CoordinateRole ) {
-        return qVariantFromValue( d->m_placemarkContainer->at( index.row() ).coordinate() );
+        return qVariantFromValue( d->m_placemarkContainer->at( index.row() )->coordinate() );
     } else if ( role == StyleRole ) {
-        return qVariantFromValue( d->m_placemarkContainer->at( index.row() ).style() );
+        return qVariantFromValue( d->m_placemarkContainer->at( index.row() )->style() );
+    } else if ( role == GmtRole ) {
+        return qVariantFromValue( d->m_placemarkContainer->at( index.row() )->extendedData().value("gmt").value() );
+    } else if ( role == DstRole ) {
+        return qVariantFromValue( d->m_placemarkContainer->at( index.row() )->extendedData().value("dst").value() );
     } else if ( role == GeometryRole ) {
-        return qVariantFromValue( *d->m_placemarkContainer->at( index.row() ).geometry() );
-/*    } else if ( role == ObjectPointerRole ) {
-        return qVariantFromValue( dynamic_cast<GeoDataObject*>( d->m_placemarkContainer->at( index.row() ) ) );*/
+        return qVariantFromValue( d->m_placemarkContainer->at( index.row() )->geometry() );
+    } else if ( role == ObjectPointerRole ) {
+        return qVariantFromValue( dynamic_cast<GeoDataObject*>( d->m_placemarkContainer->at( index.row() ) ) );
     } else
         return QVariant();
 }
@@ -182,6 +189,7 @@ void  MarblePlacemarkModel::removePlacemarks( const QString &containerName,
     beginRemoveRows( QModelIndex(), start, start + length );
     d->m_size -= length;
     endRemoveRows();
+    emit layoutChanged();
     mDebug() << "removePlacemarks(" << containerName << "): Time elapsed:" << t.elapsed() << "ms for" << length << "Placemarks.";
 }
 
