@@ -1,17 +1,16 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
 // the source code.
 //
-// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>"
-// Copyright 2007      Inge Wallin  <ingwa@kde.org>"
+// Copyright 2006-2007 Torsten Rahn <tackat@kde.org>
+// Copyright 2007      Inge Wallin  <ingwa@kde.org>
 //
 
-
-#ifndef MARBLESEARCHLISTVIEW_H
-#define MARBLESEARCHLISTVIEW_H
+#ifndef MARBLE_MARBLESEARCHLISTVIEW_H
+#define MARBLE_MARBLESEARCHLISTVIEW_H
 
 
 #include <QtGui/QListView>
@@ -46,4 +45,4 @@ class MARBLE_EXPORT MarbleSearchListView : public QListView
 
 }
 
-#endif // MARBLESEARCHLISTVIEW_H
+#endif
