@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -25,7 +25,7 @@
 #include <QtGui/QToolTip>
 
 #if QT_VERSION >= 0x40600
-  #include <QTouchEvent>
+  #include <QtGui/QTouchEvent>
 #endif
 
 #include "global.h"
@@ -35,7 +35,6 @@
 #include "MarbleWidget.h"
 #include "MarbleMap.h"
 #include "MarbleModel.h"
-#include "ViewParams.h"
 #include "ViewportParams.h"
 #include "AbstractFloatItem.h"
 #include "AbstractDataPluginItem.h"
@@ -44,7 +43,7 @@
 #include "AbstractProjection.h"
 #include "Planet.h"
 
-using namespace Marble;
+namespace Marble {
 
 const int TOOLTIP_START_INTERVAL = 1000;
 
@@ -158,16 +157,25 @@ class MarbleWidgetDefaultInputHandler::Private
     QPixmap m_curpmbc;
     QPixmap m_curpmbr;
 
-    QCursor m_arrowcur[3][3];
+    QCursor m_arrowCur[3][3];
 
-    bool m_leftpressed;
-    int m_leftpresseddirection;
-    bool m_midpressed;
-    int m_leftpressedx;
-    int m_leftpressedy;
-    int m_midpressedy;
-    qreal m_leftpresseda;
-    qreal m_leftpressedb;
+    // Indicates if the left mouse button has been pressed already.
+    bool m_leftPressed;
+    // Indicates whether the drag was started by a click above or below the visible pole.
+    int m_leftPressedDirection;
+    // Indicates if the middle mouse button has been pressed already.
+    bool m_midPressed;
+    // The mouse pointer x position when the left mouse button has been pressed.
+    int m_leftPressedX;
+    // The mouse pointer y position when the left mouse button has been pressed.
+    int m_leftPressedY;
+    // The mouse pointer y position when the middle mouse button has been pressed.
+    int m_midPressedY;
+    int m_radiusWhenPressed;
+    // The center longitude in radian when the left mouse button has been pressed.
+    qreal m_leftPressedLon;
+    // The center latitude in radian when the left mouse button has been pressed.
+    qreal m_leftPressedLat;
 
     int m_dragThreshold;
     QTimer m_lmbTimer;
@@ -187,8 +195,8 @@ class MarbleWidgetDefaultInputHandler::Private
 };
 
 MarbleWidgetDefaultInputHandler::Private::Private()
-    : m_leftpressed( false ),
-      m_midpressed( false ),
+    : m_leftPressed( false ),
+      m_midPressed( false ),
       m_dragThreshold( 3 ),
       m_popupmenu( 0 )
 {
@@ -201,15 +209,15 @@ MarbleWidgetDefaultInputHandler::Private::Private()
     m_curpmbc.load( MarbleDirs::path("bitmaps/cursor_bc.xpm") );
     m_curpmbr.load( MarbleDirs::path("bitmaps/cursor_br.xpm") );
 
-    m_arrowcur[0][0] = QCursor( m_curpmtl, 2, 2 );
-    m_arrowcur[1][0] = QCursor( m_curpmtc, 10, 3 );
-    m_arrowcur[2][0] = QCursor( m_curpmtr, 19, 2 );
-    m_arrowcur[0][1] = QCursor( m_curpmcl, 3, 10 );
-    m_arrowcur[1][1] = QCursor( Qt::OpenHandCursor );
-    m_arrowcur[2][1] = QCursor( m_curpmcr, 18, 10 );
-    m_arrowcur[0][2] = QCursor( m_curpmbl, 2, 19 );
-    m_arrowcur[1][2] = QCursor( m_curpmbc, 11, 18 );
-    m_arrowcur[2][2] = QCursor( m_curpmbr, 19, 19 );
+    m_arrowCur[0][0] = QCursor( m_curpmtl, 2, 2 );
+    m_arrowCur[1][0] = QCursor( m_curpmtc, 10, 3 );
+    m_arrowCur[2][0] = QCursor( m_curpmtr, 19, 2 );
+    m_arrowCur[0][1] = QCursor( m_curpmcl, 3, 10 );
+    m_arrowCur[1][1] = QCursor( Qt::OpenHandCursor );
+    m_arrowCur[2][1] = QCursor( m_curpmcr, 18, 10 );
+    m_arrowCur[0][2] = QCursor( m_curpmbl, 2, 19 );
+    m_arrowCur[1][2] = QCursor( m_curpmbc, 11, 18 );
+    m_arrowCur[2][2] = QCursor( m_curpmbr, 19, 19 );
 }
 
 MarbleWidgetDefaultInputHandler::Private::~Private()
@@ -227,7 +235,7 @@ void MarbleWidgetDefaultInputHandler::Private::ZoomAt(MarbleWidget* marbleWidget
         return;
     }
 
-    ViewportParams* now = marbleWidget->map()->viewParams()->viewport();
+    ViewportParams* now = marbleWidget->viewport();
 
     qreal x(0), y(0);
     if (!now->currentProjection()->screenCoordinates(destLon, destLat, now, x, y)) {
@@ -258,9 +266,8 @@ void MarbleWidgetDefaultInputHandler::Private::ZoomAt(MarbleWidget* marbleWidget
     lookAt.setLatitude( lat );
     lookAt.setAltitude( 0.0 );
     lookAt.setRange( newDistance * KM2METER );
-    
-    marbleWidget->map()->viewParams()->viewport()->setFocusPoint( GeoDataCoordinates( destLon,
-                                                                                      destLat ) );
+
+    marbleWidget->viewport()->setFocusPoint( GeoDataCoordinates( destLon, destLat ) );
     marbleWidget->flyTo( lookAt, Linear );
 }
 
@@ -272,7 +279,7 @@ void MarbleWidgetDefaultInputHandler::Private::MoveTo( MarbleWidget* marbleWidge
 
     qreal  destLat;
     qreal  destLon;
-    qreal distance = marbleWidget->map()->distance();
+    qreal distance = marbleWidget->distance();
     marbleWidget->geoCoordinates( pos.x(), pos.y(), destLon, destLat, GeoDataCoordinates::Radian );
     
     GeoDataLookAt lookAt;
@@ -308,7 +315,7 @@ MarbleWidgetDefaultInputHandler::~MarbleWidgetDefaultInputHandler()
 void MarbleWidgetDefaultInputHandler::lmbTimeout()
 {
     if (!d->m_selectionRubber->isVisible()) {
-        emit lmbRequest( d->m_leftpressedx, d->m_leftpressedy );
+        emit lmbRequest( d->m_leftPressedX, d->m_leftPressedY );
     }
 }
 
@@ -319,13 +326,7 @@ void MarbleWidgetInputHandler::restoreViewContext()
 
     // Redraw the map with the quality set for Still (if necessary).
     d->m_widget->setViewContext( Still );
-    if ( d->m_widget->mapQuality( Still )
-        != d->m_widget->mapQuality( Animation ) )
-    {
-        d->m_widget->updateChangedMap();
-    }
-
-    d->m_widget->map()->viewParams()->viewport()->resetFocusPoint();
+    d->m_widget->viewport()->resetFocusPoint();
     d->m_wheelZoomTargetDistance = 0.0;
 }
 
@@ -340,7 +341,7 @@ void MarbleWidgetDefaultInputHandler::init( MarbleWidget *w )
 
     // The interface to the measure tool consists of a RMB popup menu
     // and some signals.
-    MeasureTool *measureTool = MarbleWidgetInputHandler::d->m_widget->map()->measureTool();
+    MeasureTool *measureTool = MarbleWidgetInputHandler::d->m_widget->measureTool();
 
     // Connect the inputHandler and the measure tool to the popup menu
     if ( !d->m_popupmenu ) {
@@ -424,7 +425,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         return true;
     }
 
-    int polarity = MarbleWidgetInputHandler::d->m_widget->map()->viewParams()->viewport()->polarity();
+    int polarity = MarbleWidgetInputHandler::d->m_widget->viewport()->polarity();
 
     if ( e->type() == QEvent::MouseMove
          || e->type() == QEvent::MouseButtonPress
@@ -440,12 +441,12 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         if ( event->type() == QEvent::MouseMove
              && !( event->buttons() & Qt::LeftButton ) )
         {
-            d->m_leftpressed = false;
+            d->m_leftPressed = false;
         }
         if ( event->type() == QEvent::MouseMove
              && !( event->buttons() & Qt::MidButton ) )
         {
-            d->m_midpressed = false;
+            d->m_midPressed = false;
         }
 
         // Do not handle (and therefore eat) mouse press and release events
@@ -491,37 +492,39 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 if (isMouseButtonPopupEnabled(Qt::LeftButton))
                     d->m_lmbTimer.start(400);
 
-                d->m_leftpressed = true;
-                d->m_midpressed = false;
+                d->m_leftPressed = true;
+                d->m_midPressed = false;
                 d->m_selectionRubber->hide();
 
                 // On the single event of a mouse button press these
                 // values get stored, to enable us to e.g. calculate the
                 // distance of a mouse drag while the mouse button is
                 // still down.
-                d->m_leftpressedx = event->x();
-                d->m_leftpressedy = event->y();
+                d->m_leftPressedX = event->x();
+                d->m_leftPressedY = event->y();
 
                 // Calculate translation of center point
-                d->m_leftpresseda = MarbleWidgetInputHandler::d->m_widget->centerLongitude() * DEG2RAD;
-                d->m_leftpressedb = MarbleWidgetInputHandler::d->m_widget->centerLatitude() * DEG2RAD;
+                d->m_leftPressedLon = MarbleWidgetInputHandler::d->m_widget->centerLongitude() * DEG2RAD;
+                d->m_leftPressedLat = MarbleWidgetInputHandler::d->m_widget->centerLatitude() * DEG2RAD;
 
-                d->m_leftpresseddirection = 1;
+                d->m_leftPressedDirection = 1;
 
                 // Choose spin direction by taking into account whether we
                 // drag above or below the visible pole.
                 if ( MarbleWidgetInputHandler::d->m_widget->projection() == Spherical ) {
-                    if ( d->m_leftpressedb >= 0 ) {
+                    if ( d->m_leftPressedLat >= 0 ) {
                         // The visible pole is the north pole
-                        if ( event->y() < ( - MarbleWidgetInputHandler::d->m_widget->northPoleY()
-                                            + MarbleWidgetInputHandler::d->m_widget->height() / 2 ) )
-                            d->m_leftpresseddirection = -1;
+                        qreal northPoleX, northPoleY;
+                        MarbleWidgetInputHandler::d->m_widget->screenCoordinates( 0.0, 90.0, northPoleX, northPoleY );
+                        if ( event->y() < northPoleY )
+                            d->m_leftPressedDirection = -1;
                     }
                     else {
                         // The visible pole is the south pole
-                        if ( event->y() > ( + MarbleWidgetInputHandler::d->m_widget->northPoleY()
-                                            + MarbleWidgetInputHandler::d->m_widget->height() / 2 ) )
-                            d->m_leftpresseddirection = -1;
+                        qreal southPoleX, southPoleY;
+                        MarbleWidgetInputHandler::d->m_widget->screenCoordinates( 0.0, -90.0, southPoleX, southPoleY );
+                        if ( event->y() > southPoleY )
+                            d->m_leftPressedDirection = -1;
                     }
                 }
 
@@ -530,9 +533,10 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
             if ( e->type() == QEvent::MouseButtonPress
                  && event->button() == Qt::MidButton ) {
-                d->m_midpressed = true;
-                d->m_leftpressed = false;
-                d->m_midpressedy = event->y();
+                d->m_midPressed = true;
+                d->m_leftPressed = false;
+                d->m_radiusWhenPressed = MarbleWidgetInputHandler::d->m_widget->radius();
+                d->m_midPressedY = event->y();
 
                 d->m_selectionRubber->hide();
                 MarbleWidgetInputHandler::d->m_widget->setViewContext( Animation );
@@ -561,28 +565,18 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
                 //emit current coordinates to be be interpreted
                 //as requested
-                emit mouseClickScreenPosition( d->m_leftpressedx, d->m_leftpressedy );
+                emit mouseClickScreenPosition( d->m_leftPressedX, d->m_leftPressedY );
 
                 MarbleWidgetInputHandler::d->m_widget->setViewContext( Still );
-                if ( MarbleWidgetInputHandler::d->m_widget->mapQuality( Still )
-                    != MarbleWidgetInputHandler::d->m_widget->mapQuality( Animation ) )
-                {
-                    MarbleWidgetInputHandler::d->m_widget->updateChangedMap();
-                }
 
-                d->m_leftpressed = false;
+                d->m_leftPressed = false;
             }
 
             if ( e->type() == QEvent::MouseButtonRelease
                  && event->button() == Qt::MidButton ) {
-                d->m_midpressed = false;
+                d->m_midPressed = false;
 
                 MarbleWidgetInputHandler::d->m_widget->setViewContext( Still );
-                if ( MarbleWidgetInputHandler::d->m_widget->mapQuality( Still )
-                    != MarbleWidgetInputHandler::d->m_widget->mapQuality( Animation ) )
-                {
-                    MarbleWidgetInputHandler::d->m_widget->updateChangedMap();
-                }
             }
 
             if ( e->type() == QEvent::MouseButtonRelease
@@ -601,28 +595,27 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             }
 
             // Regarding all kinds of mouse moves:
-            if ( d->m_leftpressed && !d->m_selectionRubber->isVisible() ) {
+            if ( d->m_leftPressed && !d->m_selectionRubber->isVisible() ) {
                 qreal radius = ( qreal )( MarbleWidgetInputHandler::d->m_widget->radius() );
-                int deltax = event->x() - d->m_leftpressedx;
-                int deltay = event->y() - d->m_leftpressedy;
+                int deltax = event->x() - d->m_leftPressedX;
+                int deltay = event->y() - d->m_leftPressedY;
 
                 if ( abs( deltax ) > d->m_dragThreshold
                      || abs( deltay ) > d->m_dragThreshold ) {
 
                     d->m_lmbTimer.stop();
-                    MarbleWidgetInputHandler::d->m_widget->centerOn( RAD2DEG * ( qreal )( d->m_leftpresseda )
-                                                                     - 90.0 * d->m_leftpresseddirection * deltax / radius,
-                                                                     RAD2DEG * ( qreal )( d->m_leftpressedb )
+                    MarbleWidgetInputHandler::d->m_widget->centerOn( RAD2DEG * ( qreal )( d->m_leftPressedLon )
+                                                                     - 90.0 * d->m_leftPressedDirection * deltax / radius,
+                                                                     RAD2DEG * ( qreal )( d->m_leftPressedLat )
                                                                      + 90.0 * deltay / radius );
                 }
             }
 
 
-            if ( d->m_midpressed ) {
+            if ( d->m_midPressed ) {
                 int eventy = event->y();
-                int dy = d->m_midpressedy - eventy;
-                d->m_midpressed = eventy;
-                MarbleWidgetInputHandler::d->m_widget->zoomViewBy( (int)( 2 * dy / 3 ) );
+                int dy = d->m_midPressedY - eventy;
+                MarbleWidgetInputHandler::d->m_widget->setRadius( d->m_radiusWhenPressed * pow( 1.005, dy ) );
             }
 
             if ( d->m_selectionRubber->isVisible() ) 
@@ -633,7 +626,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             }
         }
         else {
-            d->m_leftpressed = false;
+            d->m_leftPressed = false;
 
             QRect boundingRect = MarbleWidgetInputHandler::d->m_widget->mapRegion().boundingRect();
 
@@ -674,7 +667,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         
         // Find out if there are data items and if one has defined an action
         QList<AbstractDataPluginItem *> dataItems
-            = MarbleWidgetInputHandler::d->m_widget->model()->whichItemAt( mousePosition );
+            = MarbleWidgetInputHandler::d->m_widget->whichItemAt( mousePosition );
         bool dataAction = false;
         QPointer<AbstractDataPluginItem> toolTipItem;
         QList<AbstractDataPluginItem *>::iterator it = dataItems.begin();
@@ -705,20 +698,20 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             d->m_toolTipPosition = mousePosition;
         }
 
-        if ( ( MarbleWidgetInputHandler::d->m_widget->model()->whichFeatureAt( mousePosition ).size() == 0 )
+        if ( ( MarbleWidgetInputHandler::d->m_widget->whichFeatureAt( mousePosition ).size() == 0 )
              && ( !dataAction ) )
         {
-            if ( !d->m_leftpressed )
-                d->m_arrowcur [1][1] = QCursor( Qt::OpenHandCursor );
+            if ( !d->m_leftPressed )
+                d->m_arrowCur [1][1] = QCursor( Qt::OpenHandCursor );
             else
-                d->m_arrowcur [1][1] = QCursor( Qt::ClosedHandCursor );
+                d->m_arrowCur [1][1] = QCursor( Qt::ClosedHandCursor );
         }
         else {
-            if ( !d->m_leftpressed )
-                d->m_arrowcur [1][1] = QCursor( Qt::PointingHandCursor );
+            if ( !d->m_leftPressed )
+                d->m_arrowCur [1][1] = QCursor( Qt::PointingHandCursor );
         }
 
-        MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowcur[dirX+1][dirY+1] );
+        MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowCur[dirX+1][dirY+1] );
 
         // let others, especially float items, still process the event
         // Mouse move events need to be eaten to avoid the default oxygen behavior of
@@ -734,7 +727,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             marbleWidget->setViewContext( Animation );
 
             int steps = wheelevt->delta() / 3;
-            qreal zoom = marbleWidget->map()->zoom();
+            qreal zoom = marbleWidget->zoom();
             qreal target = MarbleWidgetInputHandler::d->m_wheelZoomTargetDistance;
             if ( marbleWidget->animationsEnabled() && target > 0.0 ) {
                 // Do not use intermediate (interpolated) distance values caused by animations
@@ -777,7 +770,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                              destLon, destLat, GeoDataCoordinates::Radian );
 
                 if (isValid) {
-                    marbleWidget->map()->viewParams()->viewport()->setFocusPoint(GeoDataCoordinates(destLon, destLat));
+                    marbleWidget->viewport()->setFocusPoint(GeoDataCoordinates(destLon, destLat));
                 }
 
                 //convert the scaleFactor to be 0: the same: < 0: smaller, > 0: bigger and make it bigger by multiplying for an arbitrary big value
@@ -838,7 +831,7 @@ void MarbleWidgetDefaultInputHandler::addMeasurePoint()
     qreal  lon;
 
     MarbleWidgetInputHandler::d->m_widget->geoCoordinates( p.x(), p.y(), lon, lat, GeoDataCoordinates::Radian );
-    MeasureTool *measureTool = MarbleWidgetInputHandler::d->m_widget->map()->measureTool();
+    MeasureTool *measureTool = MarbleWidgetInputHandler::d->m_widget->measureTool();
     measureTool->addMeasurePoint( lon, lat );
 }
 
@@ -849,5 +842,6 @@ void MarbleWidgetDefaultInputHandler::setNumberOfMeasurePoints( int newNumber )
     d->m_removeLastMeasurePointAction->setEnabled(enableMeasureActions);
 }
 
+}
 
 #include "MarbleWidgetInputHandler.moc"
