@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -36,20 +36,31 @@ class MARBLE_EXPORT PositionProviderPlugin : public QObject, public PositionProv
      */
     virtual PositionProviderPlugin * newInstance() const = 0;
 
-    
+    /**
+      * Returns the speed of the gps device in metres per second
+      */
+    virtual qreal speed() const;
+
+    /**
+      * Returns the direction in which the gps device is moving.
+      * The direction is an angle(in degrees) clockwise from geographical north
+      */
+    virtual qreal direction() const;
+
+
  signals:
-    void statusChanged(Marble::PositionProviderStatus status) const;
-    void positionChanged(Marble::GeoDataCoordinates position, Marble::GeoDataAccuracy accuracy) const;
+    void statusChanged( PositionProviderStatus status ) const;
+    void positionChanged( GeoDataCoordinates position,
+                          GeoDataAccuracy accuracy ) const;
 
  protected:
     PositionProviderPlugin();
 
  private:
     Q_DISABLE_COPY( PositionProviderPlugin )
+
 };
 
 }
 
-
-
 #endif
