@@ -1,5 +1,5 @@
 /**
- * This file is part of the Marble Desktop Globe.
+ * This file is part of the Marble Virtual Globe.
  *
  * Copyright 2005-2007 Torsten Rahn <tackat@kde.org>
  * Copyright 2007      Inge Wallin  <ingwa@kde.org>
@@ -23,18 +23,18 @@
 
 #include "StackedTileLoader.h"
 
-#include "DatasetProvider.h"
 #include "GeoSceneDocument.h"
 #include "GeoSceneGroup.h"
 #include "GeoSceneHead.h"
 #include "GeoSceneLayer.h"
 #include "GeoSceneMap.h"
+#include "GeoSceneSettings.h"
 #include "GeoSceneTexture.h"
 #include "HttpDownloadManager.h"
 #include "MapThemeManager.h"
 #include "MarbleDebug.h"
 #include "MarbleDirs.h"
-#include "MarbleModel.h"
+#include "MergedLayerDecorator.h"
 #include "StackedTile.h"
 #include "TextureTile.h"
 #include "TileLoader.h"
@@ -59,79 +59,86 @@ namespace Marble
 class StackedTileLoaderPrivate
 {
 public:
-    StackedTileLoaderPrivate()
-        : m_datasetProvider( 0 ),
-          m_mapThemeManager( 0 ),
-          m_tileLoader( 0 ),
-          m_textureLayerSettings( 0 )
+    StackedTileLoaderPrivate( TileLoader *tileLoader, SunLocator *sunLocator )
+        : m_layerDecorator( tileLoader, sunLocator ),
+          m_tileLoader( tileLoader ),
+          m_maxTileLevel( 0 )
     {
         m_tileCache.setMaxCost( 20000 * 1024 ); // Cache size measured in bytes
     }
 
-    bool isTextureLayerEnabled( QString const & name ) const;
+    void detectMaxTileLevel();
 
-    DatasetProvider *m_datasetProvider;
-    MapThemeManager const *m_mapThemeManager;
-    // TODO: comment about uint hash key
-    QHash<uint, GeoSceneLayer const *> m_sceneLayers;
-    QHash<uint, GeoSceneTexture*> m_textureLayers;
+    MergedLayerDecorator m_layerDecorator;
     TileLoader *m_tileLoader;
+    int         m_maxTileLevel;
+    QVector<GeoSceneTexture const *> m_textureLayers;
     QHash <TileId, StackedTile*>  m_tilesOnDisplay;
     QCache <TileId, StackedTile>  m_tileCache;
-    // we cannot use a const GeoSceneGroup because of QObject connects/disconnects
-    GeoSceneGroup * m_textureLayerSettings;
 };
 
-bool StackedTileLoaderPrivate::isTextureLayerEnabled( QString const & name ) const
+StackedTileLoader::StackedTileLoader( TileLoader * const tileLoader,
+                                      SunLocator * const sunLocator )
+    : d( new StackedTileLoaderPrivate( tileLoader, sunLocator ) )
 {
-    if ( !m_textureLayerSettings )
-        return true;
-    bool enabled;
-    m_textureLayerSettings->propertyValue( name, enabled );
-    return enabled;
-}
-
-StackedTileLoader::StackedTileLoader( MapThemeManager const * const mapThemeManager,
-                                      GeoSceneGroup * const textureLayerSettings,
-                                      HttpDownloadManager * const downloadManager,
-                                      MarbleModel * const model )
-    : d( new StackedTileLoaderPrivate ),
-      m_parent( model )
-{
-    d->m_mapThemeManager = mapThemeManager;
-    connect( d->m_mapThemeManager, SIGNAL( themesChanged() ),
-             this, SLOT( updateTextureLayers() ) );
-    setTextureLayerSettings( textureLayerSettings );
-    d->m_tileLoader = new TileLoader( mapThemeManager, downloadManager );
-    updateTextureLayers();
-    connect( d->m_tileLoader, SIGNAL( tileCompleted( TileId, TileId )),
-             SLOT( updateTile( TileId, TileId )));
-    setDownloadManager( downloadManager );
+    connect( d->m_tileLoader, SIGNAL( tileCompleted( TileId )),
+             SLOT( updateTile( TileId )));
 }
 
 StackedTileLoader::~StackedTileLoader()
 {
     flush();
     d->m_tileCache.clear();
-    delete d->m_tileLoader;
     delete d;
 }
 
-void StackedTileLoader::setDownloadManager( HttpDownloadManager * )
+void StackedTileLoader::setTextureLayers( QVector<GeoSceneTexture const *> & textureLayers )
 {
+    mDebug() << "StackedTileLoader::setTextureLayers";
+
+    d->m_textureLayers = textureLayers;
+
+    d->m_tilesOnDisplay.clear();
+    d->m_tileCache.clear();
+
+    d->detectMaxTileLevel();
 }
 
-void StackedTileLoader::setTextureLayerSettings( GeoSceneGroup * const textureLayerSettings )
+void StackedTileLoader::setShowTileId( bool show )
 {
-    if ( d->m_textureLayerSettings ) {
-        disconnect( d->m_textureLayerSettings, SIGNAL( valueChanged( QString, bool ) ),
-                    this,                      SLOT( reset() ) );
-    }
-    d->m_textureLayerSettings = textureLayerSettings;
-    if ( d->m_textureLayerSettings ) {
-        connect( d->m_textureLayerSettings, SIGNAL( valueChanged( QString, bool )),
-                 this,                      SLOT( reset() ) );
-    }
+    d->m_layerDecorator.setShowTileId( show );
+}
+
+int StackedTileLoader::tileColumnCount( int level ) const
+{
+    Q_ASSERT( !d->m_textureLayers.isEmpty() );
+
+    const int levelZeroColumns = d->m_textureLayers.at( 0 )->levelZeroColumns();
+
+    return TileLoaderHelper::levelToColumn( levelZeroColumns, level );
+}
+
+int StackedTileLoader::tileRowCount( int level ) const
+{
+    Q_ASSERT( !d->m_textureLayers.isEmpty() );
+
+    const int levelZeroRows = d->m_textureLayers.at( 0 )->levelZeroRows();
+
+    return TileLoaderHelper::levelToRow( levelZeroRows, level );
+}
+
+GeoSceneTexture::Projection StackedTileLoader::tileProjection() const
+{
+    Q_ASSERT( !d->m_textureLayers.isEmpty() );
+
+    return d->m_textureLayers.at( 0 )->projection();
+}
+
+QSize StackedTileLoader::tileSize() const
+{
+    Q_ASSERT( !d->m_textureLayers.isEmpty() );
+
+    return d->m_textureLayers.at( 0 )->tileSize();
 }
 
 void StackedTileLoader::resetTilehash()
@@ -175,8 +182,7 @@ void StackedTileLoader::flush()
     d->m_tilesOnDisplay.clear();
 }
 
-StackedTile* StackedTileLoader::loadTile( TileId const & stackedTileId, DownloadUsage const usage,
-                                          bool const forMergedLayerDecorator )
+StackedTile* StackedTileLoader::loadTile( TileId const & stackedTileId, DownloadUsage const usage )
 {
     // check if the tile is in the hash
     StackedTile * stackedTile = d->m_tilesOnDisplay.value( stackedTileId, 0 );
@@ -201,17 +207,12 @@ StackedTile* StackedTileLoader::loadTile( TileId const & stackedTileId, Download
         }
     }
 
-    GeoSceneTexture * const texture = d->m_textureLayers[ stackedTileId.mapThemeIdHash() ];
-
     // tile (valid) has not been found in hash or cache, so load it from disk
     // and place it in the hash from where it will get transferred to the cache
 
     // mDebug() << "load Tile from Disk: " << stackedTileId.toString();
-    stackedTile = new StackedTile( stackedTileId );
-    if ( forMergedLayerDecorator )
-        stackedTile->setForMergedLayerDecorator();
-    d->m_tilesOnDisplay[ stackedTileId ] = stackedTile;
 
+    QVector<QSharedPointer<TextureTile> > tiles;
     QVector<GeoSceneTexture const *> const textureLayers = findRelevantTextureLayers( stackedTileId );
     QVector<GeoSceneTexture const *>::const_iterator pos = textureLayers.constBegin();
     QVector<GeoSceneTexture const *>::const_iterator const end = textureLayers.constEnd();
@@ -220,20 +221,21 @@ StackedTile* StackedTileLoader::loadTile( TileId const & stackedTileId, Download
         TileId const tileId( textureLayer->sourceDir(), stackedTileId.zoomLevel(),
                              stackedTileId.x(), stackedTileId.y() );
         mDebug() << "StackedTileLoader::loadTile: tile" << textureLayer->sourceDir()
-                 << tileId.toString();
+                 << tileId.toString() << textureLayer->tileSize();
         QSharedPointer<TextureTile> const tile = d->m_tileLoader->loadTile( stackedTileId, tileId,
                                                                             usage );
         if ( tile ) {
             tile->setBlending( textureLayer->blending() );
-            stackedTile->addTile( tile );
+            tiles.append( tile );
         }
     }
-    Q_ASSERT( stackedTile->hasTiles() );
+    Q_ASSERT( !tiles.isEmpty() );
+
+    stackedTile = new StackedTile( stackedTileId, tiles );
+    d->m_tilesOnDisplay[ stackedTileId ] = stackedTile;
+    stackedTile->initResultTile();
+    mergeDecorations( stackedTile );
 
-    if ( stackedTile->state() != StackedTile::TileEmpty ) {
-        stackedTile->initResultTile();
-        mergeDecorations( stackedTile, texture );
-    }
     return stackedTile;
 }
 
@@ -262,9 +264,7 @@ StackedTile* StackedTileLoader::reloadTile( TileId const & stackedTileId,
         return cachedTile;
     }
 
-    StackedTile * const stackedTile = new StackedTile( stackedTileId );
-    d->m_tilesOnDisplay[ stackedTileId ] = stackedTile;
-
+    QVector<QSharedPointer<TextureTile> > tiles;
     QVector<GeoSceneTexture const *> const textureLayers = findRelevantTextureLayers( stackedTileId );
     QVector<GeoSceneTexture const *>::const_iterator pos = textureLayers.constBegin();
     QVector<GeoSceneTexture const *>::const_iterator const end = textureLayers.constEnd();
@@ -276,14 +276,15 @@ StackedTile* StackedTileLoader::reloadTile( TileId const & stackedTileId,
                                                                               usage );
         if ( tile ) {
             tile->setBlending( textureLayer->blending() );
-            stackedTile->addTile( tile );
+            tiles.append( tile );
         }
     }
-    Q_ASSERT( stackedTile->hasTiles() );
+    Q_ASSERT( !tiles.isEmpty() );
+
+    StackedTile * const stackedTile = new StackedTile( stackedTileId, tiles );
+    d->m_tilesOnDisplay[ stackedTileId ] = stackedTile;
+    stackedTile->initResultTile();
 
-    if ( stackedTile->state() != StackedTile::TileEmpty ) {
-        stackedTile->initResultTile();
-    }
     return stackedTile;
 }
 
@@ -318,15 +319,26 @@ QList<TileId> StackedTileLoader::tilesOnDisplay() const
     return result;
 }
 
-int StackedTileLoader::maximumTileLevel( GeoSceneTexture const * const texture )
+int StackedTileLoader::maximumTileLevel() const
 {
-    if ( !texture )
-        return -1;
+    return d->m_maxTileLevel;
+}
+
+void StackedTileLoaderPrivate::detectMaxTileLevel()
+{
+    if ( m_textureLayers.isEmpty() ) {
+        m_maxTileLevel = -1;
+        return;
+    }
+
+    GeoSceneTexture const * const texture = m_textureLayers.at( 0 );
 
     // if maximum tile level is configured in the DGML files,
     // then use it, otherwise use old detection code.
-    if ( texture->maximumTileLevel() >= 0 )
-        return texture->maximumTileLevel();
+    if ( texture->maximumTileLevel() >= 0 ) {
+        m_maxTileLevel = texture->maximumTileLevel();
+        return;
+    }
 
     int maximumTileLevel = -1;
     QString tilepath = MarbleDirs::path( texture->themeStr() );
@@ -334,19 +346,19 @@ int StackedTileLoader::maximumTileLevel( GeoSceneTexture const * const texture )
     QStringList leveldirs = QDir( tilepath ).entryList( QDir::AllDirs | QDir::NoSymLinks
                                                         | QDir::NoDotAndDotDot );
 
-    bool ok = true;
-
     QStringList::const_iterator it = leveldirs.constBegin();
     QStringList::const_iterator const end = leveldirs.constEnd();
     for (; it != end; ++it ) {
-        int value = (*it).toInt( &ok, 10 );
+        bool ok = true;
+        const int value = (*it).toInt( &ok, 10 );
+
         if ( ok && value > maximumTileLevel )
             maximumTileLevel = value;
     }
 
     //    mDebug() << "Detected maximum tile level that contains data: "
     //             << maxtilelevel;
-    return maximumTileLevel + 1;
+    m_maxTileLevel = maximumTileLevel + 1;
 }
 
 bool StackedTileLoader::baseTilesAvailable( GeoSceneLayer * layer )
@@ -374,12 +386,6 @@ bool StackedTileLoader::baseTilesAvailable( GeoSceneLayer * layer )
     return noerr;
 }
 
-void StackedTileLoader::reset()
-{
-    mDebug() << "StackedTileLoader::reset";
-    d->m_tilesOnDisplay.clear();
-    d->m_tileCache.clear();
-}
 
 void StackedTileLoader::setVolatileCacheLimit( quint64 kiloBytes )
 {
@@ -387,22 +393,22 @@ void StackedTileLoader::setVolatileCacheLimit( quint64 kiloBytes )
     d->m_tileCache.setMaxCost( kiloBytes * 1024 );
 }
 
-void StackedTileLoader::updateTile( TileId const & stackedTileId, TileId const & tileId )
+void StackedTileLoader::updateTile( TileId const & stackedTileId )
 {
-    Q_UNUSED(tileId)
+    d->detectMaxTileLevel();
+
     StackedTile * const displayedTile = d->m_tilesOnDisplay.value( stackedTileId, 0 );
     if ( displayedTile ) {
-        displayedTile->deriveCompletionState();
         displayedTile->initResultTile();
-        mergeDecorations( displayedTile, findTextureLayer( stackedTileId ));
-        emit tileUpdateAvailable();
+        mergeDecorations( displayedTile );
+        emit tileUpdateAvailable( stackedTileId );
     }
     else {
         StackedTile * const cachedTile = d->m_tileCache.object( stackedTileId );
         if ( cachedTile ) {
-            cachedTile->deriveCompletionState();
             cachedTile->initResultTile();
-            mergeDecorations( cachedTile, findTextureLayer( stackedTileId ));
+            mergeDecorations( cachedTile );
+            emit tileUpdateAvailable( stackedTileId );
         }
     }
 }
@@ -412,40 +418,16 @@ void StackedTileLoader::update()
     mDebug() << "StackedTileLoader::update()";
     flush(); // trigger a reload of all tiles that are currently in use
     d->m_tileCache.clear(); // clear the tile cache in physical memory
-    emit tileUpdateAvailable();
-}
-
-inline GeoSceneLayer const * StackedTileLoader::findSceneLayer( TileId const & stackedTileId ) const
-{
-    GeoSceneLayer const * const result = d->m_sceneLayers.value( stackedTileId.mapThemeIdHash(),
-                                                                 0 );
-    Q_ASSERT( result );
-    return result;
-}
-
-inline GeoSceneTexture const * StackedTileLoader::findTextureLayer( TileId const & id ) const
-{
-    GeoSceneTexture const * const textureLayer = d->m_textureLayers.value( id.mapThemeIdHash(), 0 );
-    Q_ASSERT( textureLayer );
-    return textureLayer;
-}
-
-inline GeoSceneTexture * StackedTileLoader::findTextureLayer( TileId const & id )
-{
-    GeoSceneTexture * const textureLayer = d->m_textureLayers.value( id.mapThemeIdHash(), 0 );
-    Q_ASSERT( textureLayer );
-    return textureLayer;
+    emit tileUpdatesAvailable();
 }
 
 // 
 QVector<GeoSceneTexture const *>
 StackedTileLoader::findRelevantTextureLayers( TileId const & stackedTileId ) const
 {
-    GeoSceneLayer const * const sceneLayer = findSceneLayer( stackedTileId );
-    QVector<GeoSceneAbstractDataset*> textureLayers = sceneLayer->datasets();
     QVector<GeoSceneTexture const *> result;
-    QVector<GeoSceneAbstractDataset*>::const_iterator pos = textureLayers.constBegin();
-    QVector<GeoSceneAbstractDataset*>::const_iterator const end = textureLayers.constEnd();
+    QVector<GeoSceneTexture const *>::const_iterator pos = d->m_textureLayers.constBegin();
+    QVector<GeoSceneTexture const *>::const_iterator const end = d->m_textureLayers.constEnd();
     for (; pos != end; ++pos ) {
         GeoSceneTexture const * const candidate = dynamic_cast<GeoSceneTexture const *>( *pos );
         // check if layer is enabled. A layer is considered to be enabled if one of the
@@ -455,8 +437,6 @@ StackedTileLoader::findRelevantTextureLayers( TileId const & stackedTileId ) con
         // 3) the layer is configured and enabled in the settings
         // also check, if layer provides tiles for the current level
         if ( candidate
-             && ( pos == textureLayers.constBegin()
-                  || d->isTextureLayerEnabled( candidate->name() ))
              && ( !candidate->hasMaximumTileLevel()
                   || stackedTileId.zoomLevel() <= candidate->maximumTileLevel() ))
             result.append( candidate );
@@ -464,60 +444,17 @@ StackedTileLoader::findRelevantTextureLayers( TileId const & stackedTileId ) con
     return result;
 }
 
-void StackedTileLoader::updateTextureLayers()
+void StackedTileLoader::mergeDecorations( StackedTile * const tile ) const
 {
-    QList<GeoSceneDocument const *> const & mapThemes = d->m_mapThemeManager->mapThemes();
-    QList<GeoSceneDocument const *>::const_iterator pos = mapThemes.constBegin();
-    QList<GeoSceneDocument const *>::const_iterator const end = mapThemes.constEnd();
-    for (; pos != end; ++pos ) {
-        GeoSceneHead const * head = (*pos)->head();
-        Q_ASSERT( head );
-        const QString mapThemeId = head->target() + '/' + head->theme();
-        mDebug() << "StackedTileLoader::updateTextureLayers" << mapThemeId;
+    Q_ASSERT( !tile->resultTile()->isNull() );
+    Q_ASSERT( !d->m_textureLayers.isEmpty() );
+//    mDebug() << "MarbleModel::paintTile: " << "x: " << x << "y:" << y << "level: " << level
+//             << "requestTileUpdate" << requestTileUpdate;
 
-        GeoSceneMap const * map = (*pos)->map();
-        Q_ASSERT( map );
-        GeoSceneLayer const * sceneLayer = map->layer( head->theme() );
-        if ( !sceneLayer ) {
-            mDebug() << "ignoring, has no GeoSceneLayer for" << head->theme();
-            continue;
-        }
+    d->m_layerDecorator.setInfo( tile->id() );
+    d->m_layerDecorator.setTile( tile->resultTile() );
 
-        uint hash = qHash( mapThemeId );
-        if ( d->m_sceneLayers.contains( hash ) ) {
-            mDebug() << "StackedTileLoader::updateTextureLayers:"
-                     << hash << mapThemeId
-                     << "already exists";
-            continue;
-        }
-
-        d->m_sceneLayers.insert( qHash( mapThemeId ), sceneLayer );
-
-        // find all texture layers
-        QVector<GeoSceneAbstractDataset *> layers = sceneLayer->datasets();
-        QVector<GeoSceneAbstractDataset *>::const_iterator pos = layers.constBegin();
-        QVector<GeoSceneAbstractDataset *>::const_iterator const end = layers.constEnd();
-        for (; pos != end; ++pos ) {
-            GeoSceneTexture * const textureLayer = dynamic_cast<GeoSceneTexture *>( *pos );
-            if ( !textureLayer ) {
-                mDebug() << "ignoring dataset, is not a texture layer";
-                continue;
-            }
-            d->m_textureLayers.insert( qHash( textureLayer->sourceDir() ), textureLayer );
-            mDebug() << "StackedTileLoader::updateTextureLayers" << "added texture layer:"
-                     << qHash( textureLayer->sourceDir() ) << textureLayer->sourceDir();
-        }
-    }
-
-    d->m_tileLoader->setTextureLayers( d->m_textureLayers );
-}
-
-void StackedTileLoader::mergeDecorations( StackedTile * const tile,
-                                          GeoSceneTexture * const textureLayer ) const
-{
-    Q_ASSERT( tile->state() != StackedTile::TileEmpty );
-    if ( !tile->forMergedLayerDecorator() )
-        m_parent->paintTile( tile, textureLayer );
+    d->m_layerDecorator.paint( "maps/" + d->m_textureLayers.at( 0 )->sourceDir() );
 }
 
 // This method should not alter m_tileCache, as the given tile is managed
@@ -533,10 +470,9 @@ void StackedTileLoader::reloadCachedTile( StackedTile * const cachedTile,
     for (; pos != end; ++pos ) {
         d->m_tileLoader->reloadTile( *pos, usage );
     }
-    cachedTile->deriveCompletionState();
     cachedTile->initResultTile();
-    mergeDecorations( cachedTile, findTextureLayer( cachedTile->id() ));
-    emit tileUpdateAvailable();
+    mergeDecorations( cachedTile );
+    emit tileUpdateAvailable( cachedTile->id() );
 }
 
 }
