@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -41,7 +41,6 @@ class StackedTilePrivate : AbstractTilePrivate
 
     int       m_depth;
     bool      m_isGrayscale;
-    bool      m_forMergedLayerDecorator;
 
     explicit StackedTilePrivate( const TileId& id );
     virtual ~StackedTilePrivate();
