@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -15,10 +15,10 @@
 #include "GeoDataContainer_p.h"
 
 // Qt
-#include <QtCore/QDebug>
 #include <QtGui/QImage>
 
 // Marble
+#include "MarbleDebug.h"
 #include "GeoDataFeature.h"
 #include "GeoDataFolder.h"
 #include "GeoDataPlacemark.h"
@@ -41,11 +41,6 @@ GeoDataContainer::GeoDataContainer( const GeoDataContainer& other )
 {
 }
 
-GeoDataContainer::GeoDataContainer( const GeoDataFeature& other )
-    : GeoDataFeature( other )
-{
-}
-
 GeoDataContainer::~GeoDataContainer()
 {
 }
@@ -55,33 +50,38 @@ GeoDataContainerPrivate* GeoDataContainer::p() const
     return static_cast<GeoDataContainerPrivate*>(d);
 }
 
-QVector<GeoDataFolder> GeoDataContainer::folders() const
+QString GeoDataContainer::nodeType() const
 {
-    QVector<GeoDataFolder> results;
+    return p()->nodeType();
+}
 
-    QVector<GeoDataFeature>::const_iterator it = p()->m_vector.constBegin();
-    QVector<GeoDataFeature>::const_iterator end = p()->m_vector.constEnd();
+QVector<GeoDataFolder*> GeoDataContainer::folderList() const
+{
+    QVector<GeoDataFolder*> results;
+
+    QVector<GeoDataFeature*>::const_iterator it = p()->m_vector.constBegin();
+    QVector<GeoDataFeature*>::const_iterator end = p()->m_vector.constEnd();
 
     for (; it != end; ++it) {
-        GeoDataFeature feature = *it;
-
-        if ( feature.featureId() == GeoDataFolderId )
-            results.append( feature );
+        GeoDataFolder *folder = dynamic_cast<GeoDataFolder*>(*it);
+        if ( folder ) {
+            results.append( folder );
+        }
     }
 
     return results;
 }
 
-QVector<GeoDataPlacemark> GeoDataContainer::placemarks() const
+QVector<GeoDataPlacemark*> GeoDataContainer::placemarkList() const
 {
-    QVector<GeoDataPlacemark> results;
+    QVector<GeoDataPlacemark*> results;
 
-    QVector<GeoDataFeature>::const_iterator it = p()->m_vector.constBegin();
-    QVector<GeoDataFeature>::const_iterator end = p()->m_vector.constEnd();
+    QVector<GeoDataFeature*>::const_iterator it = p()->m_vector.constBegin();
+    QVector<GeoDataFeature*>::const_iterator end = p()->m_vector.constEnd();
 
     for (; it != end; ++it) {
-        if ( GeoDataPlacemarkId == it->featureId() ) {
-            GeoDataPlacemark placemark = *it;
+        GeoDataPlacemark *placemark = dynamic_cast<GeoDataPlacemark*>( *it );
+        if ( placemark ) {
             results.append( placemark );
         }
     }
@@ -89,17 +89,49 @@ QVector<GeoDataPlacemark> GeoDataContainer::placemarks() const
     return results;
 }
 
-QVector<GeoDataFeature>& GeoDataContainer::features() const
+QVector<GeoDataFeature*> GeoDataContainer::featureList() const
 {
     return p()->m_vector;
 }
 
-void GeoDataContainer::append( const GeoDataFeature& other )
+/**
+ * @brief  returns the requested child item
+ */
+GeoDataFeature* GeoDataContainer::child( int i )
+{
+    return p()->m_vector.at(i);
+}
+
+/**
+ * @brief returns the position of an item in the list
+ */
+int GeoDataContainer::childPosition( GeoDataFeature* object )
+{
+    for ( int i=0; i< p()->m_vector.size(); i++ )
+    {
+        if ( p()->m_vector.at( i ) == object )
+        {
+            return i;
+        }
+    }
+    return -1;
+}
+
+
+void GeoDataContainer::append( GeoDataFeature *other )
 {
     detach();
+    other->setParent(this);
     p()->m_vector.append( other );
 }
 
+
+void GeoDataContainer::remove( int index )
+{
+    detach();
+    p()->m_vector.remove( index );
+}
+
 int GeoDataContainer::size() const
 {
     return p()->m_vector.size();
@@ -108,60 +140,59 @@ int GeoDataContainer::size() const
 GeoDataFeature& GeoDataContainer::at( int pos )
 {
     detach();
-    return p()->m_vector[ pos ];
+    return *(p()->m_vector[ pos ]);
 }
 
 const GeoDataFeature& GeoDataContainer::at( int pos ) const
 {
-    return p()->m_vector.at( pos );
+    return *(p()->m_vector.at( pos ));
 }
 
 GeoDataFeature& GeoDataContainer::last()
 {
     detach();
-    return p()->m_vector.last();
+    return *(p()->m_vector.last());
 }
 
 const GeoDataFeature& GeoDataContainer::last() const
 {
-    return p()->m_vector.last();
+    return *(p()->m_vector.last());
 }
 
 GeoDataFeature& GeoDataContainer::first()
 {
     detach();
-    return p()->m_vector.first();
+    return *(p()->m_vector.first());
 }
 
 const GeoDataFeature& GeoDataContainer::first() const
 {
-    return p()->m_vector.first();
+    return *(p()->m_vector.first());
 }
 
 void GeoDataContainer::clear()
 {
     GeoDataContainer::detach();
+    qDeleteAll(p()->m_vector);
     p()->m_vector.clear();
 }
 
-QVector<GeoDataFeature>::Iterator GeoDataContainer::begin()
+QVector<GeoDataFeature*>::Iterator GeoDataContainer::begin()
 {
-    GeoDataContainer::detach();
     return p()->m_vector.begin();
 }
 
-QVector<GeoDataFeature>::Iterator GeoDataContainer::end()
+QVector<GeoDataFeature*>::Iterator GeoDataContainer::end()
 {
-    GeoDataContainer::detach();
     return p()->m_vector.end();
 }
 
-QVector<GeoDataFeature>::ConstIterator GeoDataContainer::constBegin() const
+QVector<GeoDataFeature*>::ConstIterator GeoDataContainer::constBegin() const
 {
     return p()->m_vector.constBegin();
 }
 
-QVector<GeoDataFeature>::ConstIterator GeoDataContainer::constEnd() const
+QVector<GeoDataFeature*>::ConstIterator GeoDataContainer::constEnd() const
 {
     return p()->m_vector.constEnd();
 }
@@ -172,13 +203,13 @@ void GeoDataContainer::pack( QDataStream& stream ) const
 
     stream << p()->m_vector.count();
 
-    for ( QVector <GeoDataFeature>::const_iterator iterator = p()->m_vector.constBegin();
+    for ( QVector <GeoDataFeature*>::const_iterator iterator = p()->m_vector.constBegin();
           iterator != p()->m_vector.constEnd();
           ++iterator )
     {
-        const GeoDataFeature& feature = *iterator;
-        stream << feature.featureId();
-        feature.pack( stream );
+        const GeoDataFeature *feature = *iterator;
+        stream << feature->featureId();
+        feature->pack( stream );
     }
 }
 
@@ -198,15 +229,15 @@ void GeoDataContainer::unpack( QDataStream& stream )
                 /* not usable!!!! */ break;
             case GeoDataFolderId:
                 {
-                GeoDataFolder folder;
-                folder.unpack( stream );
+                GeoDataFolder *folder = new GeoDataFolder;
+                folder->unpack( stream );
                 p()->m_vector.append( folder );
                 }
                 break;
             case GeoDataPlacemarkId:
                 {
-                GeoDataPlacemark placemark;
-                placemark.unpack( stream );
+                GeoDataPlacemark *placemark = new GeoDataPlacemark;
+                placemark->unpack( stream );
                 p()->m_vector.append( placemark );
                 }
                 break;
