@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -8,21 +8,28 @@
 // Copyright 2009      Patrick Spendrin <ps_ml@gmx.de>
 //
 
-#ifndef GEODATACONTAINERPRIVATE_H
-#define GEODATACONTAINERPRIVATE_H
+#ifndef MARBLE_GEODATACONTAINERPRIVATE_H
+#define MARBLE_GEODATACONTAINERPRIVATE_H
 
 #include "GeoDataFeature_p.h"
 
+#include "GeoDataTypes.h"
+
 namespace Marble
 {
 
-class GeoDataContainerPrivate : public Marble::GeoDataFeaturePrivate
+class GeoDataContainerPrivate : public GeoDataFeaturePrivate
 {
   public:
     GeoDataContainerPrivate()
     {
     }
     
+    ~GeoDataContainerPrivate()
+    {
+        qDeleteAll(m_vector);
+    }
+
     virtual void* copy() 
     { 
         GeoDataContainerPrivate* copy = new GeoDataContainerPrivate;
@@ -40,9 +47,9 @@ class GeoDataContainerPrivate : public Marble::GeoDataFeaturePrivate
         return GeoDataFolderId;
     }
 
-    QVector<GeoDataFeature> m_vector;
+    QVector<GeoDataFeature*> m_vector;
 };
 
 } // namespace Marble
 
-#endif //GEODATACONTAINERPRIVATE_H
+#endif
