@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -10,8 +10,8 @@
 //
 
 
-#ifndef GEODATAFEATURE_H
-#define GEODATAFEATURE_H
+#ifndef MARBLE_GEODATAFEATURE_H
+#define MARBLE_GEODATAFEATURE_H
 
 
 #include <QtCore/QString>
@@ -21,7 +21,6 @@
 #include <QtGui/QColor>
 
 #include "GeoDataObject.h"
-#include "GeoDataLookAt.h"
 
 #include "geodata_export.h"
 
@@ -34,10 +33,15 @@ class GeoDataFolder;
 class GeoDataDocument;
 class GeoDataPlacemark;
 class GeoDataRegion;
+class GeoDataAbstractView;
 
 class GeoDataStyle;
 class GeoDataStyleMap;
 
+class GeoDataExtendedData;
+
+class GeoDataTimeSpan;
+class GeoDataTimeStamp;
 
 class GeoDataFeaturePrivate;
 
@@ -66,13 +70,7 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
     GeoDataFeature( const QString& name );
 
     GeoDataFeature( const GeoDataFeature& other );
-    
-    // copy ctors for derived classes
-    GeoDataFeature( const GeoDataContainer& other );
-    GeoDataFeature( const GeoDataFolder& other );
-    GeoDataFeature( const GeoDataDocument& other );
-    GeoDataFeature( const GeoDataPlacemark& other );
-    
+
     virtual ~GeoDataFeature();
 
     GeoDataFeature& operator=( const GeoDataFeature& other );
@@ -80,8 +78,6 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
 
     /// Provides type information for downcasting a GeoData
     virtual QString nodeType() const;
-    virtual bool isFolder() const { return false; }
-    virtual bool isPlacemark() const { return false; }
 
     EnumFeatureId featureId() const;
     /**
@@ -124,7 +120,7 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
         // Space Terrain
         Crater,                  // c
         Mare,                    // a
-        
+
         // Places of Interest
         GeographicPole,
         MagneticPole,
@@ -191,11 +187,16 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
     /// Set the description to be CDATA See: @see descriptionIsCDATA()
     void setDescriptionCDATA( bool cdata );
 
+    /// Get the Abstract view of the feature
+    GeoDataAbstractView abstractView() const;
+    /// Set the abstract view of the feature
+    void setAbstractView( const GeoDataAbstractView &abstractView );
+
     /// Return the styleUrl of the feature.
     QString styleUrl() const;
     /// Set the styleUrl of this feature to @p value.
     void setStyleUrl( const QString &value );
-    
+
     /// Return whether this feature is visible or not
     bool isVisible() const;
     /**
@@ -208,6 +209,28 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
      */ 
     void setVisible( bool value );
 
+    /**
+     * Return the timespan of the feature.
+     */
+    GeoDataTimeSpan& timeSpan() const;
+
+    /**
+     * Set the timespan of the feature.
+     * @param timeSpan new of timespan.	
+     */
+    void setTimeSpan( GeoDataTimeSpan timeSpan );
+
+    /**
+     * Return the timestamp of the feature.
+     */
+    GeoDataTimeStamp& timeStamp() const;
+
+    /**
+     * Set the timestamp of the feature.
+     * @param timeStamp new of the timestamp.
+     */
+    void setTimeStamp( GeoDataTimeStamp timeStamp );
+
     /**
      * Return the style assigned to the placemark.
      */
@@ -218,6 +241,17 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
      */
     void setStyle( GeoDataStyle* style );
 
+    /**
+     * Return the ExtendedData assigned to the feature.
+     */
+    GeoDataExtendedData& extendedData() const;
+
+    /**
+     * Sets the ExtendedData of the feature.
+     * @param  extendedData  the new ExtendedData to be used.
+     */
+    void setExtendedData( const GeoDataExtendedData& extendedData );
+
     /**
      * Return the region assigned to the placemark.
      */
@@ -233,7 +267,7 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
     /**
      * Return the symbol index of the placemark.
      */
-    GeoDataVisualCategory visualCategory() const; 
+    GeoDataVisualCategory visualCategory() const;
     /**
      * Sets the symbol @p index of the placemark.
      * @param  category  the new category to be used.
@@ -245,13 +279,12 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
      *
      * FIXME: describe roles here!
      */
-    const QChar role() const;
+    const QString role() const;
     /**
      * Sets the role of the placemark.
      * @param  role  the new role to be used.
      */
-    void setRole( const QChar &role );
-
+    void setRole( const QString &role );
 
     /**
      * @brief Return the popularity index of the placemark.
@@ -275,7 +308,7 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
      * @param  popularity  the new popularity value
      */
     void setPopularity( qint64 popularity );
-    
+
     /**
      * Return a pointer to a GeoDataStyleMap object which represents the styleMap
      * of this feature. A styleMap is simply a QMap<QString,QString> which can connect
@@ -289,18 +322,6 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
      */
     void setStyleMap( GeoDataStyleMap* map );
 
-    /**
-     * Returns pointer to the GeoDataLooAt of the feature.
-     */
-    GeoDataLookAt* lookAt() const;
-
-    /**
-     * Sets the lookAt of the placemark.
-     * @param  LookAt  the new LookAt of the feature.
-     */
-    void setLookAt( GeoDataLookAt *lookAt );
-
-	
 
     // ----------------------------------------------------------------
     // The following functions are use for painting, and mostly for placemarks.
@@ -352,4 +373,4 @@ class GEODATA_EXPORT GeoDataFeature : public GeoDataObject
 
 }
 
-#endif // GEODATAFEATURE_H
+#endif
