@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -14,9 +14,12 @@
 #include <QtCore/QString>
 #include <QtCore/QAtomicInt>
 
+#include "GeoDataExtendedData.h"
 #include "GeoDataAbstractView.h"
 #include "GeoDataFeature.h"
 #include "GeoDataRegion.h"
+#include "GeoDataTimeStamp.h"
+#include "GeoDataTimeSpan.h"
 
 #include "GeoDataTypes.h"
 
@@ -38,9 +41,12 @@ class GeoDataFeaturePrivate
         m_popularityIndex( 19 ),
         m_visible( true ),
         m_visualCategory( GeoDataFeature::Default ),
-        m_role(' '),
+        m_role(" "),
         m_style( 0 ),
         m_styleMap( 0 ),
+        m_extendedData(),
+        m_timeSpan(),
+        m_timeStamp(),
         m_region(),
         ref( 0 )
     {
@@ -60,6 +66,9 @@ class GeoDataFeaturePrivate
         m_role( other.m_role ),
         m_style( other.m_style ),               //FIXME: both style and stylemap need to be reworked internally!!!!
         m_styleMap( other.m_styleMap ),
+        m_extendedData( other.m_extendedData ),
+        m_timeSpan( other.m_timeSpan ),
+        m_timeStamp( other.m_timeStamp ),
         m_region( other.m_region ),
         ref( 0 )
     {
@@ -79,7 +88,10 @@ class GeoDataFeaturePrivate
         m_role = other.m_role;
         m_style = other.m_style;
         m_styleMap = other.m_styleMap;
+        m_timeSpan = other.m_timeSpan;
+        m_timeStamp = other.m_timeStamp;
         m_visualCategory = other.m_visualCategory;
+        m_extendedData = other.m_extendedData;
         m_region = other.m_region;
     }
     
@@ -118,11 +130,16 @@ class GeoDataFeaturePrivate
     GeoDataFeature::GeoDataVisualCategory  m_visualCategory; // the visual category
 
 
-    QChar       m_role;
+    QString       m_role;
 
     GeoDataStyle*    m_style;
     GeoDataStyleMap* m_styleMap;
 
+    GeoDataExtendedData m_extendedData;
+
+    GeoDataTimeSpan  m_timeSpan;
+    GeoDataTimeStamp m_timeStamp;
+
     GeoDataRegion m_region;
     
     QAtomicInt  ref;
