@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -47,99 +47,160 @@ QString GeoDataMultiGeometry::nodeType() const
     return p()->nodeType();
 }
 
+
+GeoDataLatLonAltBox GeoDataMultiGeometry::latLonAltBox() const
+{
+    QVector<GeoDataGeometry*>::const_iterator it = p()->m_vector.constBegin();
+    QVector<GeoDataGeometry*>::const_iterator end = p()->m_vector.constEnd();
+
+    GeoDataLatLonAltBox box;
+    for (; it != end; ++it) {
+        if ( !(*it)->latLonAltBox().isEmpty() ) {
+            if ( box.isEmpty() ) {
+                box = (*it)->latLonAltBox();
+            }
+            else {
+                box |= (*it)->latLonAltBox();
+            }
+        }
+    }
+    return box;
+}
+
 int GeoDataMultiGeometry::size() const
 {
     return p()->m_vector.size();
 }
 
-QVector<GeoDataGeometry>& GeoDataMultiGeometry::vector() const
+QVector<GeoDataGeometry> GeoDataMultiGeometry::vector() const
 {
-    return p()->m_vector;
+    QVector<GeoDataGeometry> results;
+
+    QVector<GeoDataGeometry*>::const_iterator it = p()->m_vector.constBegin();
+    QVector<GeoDataGeometry*>::const_iterator end = p()->m_vector.constEnd();
+
+    for (; it != end; ++it) {
+            GeoDataGeometry f = **it;
+            results.append( f );
+    }
+
+    return results;
 }
 
 GeoDataGeometry& GeoDataMultiGeometry::at( int pos )
 {
     mDebug() << "detaching!";
-    GeoDataGeometry::detach();
-    return p()->m_vector[ pos ];
+    detach();
+    return *(p()->m_vector[ pos ]);
 }
 
 const GeoDataGeometry& GeoDataMultiGeometry::at( int pos ) const
 {
-    return p()->m_vector.at( pos );
+    return *(p()->m_vector.at( pos ));
 }
 
 GeoDataGeometry& GeoDataMultiGeometry::operator[]( int pos )
 {
-    GeoDataGeometry::detach();
-    return p()->m_vector[ pos ];
+    detach();
+    return *(p()->m_vector[ pos ]);
 }
 
 const GeoDataGeometry& GeoDataMultiGeometry::operator[]( int pos ) const
 {
-    return p()->m_vector[ pos ];
+    return *(p()->m_vector[ pos ]);
 }
 
 GeoDataGeometry& GeoDataMultiGeometry::last()
 {
-    GeoDataGeometry::detach();
-    return p()->m_vector.last();
+    detach();
+    return *(p()->m_vector.last());
 }
 
 GeoDataGeometry& GeoDataMultiGeometry::first()
 {
-    GeoDataGeometry::detach();
-    return p()->m_vector.first();
+    detach();
+    return *(p()->m_vector.first());
 }
 
 const GeoDataGeometry& GeoDataMultiGeometry::last() const
 {
-    return p()->m_vector.last();
+    return *(p()->m_vector.last());
 }
 
 const GeoDataGeometry& GeoDataMultiGeometry::first() const
 {
-    return p()->m_vector.first();
+    return *(p()->m_vector.first());
 }
 
-QVector<GeoDataGeometry>::Iterator GeoDataMultiGeometry::begin()
+QVector<GeoDataGeometry*>::Iterator GeoDataMultiGeometry::begin()
 {
-    GeoDataGeometry::detach();
+    detach();
     return p()->m_vector.begin();
 }
 
-QVector<GeoDataGeometry>::Iterator GeoDataMultiGeometry::end()
+QVector<GeoDataGeometry*>::Iterator GeoDataMultiGeometry::end()
 {
-    GeoDataGeometry::detach();
+    detach();
     return p()->m_vector.end();
 }
 
-QVector<GeoDataGeometry>::ConstIterator GeoDataMultiGeometry::constBegin() const
+QVector<GeoDataGeometry*>::ConstIterator GeoDataMultiGeometry::constBegin() const
 {
     return p()->m_vector.constBegin();
 }
 
-QVector<GeoDataGeometry>::ConstIterator GeoDataMultiGeometry::constEnd() const
+QVector<GeoDataGeometry*>::ConstIterator GeoDataMultiGeometry::constEnd() const
 {
     return p()->m_vector.constEnd();
 }
 
-void GeoDataMultiGeometry::append ( const GeoDataGeometry& value )
+/**
+ * @brief  returns the requested child item
+ */
+GeoDataGeometry* GeoDataMultiGeometry::child( int i )
 {
-    GeoDataGeometry::detach();
-    p()->m_vector.append( value );
+    return p()->m_vector.at( i );
 }
 
+/**
+ * @brief returns the position of an item in the list
+ */
+int GeoDataMultiGeometry::childPosition( GeoDataGeometry *object)
+{
+    for ( int i=0; i< p()->m_vector.size(); i++ )
+    {
+        if ( p()->m_vector.at( i ) == object )
+        {
+            return i;
+        }
+    }
+    return -1;
+}
+
+/**
+* @brief add an element
+*/
+void GeoDataMultiGeometry::append( GeoDataGeometry *other )
+{
+    detach();
+    other->setParent( this );
+    p()->m_vector.append( other );
+}
+
+
 GeoDataMultiGeometry& GeoDataMultiGeometry::operator << ( const GeoDataGeometry& value )
 {
-    GeoDataGeometry::detach();
-    p()->m_vector.append( value );
+    detach();
+    GeoDataGeometry *g = new GeoDataGeometry( value );
+    g->setParent( this );
+    p()->m_vector.append( g );
     return *this;
 }
 
 void GeoDataMultiGeometry::clear()
 {
-    GeoDataGeometry::detach();
+    detach();
+    qDeleteAll(p()->m_vector);
     p()->m_vector.clear();
 }
 
@@ -149,19 +210,19 @@ void GeoDataMultiGeometry::pack( QDataStream& stream ) const
 
     stream << p()->m_vector.size();
     
-    for( QVector<GeoDataGeometry>::const_iterator iterator 
+    for( QVector<GeoDataGeometry*>::const_iterator iterator
           = p()->m_vector.constBegin(); 
          iterator != p()->m_vector.constEnd();
          ++iterator ) {
-        const GeoDataGeometry& geometry = *iterator;
-        stream << geometry.geometryId();
-        geometry.pack( stream );
+        const GeoDataGeometry *geometry = *iterator;
+        stream << geometry->geometryId();
+        geometry->pack( stream );
     }
 }
 
 void GeoDataMultiGeometry::unpack( QDataStream& stream )
 {
-    GeoDataGeometry::detach();
+    detach();
     GeoDataGeometry::unpack( stream );
 
     int size = 0;
@@ -176,36 +237,36 @@ void GeoDataMultiGeometry::unpack( QDataStream& stream )
                 break;
             case GeoDataPointId:
                 {
-                GeoDataPoint point;
-                point.unpack( stream );
+                GeoDataPoint *point = new GeoDataPoint;
+                point->unpack( stream );
                 p()->m_vector.append( point );
                 }
                 break;
             case GeoDataLineStringId:
                 {
-                GeoDataLineString lineString;
-                lineString.unpack( stream );
+                GeoDataLineString *lineString = new GeoDataLineString;
+                lineString->unpack( stream );
                 p()->m_vector.append( lineString );
                 }
                 break;
             case GeoDataLinearRingId:
                 {
-                GeoDataLinearRing linearRing;
-                linearRing.unpack( stream );
+                GeoDataLinearRing *linearRing = new GeoDataLinearRing;
+                linearRing->unpack( stream );
                 p()->m_vector.append( linearRing );
                 }
                 break;
             case GeoDataPolygonId:
                 {
-                GeoDataPolygon polygon;
-                polygon.unpack( stream );
+                GeoDataPolygon *polygon = new GeoDataPolygon;
+                polygon->unpack( stream );
                 p()->m_vector.append( polygon );
                 }
                 break;
             case GeoDataMultiGeometryId:
                 {
-                GeoDataMultiGeometry multiGeometry;
-                multiGeometry.unpack( stream );
+                GeoDataMultiGeometry *multiGeometry = new GeoDataMultiGeometry;
+                multiGeometry->unpack( stream );
                 p()->m_vector.append( multiGeometry );
                 }
                 break;
