@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -16,6 +16,8 @@
 
 #include "GeoDataDocument.h"
 
+#include "GeoDataTypes.h"
+
 
 namespace Marble
 {
@@ -25,7 +27,8 @@ class GeoDataObjectPrivate
   public:
     GeoDataObjectPrivate()
         : m_id(0),
-          m_targetId(0)
+          m_targetId(0),
+          m_parent(0)
     {
     }
 
@@ -36,6 +39,7 @@ class GeoDataObjectPrivate
 
     int  m_id;
     int  m_targetId;
+    GeoDataObject *m_parent;
 };
 
 GeoDataObject::GeoDataObject()
@@ -66,6 +70,16 @@ QString GeoDataObject::nodeType() const
     return d->nodeType();
 }
 
+GeoDataObject *GeoDataObject::parent() const
+{
+    return d->m_parent;
+}
+
+void GeoDataObject::setParent(GeoDataObject *parent)
+{
+    d->m_parent = parent;
+}
+
 int GeoDataObject::id() const
 {
     return d->m_id;
@@ -86,27 +100,6 @@ void GeoDataObject::setTargetId( int value )
     d->m_targetId = value;
 }
 
-GeoDataObject* GeoDataObject::child(int /* pos */ )
-{
-    return 0;
-}
-
-int GeoDataObject::row()
-{
-    return 0;
-}
-
-int GeoDataObject::childPosition( GeoDataObject *child )
-{
-    Q_UNUSED( child );
-    return 0;
-}
-
-int GeoDataObject::childCount()
-{
-    return 0;
-}
-
 void GeoDataObject::pack( QDataStream& stream ) const
 {
     stream << d->m_id;
