@@ -42,7 +42,7 @@ GeoNode* KmlMultiGeometryTagHandler::parse( GeoParser& parser ) const
 
     GeoStackItem parentItem = parser.parentElement();
 
-    GeoDataMultiGeometry geom;
+    GeoDataMultiGeometry *geom = new GeoDataMultiGeometry;
     if( parentItem.represents( kmlTag_Placemark ) ) {
         parentItem.nodeAs<GeoDataPlacemark>()->setGeometry( geom );
 #ifdef DEBUG_TAGS
@@ -50,14 +50,16 @@ GeoNode* KmlMultiGeometryTagHandler::parse( GeoParser& parser ) const
                  << " parent item name: " << parentItem.qualifiedName().first;
 #endif
         return parentItem.nodeAs<GeoDataPlacemark>()->geometry();
+
     } else if( parentItem.represents( kmlTag_MultiGeometry ) ) {
         parentItem.nodeAs<GeoDataMultiGeometry>()->append( geom );
 #ifdef DEBUG_TAGS
         mDebug() << "Parsed <" << kmlTag_MultiGeometry << ">"
                  << " parent item name: " << parentItem.qualifiedName().first;
 #endif
-        return &parentItem.nodeAs<GeoDataMultiGeometry>()->last();
+        return geom;
     } else {
+        delete geom;
         return 0;
     }
 }
