@@ -46,14 +46,15 @@ GeoNode* KmlPointTagHandler::parse( GeoParser& parser ) const
                  << " parent item name: " << parentItem.qualifiedName().first;
 #endif // DEBUG_TAGS
         return parentItem.nodeAs<GeoDataPlacemark>();
+
     } else if( parentItem.represents( kmlTag_MultiGeometry ) ) {
-        GeoDataPoint point;
+        GeoDataPoint *point = new GeoDataPoint;
         parentItem.nodeAs<GeoDataMultiGeometry>()->append( point );
 #ifdef DEBUG_TAGS
         mDebug() << "Parsed <" << kmlTag_Point << "> returning: " << parentItem.nodeAs<GeoDataMultiGeometry>()
                  << " parent item name: " << parentItem.qualifiedName().first;
 #endif // DEBUG_TAGS
-        return &parentItem.nodeAs<GeoDataMultiGeometry>()->last();
+        return point;
     }
     return 0;
 }
