@@ -20,9 +20,14 @@
 */
 
 #include "GeoDocument.h"
+#include <cstdio>
 
 namespace Marble
 {
+#if DUMP_GEONODE_LEAKS > 0
+     // Initialize here, as there is no GeoDocument.cpp file     
+     unsigned long GeoDocument::s_leakProtector = 0;     
+#endif      
 
 GeoDocument::GeoDocument()
 {
@@ -30,6 +35,14 @@ GeoDocument::GeoDocument()
 
 GeoDocument::~GeoDocument()
 {
+#if DUMP_GEONODE_LEAKS > 0
+    if (s_leakProtector != 0) {
+        fprintf(stderr, "Found %li GeoNode object LEAKS!\n", s_leakProtector);
+        s_leakProtector = 0;
+    } else {
+        fprintf(stderr, "No GeoNode object leak!\n");
+    }
+#endif
 }
 
 bool GeoDocument::isGeoDataDocument() const
@@ -45,10 +58,16 @@ bool GeoDocument::isGeoSceneDocument() const
 
 GeoNode::GeoNode()
 {
+#if DUMP_GEONODE_LEAKS > 0
+    GeoDocument::s_leakProtector++;
+#endif
 }
 
 GeoNode::~GeoNode()
 {
+#if DUMP_GEONODE_LEAKS > 0
+    --GeoDocument::s_leakProtector;
+#endif
 }
 
 }
