@@ -21,7 +21,7 @@
 
 #include "GeoSceneSettings.h"
 
-#include <QtCore/QDebug>
+#include "MarbleDebug.h"
 
 #include "GeoSceneProperty.h"
 #include "GeoSceneGroup.h"
@@ -32,10 +32,6 @@ namespace Marble
 class GeoSceneSettingsPrivate
 {
   public:
-    GeoSceneSettingsPrivate()
-    {
-    }
-
     ~GeoSceneSettingsPrivate()
     {
         qDeleteAll(m_properties);
@@ -43,7 +39,7 @@ class GeoSceneSettingsPrivate
     }
 
     /// The hash table holding all the properties in the settings.
-    QVector<GeoSceneProperty*>  m_properties;
+    QVector<GeoSceneProperty*> m_properties;
     QVector<GeoSceneGroup*> m_groups;
 };
 
@@ -58,10 +54,11 @@ GeoSceneSettings::~GeoSceneSettings()
     delete d;
 }
 
-bool GeoSceneSettings::propertyAvailable( const QString& name, bool& available )
+bool GeoSceneSettings::propertyAvailable( const QString& name, bool& available ) const
 {
     QVector<GeoSceneProperty*>::const_iterator it = d->m_properties.constBegin();
-    for (; it != d->m_properties.constEnd(); ++it) {
+    QVector<GeoSceneProperty*>::const_iterator propEnd = d->m_properties.constEnd();
+    for (; it != propEnd; ++it) {
         if ( (*it)->name() == name ) {
             available = (*it)->available();
             return true;
@@ -69,7 +66,8 @@ bool GeoSceneSettings::propertyAvailable( const QString& name, bool& available )
     }
 
     QVector<GeoSceneGroup*>::const_iterator itGroup = d->m_groups.constBegin();
-    for (; itGroup != d->m_groups.constEnd(); ++itGroup) {
+    QVector<GeoSceneGroup*>::const_iterator groupEnd = d->m_groups.constEnd();
+    for (; itGroup != groupEnd; ++itGroup) {
         bool success = (*itGroup)->propertyAvailable( name, available );
         if ( success ) {
             return true;
@@ -83,10 +81,11 @@ bool GeoSceneSettings::propertyAvailable( const QString& name, bool& available )
 
 bool GeoSceneSettings::setPropertyValue( const QString& name, bool value )
 {
-    qDebug() << "GeoSceneSettings: Property " << name << "to" << value; 
+    mDebug() << "GeoSceneSettings: Property " << name << "to" << value; 
     
     QVector<GeoSceneProperty*>::const_iterator it = d->m_properties.constBegin();
-    for (; it != d->m_properties.constEnd(); ++it) {
+    QVector<GeoSceneProperty*>::const_iterator propEnd = d->m_properties.constEnd();
+    for (; it != propEnd; ++it) {
         if ( (*it)->name() == name ) {
             (*it)->setValue( value );
             return true;
@@ -94,7 +93,8 @@ bool GeoSceneSettings::setPropertyValue( const QString& name, bool value )
     }
 
     QVector<GeoSceneGroup*>::const_iterator itGroup = d->m_groups.constBegin();
-    for (; itGroup != d->m_groups.constEnd(); ++itGroup) {
+    QVector<GeoSceneGroup*>::const_iterator groupEnd = d->m_groups.constEnd();
+    for (; itGroup != groupEnd; ++itGroup) {
         bool success = (*itGroup)->setPropertyValue( name, value );
         if ( success ) {
             return true;
@@ -104,10 +104,11 @@ bool GeoSceneSettings::setPropertyValue( const QString& name, bool value )
     return false;
 }
 
-bool GeoSceneSettings::propertyValue( const QString& name, bool& value )
+bool GeoSceneSettings::propertyValue( const QString& name, bool& value ) const
 {
     QVector<GeoSceneProperty*>::const_iterator it = d->m_properties.constBegin();
-    for (; it != d->m_properties.constEnd(); ++it) {
+    QVector<GeoSceneProperty*>::const_iterator propEnd = d->m_properties.constEnd();
+    for (; it != propEnd; ++it) {
         if ( (*it)->name() == name ) {
             value = (*it)->value();
             return true;
@@ -115,7 +116,8 @@ bool GeoSceneSettings::propertyValue( const QString& name, bool& value )
     }
 
     QVector<GeoSceneGroup*>::const_iterator itGroup = d->m_groups.constBegin();
-    for (; itGroup != d->m_groups.constEnd(); ++itGroup) {
+    QVector<GeoSceneGroup*>::const_iterator groupEnd = d->m_groups.constEnd();
+    for (; itGroup != groupEnd; ++itGroup) {
         bool success = (*itGroup)->propertyValue( name, value );
         if ( success ) {
             return true;
@@ -132,7 +134,8 @@ QVector<GeoSceneProperty*> GeoSceneSettings::allProperties() const
     QVector<GeoSceneProperty*> allProperties;
 
     QVector<GeoSceneGroup*>::const_iterator itGroup = d->m_groups.constBegin();
-    for (; itGroup != d->m_groups.constEnd(); ++itGroup) {
+    QVector<GeoSceneGroup*>::const_iterator groupEnd = d->m_groups.constEnd();
+    for (; itGroup != groupEnd; ++itGroup) {
         allProperties << (*itGroup)->properties();
     }
 
@@ -150,6 +153,7 @@ void GeoSceneSettings::addGroup( GeoSceneGroup* group )
         if ( currentGroup->name() == group->name() ) {
             delete currentGroup;
             it = d->m_groups.erase(it);
+            break;
         }
         else {
             ++it;
@@ -165,24 +169,29 @@ void GeoSceneSettings::addGroup( GeoSceneGroup* group )
     }
 }
 
-GeoSceneGroup* GeoSceneSettings::group( const QString& name )
+const GeoSceneGroup* GeoSceneSettings::group( const QString& name ) const
 {
     GeoSceneGroup* group = 0;
 
     QVector<GeoSceneGroup*>::const_iterator it = d->m_groups.constBegin();
-    for (; it != d->m_groups.constEnd(); ++it) {
-        if ( (*it)->name() == name )
+    QVector<GeoSceneGroup*>::const_iterator groupEnd = d->m_groups.constEnd();
+    for (; it != groupEnd; ++it) {
+        if ( (*it)->name() == name ) {
             group = *it;
-    }
-
-    if ( group ) {
-        Q_ASSERT(group->name() == name);
-        return group;
+            break;
+        }
     }
 
     return group;
 }
 
+// implement non-const method by means of const method,
+// for details, see "Effective C++" (third edition)
+GeoSceneGroup* GeoSceneSettings::group( const QString& name )
+{
+    return const_cast<GeoSceneGroup*>( static_cast<GeoSceneSettings const *>( this )->group( name ));
+}
+
 void GeoSceneSettings::addProperty( GeoSceneProperty* property )
 {
     // Remove any property that has the same name
@@ -192,6 +201,7 @@ void GeoSceneSettings::addProperty( GeoSceneProperty* property )
         if ( currentProperty->name() == property->name() ) {
             delete currentProperty;
             it = d->m_properties.erase(it);
+            break;
         }
         else {
             ++it;
@@ -208,24 +218,30 @@ void GeoSceneSettings::addProperty( GeoSceneProperty* property )
     }
 }
 
-GeoSceneProperty* GeoSceneSettings::property( const QString& name )
+const GeoSceneProperty* GeoSceneSettings::property( const QString& name ) const
 {
     GeoSceneProperty* property = 0;
 
     QVector<GeoSceneProperty*>::const_iterator it = d->m_properties.constBegin();
-    for (; it != d->m_properties.constEnd(); ++it) {
-        if ( (*it)->name() == name )
+    QVector<GeoSceneProperty*>::const_iterator propEnd = d->m_properties.constEnd();
+    for (; it != propEnd; ++it) {
+        if ( (*it)->name() == name ) {
             property = *it;
-    }
-
-    if ( property ) {
-        Q_ASSERT(property->name() == name);
-        return property;
+            break;
+        }
     }
 
     return property;
 }
 
+// implement non-const method by means of const method,
+// for details, see "Effective C++" (third edition)
+GeoSceneProperty* GeoSceneSettings::property( const QString& name )
+{
+    return const_cast<GeoSceneProperty*>
+        ( static_cast<GeoSceneSettings const *>( this )->property( name ));
+}
+
 QVector<GeoSceneProperty*> GeoSceneSettings::rootProperties() const
 {
     return d->m_properties;
