@@ -24,9 +24,12 @@
 
 #include "DownloadPolicy.h"
 #include "MarbleDebug.h"
+#include "MarbleDirs.h"
 #include "ServerLayout.h"
 #include "TileId.h"
 
+#include <QtGui/QImage>
+
 namespace Marble
 {
 
@@ -113,6 +116,22 @@ void GeoSceneTexture::setMaximumTileLevel( const int maximumTileLevel )
     m_maximumTileLevel = maximumTileLevel;
 }
 
+const QSize GeoSceneTexture::tileSize() const
+{
+    if ( !m_tileSize.isValid() ) {
+        const TileId id( sourceDir(), 0, 0, 0 );
+        const QString path = MarbleDirs::path( relativeTileFileName( id ));
+
+        QImage testTile( path );
+        Q_ASSERT( !testTile.isNull() );
+
+        m_tileSize = testTile.size();
+        Q_ASSERT( !tileSize().isEmpty() );
+    }
+
+    return m_tileSize;
+}
+
 GeoSceneTexture::Projection GeoSceneTexture::projection() const
 {
     return m_projection;
@@ -123,7 +142,9 @@ void GeoSceneTexture::setProjection( const Projection projection )
     m_projection = projection;
 }
 
-QUrl GeoSceneTexture::downloadUrl( const TileId &id )
+// Even though this method changes the internal state, it may be const
+// because the compiler is forced to invoke this method for different TileIds.
+QUrl GeoSceneTexture::downloadUrl( const TileId &id ) const
 {
     // default download url
     if ( m_downloadUrls.empty() )
@@ -163,7 +184,7 @@ QString GeoSceneTexture::relativeTileFileName( const TileId &id ) const
             .arg( id.x(), tileDigits, 10, QChar('0') )
             .arg( suffix );
         break;
-    case GeoSceneTexture::Other:
+    case GeoSceneTexture::OpenStreetMap:
         relFileName = QString( "%1/%2/%3/%4.%5" )
             .arg( themeStr() )
             .arg( id.zoomLevel() )
