@@ -1,5 +1,5 @@
 //
-// This file is part of the Marble Desktop Globe.
+// This file is part of the Marble Virtual Globe.
 //
 // This program is free software licensed under the GNU LGPL. You can
 // find a copy of this license in LICENSE.txt in the top directory of
@@ -8,9 +8,9 @@
 // Copyright 2007      Tobias Koenig  <tokoe@kde.org>
 // Copyright 2008      Inge Wallin    <inge@lysator.liu.se>
 // Copyright 2009      Jens-Michael Hoffmann <jensmh@gmx.de>
+// Copyright 2010      Harshit Jain   <hjain.itbhu@gmail.com>
 //
 
-
 // Own
 #include "marble_part.h"
 
@@ -24,9 +24,7 @@
 #include <QtGui/QFontMetrics>
 #include <QtGui/QPrinter>
 #include <QtGui/QPrintDialog>
-#include <QtGui/QPrintPreviewDialog>
 #include <QtGui/QProgressBar>
-#include <QtGui/QPainter>
 #include <QtGui/QStandardItemModel>
 #include <QtNetwork/QNetworkProxy>
 
@@ -55,21 +53,35 @@
 // Marble library classes
 #include "AbstractFloatItem.h"
 #include "AbstractDataPlugin.h"
+#include "BookmarkInfoDialog.h"
+#include "BookmarkManager.h"
 #include "DownloadRegionDialog.h"
 #include "GeoDataCoordinates.h"
+#include "GeoDataFolder.h"
 #include "GeoDataLatLonAltBox.h"
+#include "GeoDataLookAt.h"
+#include "GeoDataPlacemark.h"
 #include "HttpDownloadManager.h"
 #include "MarbleCacheSettingsWidget.h"
 #include "MarbleDirs.h"
+#include "MarbleDebug.h"
 #include "MarbleLocale.h"
-#include "MarbleMap.h"
 #include "MarbleModel.h"
 #include "MarblePluginSettingsWidget.h"
+#include "NewFolderInfoDialog.h"
+#include "routing/RoutingManager.h"
+#include "routing/RoutingProfilesModel.h"
+#include "routing/RoutingProfilesWidget.h"
 #include "SunControlWidget.h"
+#include "TimeControlWidget.h"
 #include "SunLocator.h"
 #include "TileCoordsPyramid.h"
-#include "ViewParams.h"
 #include "ViewportParams.h"
+#include "MarbleClock.h"
+#include "routing/RoutingManager.h"
+#include "PositionTracking.h"
+#include "PositionProviderPlugin.h"
+#include "PluginManager.h"
 
 // Marble non-library classes
 #include "ControlView.h"
@@ -79,6 +91,7 @@ using namespace Marble;
 
 #include "ui_MarbleViewSettingsWidget.h"
 #include "ui_MarbleNavigationSettingsWidget.h"
+#include "ui_MarbleTimeSettingsWidget.h"
 
 namespace Marble
 {
@@ -88,6 +101,7 @@ namespace
     const char* POSITION_STRING = I18N_NOOP( "Position: %1" );
     const char* DISTANCE_STRING = I18N_NOOP( "Altitude: %1" );
     const char* TILEZOOMLEVEL_STRING = I18N_NOOP( "Tile Zoom Level: %1" );
+    const char* DATETIME_STRING =  "Time: %1";
 }
 
 typedef KParts::GenericFactory< MarblePart > MarblePartFactory;
@@ -96,9 +110,12 @@ K_EXPORT_COMPONENT_FACTORY( libmarble_part, MarblePartFactory )
 MarblePart::MarblePart( QWidget *parentWidget, QObject *parent, const QStringList &arguments )
   : KParts::ReadOnlyPart( parent ),
     m_sunControlDialog( 0 ),
+    m_timeControlDialog( 0 ),
     m_downloadRegionDialog( 0 ),
     m_pluginModel( 0 ),
     m_configDialog( 0 ),
+    m_position( NOT_AVAILABLE ),
+    m_tileZoomLevel( NOT_AVAILABLE ),
     m_positionLabel( 0 ),
     m_distanceLabel( 0 )
 {
@@ -131,11 +148,29 @@ MarblePart::MarblePart( QWidget *parentWidget, QObject *parent, const QStringLis
     m_statusBarExtension = new KParts::StatusBarExtension( this );
     m_statusBarExtension->statusBar()->setUpdatesEnabled( false );
 
-    m_position = NOT_AVAILABLE;
-    m_distance = m_controlView->marbleWidget()->distanceString();
-    m_tileZoomLevel = NOT_AVAILABLE;
+    // Load bookmark file. If it does not exist, a default one will be used.
+    m_controlView->marbleWidget()->loadBookmarkFile( "bookmarks/bookmarks.kml" );
 
-    QTimer::singleShot( 0, this, SLOT( initObject() ) );
+    initializeCustomTimezone();
+
+    setupStatusBar();
+    readSettings();
+    m_statusBarExtension->statusBar()->setUpdatesEnabled( true );
+
+    // Show startup location
+    switch ( MarbleSettings::onStartup() ) {
+    case LastLocationVisited: {
+            GeoDataLookAt target;
+            target.setLongitude( MarbleSettings::quitLongitude() );
+            target.setLatitude( MarbleSettings::quitLatitude() );
+            target.setRange( MarbleSettings::quitRange() );
+            m_controlView->marbleWidget()->flyTo( target, Instant );
+        }
+        break;
+    case ShowHomeLocation:
+        m_controlView->marbleWidget()->goHome( Instant );
+        break;
+    }
 }
 
 MarblePart::~MarblePart()
@@ -146,14 +181,6 @@ MarblePart::~MarblePart()
     delete m_configDialog;
 }
 
-void MarblePart::initObject()
-{
-    QCoreApplication::processEvents ();
-    setupStatusBar();
-    readSettings();
-    m_statusBarExtension->statusBar()->setUpdatesEnabled( true );
-}
-
 ControlView* MarblePart::controlView() const
 {
     return m_controlView;
@@ -162,8 +189,8 @@ ControlView* MarblePart::controlView() const
 KAboutData *MarblePart::createAboutData()
 {
     return new KAboutData( I18N_NOOP( "marble_part" ), 0,
-                           ki18n( "A Desktop Globe" ),
-                           MARBLE_VERSION_STRING.toLatin1() );
+                           ki18n( "A Virtual Globe" ),
+                           ControlView::applicationVersion().toLatin1() );
 }
 
 bool MarblePart::openUrl( const KUrl &url )
@@ -180,14 +207,7 @@ bool MarblePart::openFile()
                                             widget(), i18n("Open File")
                                            );
     foreach( const QString &fileName, fileNames ) {
-        QString extension = fileName.section( '.', -1 );
-
-        if ( extension.compare( "gpx", Qt::CaseInsensitive ) == 0 ) {
-            m_controlView->marbleWidget()->openGpxFile( fileName );
-        }
-        else if ( extension.compare( "kml", Qt::CaseInsensitive ) == 0 ) {
-            m_controlView->marbleWidget()->addPlacemarkFile( fileName );
-        }
+        m_controlView->marbleWidget()->addGeoDataFile( fileName );
     }
 
     return true;
@@ -220,64 +240,6 @@ void MarblePart::exportMapScreenShot()
 }
 
 
-void MarblePart::printMapScreenShot()
-{
-#ifndef QT_NO_PRINTER
-    QPrinter printer( QPrinter::HighResolution );
-    QPrintDialog *printDialog = KdePrint::createPrintDialog(&printer, widget());
-
-    if (printDialog->exec()) {
-        QPixmap mapPixmap = m_controlView->mapScreenShot();
-        printPixmap( &printer, mapPixmap );
-    }
-    
-    delete printDialog;
-#endif
-}
-
-void MarblePart::printPixmap( QPrinter * printer, const QPixmap& pixmap  )
-{
-#ifndef QT_NO_PRINTER
-    QSize printSize = pixmap.size();
-
-    QRect mapPageRect = printer->pageRect();
-
-    printSize.scale( printer->pageRect().size(), Qt::KeepAspectRatio );
-
-    QPoint printTopLeft( ( mapPageRect.width() - printSize.width() ) / 2 ,
-                         ( mapPageRect.height() - printSize.height() ) / 2 );
-
-    QRect mapPrintRect( printTopLeft, printSize );
-
-    QPainter painter;
-    if (!painter.begin(printer))
-        return;
-    painter.drawPixmap( mapPrintRect, pixmap, pixmap.rect() );
-    painter.end();
-#endif
-}
-
-// QPointer is used because of issues described in http://www.kdedevelopers.org/node/3919
-void MarblePart::printPreview()
-{
-#ifndef QT_NO_PRINTER
-    QPrinter printer( QPrinter::HighResolution );
-
-    QPointer<QPrintPreviewDialog> preview = new QPrintPreviewDialog( &printer, widget() );
-    preview->setWindowFlags( Qt::Window );
-    connect( preview, SIGNAL( paintRequested( QPrinter * ) ), SLOT( paintPrintPreview( QPrinter * ) ) );
-    preview->exec();
-    delete preview;
-#endif
-}
-
-void MarblePart::paintPrintPreview( QPrinter * printer )
-{
-#ifndef QT_NO_PRINTER
-    QPixmap mapPixmap = m_controlView->mapScreenShot();
-    printPixmap( printer, mapPixmap );
-#endif
-}
 
 void MarblePart::setShowClouds( bool isChecked )
 {
@@ -308,6 +270,11 @@ void MarblePart::showTileZoomLevelLabel( bool isChecked )
     m_tileZoomLevelLabel->setVisible( isChecked );
 }
 
+void MarblePart::showDateTimeLabel( bool isChecked )
+{
+    m_clockLabel->setVisible( isChecked );
+}
+
 void MarblePart::showDownloadProgressBar( bool isChecked )
 {
     MarbleSettings::setShowDownloadProgressBar( isChecked );
@@ -341,9 +308,13 @@ void MarblePart::showStatusBar( bool isChecked )
 void MarblePart::controlSun()
 {
     if ( !m_sunControlDialog ) {
-        m_sunControlDialog = new SunControlWidget( NULL, m_controlView->sunLocator() );
+        m_sunControlDialog = new SunControlWidget( m_controlView->sunLocator() );
         connect( m_sunControlDialog, SIGNAL( showSun( bool ) ),
                  this,               SLOT ( showSun( bool ) ) );
+        connect( m_sunControlDialog, SIGNAL( showSun( bool ) ),
+                 m_showShadow,               SLOT ( setChecked( bool ) ) );
+        connect( m_sunControlDialog, SIGNAL( showSunInZenith( bool ) ),
+                 m_showSunInZenith,               SLOT ( setChecked( bool ) ) );
     }
 
     m_sunControlDialog->show();
@@ -351,10 +322,27 @@ void MarblePart::controlSun()
     m_sunControlDialog->activateWindow();
 }
 
+void MarblePart::controlTime()
+{
+    if ( !m_timeControlDialog ) 
+    {
+        m_timeControlDialog = new TimeControlWidget( m_controlView->marbleWidget()->model()->clock() );
+    }
+    m_timeControlDialog->show();
+    m_timeControlDialog->raise();
+    m_timeControlDialog->activateWindow();
+}
+
 
 void MarblePart::showSun( bool active )
 {
     m_controlView->marbleWidget()->sunLocator()->setShow( active ); 
+    m_sunControlDialog->setSunShading( active );
+}
+
+void MarblePart::showSunInZenith( bool active )
+{
+    m_controlView->marbleWidget()->sunLocator()->setCentered( active );
 }
 
 void MarblePart::workOffline( bool offline )
@@ -391,21 +379,10 @@ void MarblePart::setShowCurrentLocation( bool show )
 void MarblePart::readSettings()
 {
     kDebug() << "Start: MarblePart::readSettings()";
-    // Last location on quit
-    if ( MarbleSettings::onStartup() == LastLocationVisited ) {
-        m_controlView->marbleWidget()->centerOn(
-            MarbleSettings::quitLongitude(),
-            MarbleSettings::quitLatitude() );
-        m_controlView->marbleWidget()->zoomView( MarbleSettings::quitZoom() );
-    }
-
     // Set home position
     m_controlView->marbleWidget()->setHome( MarbleSettings::homeLongitude(),
                                             MarbleSettings::homeLatitude(),
                                             MarbleSettings::homeZoom() );
-    if ( MarbleSettings::onStartup() == ShowHomeLocation ) {
-        m_controlView->marbleWidget()->goHome();
-    }
 
     // Map theme and projection
     QString mapTheme = MarbleSettings::mapTheme();
@@ -431,8 +408,10 @@ void MarblePart::readSettings()
     
     // Sun
     m_controlView->sunLocator()->setShow( MarbleSettings::showSun() );
+    m_showShadow->setChecked( MarbleSettings::showSun() );
     m_controlView->sunLocator()->setCitylights( MarbleSettings::showCitylights() );
     m_controlView->sunLocator()->setCentered( MarbleSettings::centerOnSun() );
+    m_showSunInZenith->setChecked( MarbleSettings::centerOnSun() );
 
     // View
     m_initialGraphicsSystem = (GraphicsSystem) MarbleSettings::graphicsSystem();
@@ -474,10 +453,66 @@ void MarblePart::readSettings()
         }
     }
 
+    KSharedConfig::Ptr sharedConfig = KSharedConfig::openConfig( KGlobal::mainComponent() );
+    if ( sharedConfig->hasGroup( "Routing Profiles" ) ) {
+        QList<RoutingProfile> profiles;
+        KConfigGroup profilesGroup = sharedConfig->group( "Routing Profiles" );
+        int numProfiles = profilesGroup.readEntry( "Num", 0 );
+        for ( int i = 0; i < numProfiles; ++i ) {
+            KConfigGroup profileGroup = profilesGroup.group( QString( "Profile %0" ).arg(i) );
+            QString name = profileGroup.readEntry( "Name", tr( "Unnamed" ) );
+            RoutingProfile profile( name );
+            foreach ( const QString& pluginName, profileGroup.groupList() ) {
+                KConfigGroup pluginGroup = profileGroup.group( pluginName );
+                profile.pluginSettings().insert( pluginName, QHash<QString, QVariant>() );
+                foreach ( const QString& key, pluginGroup.keyList() ) {
+                    if ( key != "Enabled" ) {
+                        profile.pluginSettings()[ pluginName ].insert( key, pluginGroup.readEntry( key ) );
+                    }
+                }
+            }
+            profiles << profile;
+        }
+        m_controlView->marbleWidget()->model()->routingManager()->profilesModel()->setProfiles( profiles );
+    } else {
+        m_controlView->marbleWidget()->model()->routingManager()->profilesModel()->loadDefaultProfiles();
+    }
+
+    QString positionProvider = MarbleSettings::activePositionTrackingPlugin();
+    if ( !positionProvider.isEmpty() ) {
+        PositionTracking* tracking = m_controlView->marbleWidget()->model()->positionTracking();
+        PluginManager* pluginManager = m_controlView->marbleWidget()->model()->pluginManager();
+        QList<PositionProviderPlugin*> plugins = pluginManager->createPositionProviderPlugins();
+        foreach( PositionProviderPlugin* plugin, plugins ) {
+            if ( plugin->nameId() == positionProvider ) {
+                plugins.removeOne( plugin );
+                tracking->setPositionProviderPlugin( plugin );
+                break;
+            }
+            qDeleteAll( plugins );
+        }
+    }
+
     readStatusBarSettings();
 
     updateSettings();
+
+    // Time
+    if( MarbleSettings::systemTime() == true  )
+    {
+        m_controlView->marbleWidget()->model()->setClockDateTime( QDateTime::currentDateTime().toUTC() );
+        m_controlView->marbleWidget()->model()->setClockSpeed( 1 );
+    }
+    else if( MarbleSettings::lastSessionTime() == true )
+    {
+        m_controlView->marbleWidget()->model()->setClockDateTime( MarbleSettings::dateTime() );
+        m_controlView->marbleWidget()->model()->setClockSpeed( MarbleSettings::speedSlider() );
+    }
+
     readPluginSettings();
+    // Load previous route settings
+    m_controlView->marbleWidget()->model()->routingManager()->readSettings();
+
     disconnect( m_controlView->marbleWidget(), SIGNAL( pluginSettingsChanged() ),
                 this,                          SLOT( writePluginSettings() ) );
     connect( m_controlView->marbleWidget(), SIGNAL( pluginSettingsChanged() ),
@@ -498,6 +533,10 @@ void MarblePart::readStatusBarSettings()
     m_showTileZoomLevelAction->setChecked( showTileZoom );
     showTileZoomLevelLabel( showTileZoom );
 
+    const bool showDateTime = MarbleSettings::showDateTimeLabel();
+    m_showDateTimeAction->setChecked( showDateTime );
+    showDateTimeLabel( showDateTime );
+
     const bool showProgress = MarbleSettings::showDownloadProgressBar();
     m_showDownloadProgressAction->setChecked( showProgress );
     showDownloadProgressBar( showProgress );
@@ -506,13 +545,13 @@ void MarblePart::readStatusBarSettings()
 void MarblePart::writeSettings()
 {
     // Get the 'quit' values from the widget and store them in the settings.
-    qreal  quitLon = m_controlView->marbleWidget()->centerLongitude();
-    qreal  quitLat = m_controlView->marbleWidget()->centerLatitude();
-    int     quitZoom = m_controlView->marbleWidget()->zoom();
+    qreal  quitLon = m_controlView->marbleWidget()->lookAt().longitude();
+    qreal  quitLat = m_controlView->marbleWidget()->lookAt().latitude();
+    qreal  quitRange = m_controlView->marbleWidget()->lookAt().range();
 
     MarbleSettings::setQuitLongitude( quitLon );
     MarbleSettings::setQuitLatitude( quitLat );
-    MarbleSettings::setQuitZoom( quitZoom );
+    MarbleSettings::setQuitRange( quitRange );
 
     // Get the 'home' values from the widget and store them in the settings.
     qreal  homeLon = 0;
@@ -566,6 +605,10 @@ void MarblePart::writeSettings()
     MarbleSettings::setPersistentTileCacheLimit( m_controlView->marbleWidget()->
                                                  persistentTileCacheLimit() / 1024 );
     
+    // Time
+    MarbleSettings::setDateTime( m_controlView->marbleWidget()->model()->clockDateTime() );
+    MarbleSettings::setSpeedSlider( m_controlView->marbleWidget()->model()->clockSpeed() );
+
     // Plugins
     QList<int>   pluginEnabled;
     QList<int>   pluginVisible;
@@ -583,11 +626,21 @@ void MarblePart::writeSettings()
     MarbleSettings::setPluginVisible( pluginVisible );
     MarbleSettings::setPluginNameId(  pluginNameId );
 
+    QString positionProvider;
+    PositionTracking* tracking = m_controlView->marbleWidget()->model()->positionTracking();
+    if ( tracking && tracking->positionProviderPlugin() ) {
+        positionProvider = tracking->positionProviderPlugin()->nameId();
+    }
+    MarbleSettings::setActivePositionTrackingPlugin( positionProvider );
+
     MarbleSettings::setLockFloatItemPositions( m_lockFloatItemsAct->isChecked() );
 
     writeStatusBarSettings();
 
     MarbleSettings::self()->writeConfig();
+
+    // Store current route settings
+    m_controlView->marbleWidget()->model()->routingManager()->writeSettings();
 }
 
 void MarblePart::writeStatusBarSettings()
@@ -595,6 +648,7 @@ void MarblePart::writeStatusBarSettings()
     MarbleSettings::setShowPositionLabel( m_showPositionAction->isChecked() );
     MarbleSettings::setShowAltitudeLabel( m_showAltitudeAction->isChecked() );
     MarbleSettings::setShowTileZoomLevelLabel( m_showTileZoomLevelAction->isChecked() );
+    MarbleSettings::setShowDateTimeLabel( m_showDateTimeAction->isChecked() );
     MarbleSettings::setShowDownloadProgressBar( m_showDownloadProgressAction->isChecked() );
 }
 
@@ -611,7 +665,7 @@ void MarblePart::setupActions()
     m_printMapAction = KStandardAction::print( this, SLOT( printMapScreenShot() ),
                                                actionCollection() );
 
-    m_printPreviewAction = KStandardAction::printPreview( this, SLOT( printPreview() ),
+    m_printPreviewAction = KStandardAction::printPreview( m_controlView, SLOT( printPreview() ),
                                                actionCollection() );
                                                
     // Action: Export Map
@@ -729,7 +783,14 @@ void MarblePart::setupActions()
     connect( m_controlSunAction, SIGNAL( triggered( bool ) ),
 	     this,               SLOT( controlSun() ) );
 
-    KStandardAction::redisplay( this, SLOT( reload() ), actionCollection() );
+    KStandardAction::redisplay( m_controlView->marbleWidget(), SLOT( reloadMap() ), actionCollection() );
+
+    // Action: Show Time options
+    m_controlTimeAction = new KAction( this );
+    actionCollection()->addAction( "control_time", m_controlTimeAction );
+    m_controlTimeAction->setText( i18nc( "Action for time control dialog", "&Time Control..." ) );
+    connect( m_controlTimeAction, SIGNAL( triggered( bool ) ),
+         this,               SLOT( controlTime() ) );
 
     // Action: Lock float items
     m_lockFloatItemsAct = new KAction ( this );
@@ -745,6 +806,22 @@ void MarblePart::setupActions()
     KStandardAction::preferences( this, SLOT( editSettings() ),
 				  actionCollection() );
 
+    //Toggle Action: Show sun shadow
+    m_showShadow = new KToggleAction( i18n( "Show Shadow" ), this );
+    m_showShadow->setIcon( KIcon( "" ) );        // Fixme: Add Icon
+    actionCollection()->addAction( "sun_shadow", m_showShadow );
+    m_showShadow->setCheckedState( KGuiItem( i18n( "Hide Shadow" ) ) );
+    m_showShadow->setToolTip(i18n("Shows and hides the shadow of the sun"));
+    connect( m_showShadow, SIGNAL( triggered( bool ) ), this, SLOT( showSun( bool ) ));
+
+    //Toggle Action: Show sun zenith
+    m_showSunInZenith = new KToggleAction( i18n( "Show Zenith" ), this );
+    m_showSunInZenith->setIcon( KIcon( MarbleDirs::path( "svg/sunshine.png" ) ) );
+    actionCollection()->addAction( "sun_zenith", m_showSunInZenith );
+    m_showSunInZenith->setCheckedState( KGuiItem( i18n( "Hide Zenith" ) ) );
+    m_showSunInZenith->setToolTip( i18n( "Shows and hides the zenith location of the sun" ) );
+    connect( m_showSunInZenith, SIGNAL( triggered( bool ) ), this, SLOT( showSunInZenith( bool ) ));
+    
     //    FIXME: Discuss if this is the best place to put this
     QList<RenderPlugin *>::const_iterator it = pluginList.constBegin();
     QList<RenderPlugin *>::const_iterator const itEnd = pluginList.constEnd();
@@ -752,6 +829,81 @@ void MarblePart::setupActions()
         connect( (*it), SIGNAL( actionGroupsChanged() ),
                  this, SLOT( createPluginMenus() ) );
     }
+
+    m_addBookmarkAction = new KAction( this );
+    actionCollection()->addAction( "add_bookmark", m_addBookmarkAction );
+    m_addBookmarkAction->setText( i18nc( "Add Bookmark", "&Add Bookmark" ) );
+    m_addBookmarkAction->setIcon( KIcon( ":/icons/bookmark-new.png" ) );
+    m_addBookmarkAction->setShortcut( Qt::CTRL + Qt::Key_B );
+    connect( m_addBookmarkAction, SIGNAL( triggered( ) ),
+             this,                SLOT( openBookmarkInfoDialog() ) );
+
+    
+    m_removeAllBookmarksAction = new KAction( this );
+    actionCollection()->addAction( "remove_all_bookmark", m_removeAllBookmarksAction );
+    m_removeAllBookmarksAction->setIcon( KIcon( ":/icons/bookmark-remove.png" ) );
+    m_removeAllBookmarksAction->setText( i18nc( "Remove All Bookmarks", "&Remove All Bookmarks" ) );
+    connect( m_removeAllBookmarksAction, SIGNAL( triggered( ) ),
+             this,                SLOT( removeAllBookmarks() ) );
+
+    
+    m_addBookmarkFolderAction = new KAction( this );
+    actionCollection()->addAction( "new_bookmark_folder", m_addBookmarkFolderAction );
+    m_addBookmarkFolderAction->setIcon( KIcon( ":/icons/bookmark-add-folder.png" ) );
+    m_addBookmarkFolderAction->setText( i18nc( "New Bookmark Folder", "&New Bookmark Folder" ) );
+    connect( m_addBookmarkFolderAction, SIGNAL( triggered( ) ),
+             this,                SLOT( openNewBookmarkFolderDialog() ) );
+
+    createFolderList();
+    connect( m_controlView->marbleWidget()->model()->bookmarkManager(),
+             SIGNAL( bookmarksChanged() ), this, SLOT( createFolderList() ) );
+}
+
+void MarblePart::createFolderList()
+{
+
+    QList<QAction*> actionList;
+
+   QVector<GeoDataFolder*> folders = m_controlView->marbleWidget()->bookmarkFolders();
+   QVector<GeoDataFolder*>::const_iterator i = folders.constBegin();
+   QVector<GeoDataFolder*>::const_iterator end = folders.constEnd();
+
+   for (; i != end; ++i ) {
+       QMenu *m_bookmarksListMenu = new QMenu( (*i)->name() );
+   
+       createBookmarksListMenu( m_bookmarksListMenu, *(*i) );
+       connect( m_bookmarksListMenu, SIGNAL( triggered ( QAction *) ),
+            this, SLOT( lookAtBookmark( QAction *) ) );
+   
+       actionList.append( m_bookmarksListMenu->menuAction() ); 
+   }
+   unplugActionList("folders");
+   plugActionList( "folders", actionList );
+}
+
+void MarblePart::createBookmarksListMenu( QMenu *m_bookmarksListMenu, const GeoDataFolder &folder )
+{
+    m_bookmarksListMenu->clear();
+    QVector<GeoDataPlacemark*> bookmarks = folder.placemarkList();
+    
+    QVector<GeoDataPlacemark*>::const_iterator i = bookmarks.constBegin();
+    QVector<GeoDataPlacemark*>::const_iterator end = bookmarks.constEnd();
+    
+    for (; i != end; ++i ) {
+        QAction *bookmarkAct = new QAction( (*i)->name(), this );
+        QVariant var;
+        GeoDataLookAt* lookAt = (*i)->lookAt();
+        if ( !lookAt ) {
+            GeoDataLookAt coordinateToLookAt;
+            coordinateToLookAt.setCoordinates( (*i)->coordinate() );
+            coordinateToLookAt.setRange( (*i)->coordinate().altitude() );
+        } else {
+            var.setValue( *lookAt );
+        }
+        bookmarkAct->setData( var );
+        m_bookmarksListMenu->addAction( bookmarkAct );
+    }
+
 }
 
 void MarblePart::createInfoBoxesMenu()
@@ -780,9 +932,8 @@ void MarblePart::createOnlineServicesMenu()
     QList<RenderPlugin *>::const_iterator const end = renderPluginList.constEnd();
     for (; i != end; ++i ) {
         // FIXME: This will go into the layer manager when AbstractDataPlugin is an interface
-        AbstractDataPlugin *dataPlugin = qobject_cast<AbstractDataPlugin *>(*i);
-        
-        if( dataPlugin ) {
+       
+        if( (*i)->renderType() == RenderPlugin::Online ) {
             actionList.append( (*i)->action() );
         }
     }
@@ -791,6 +942,11 @@ void MarblePart::createOnlineServicesMenu()
     plugActionList( "onlineservices_actionlist", actionList );
 }
 
+void MarblePart::showDateTime()
+{
+    m_clock = QLocale().toString( m_controlView->marbleWidget()->model()->clockDateTime().addSecs( m_controlView->marbleWidget()->model()->clockTimezone() ), QLocale::ShortFormat );
+    updateStatusBar();
+}
 
 void MarblePart::showPosition( const QString& position )
 {
@@ -798,12 +954,6 @@ void MarblePart::showPosition( const QString& position )
     updateStatusBar();
 }
 
-void MarblePart::showDistance( const QString& distance )
-{
-    m_distance = distance;
-    updateStatusBar();
-}
-
 void MarblePart::showZoomLevel( const int tileLevel )
 {
     if ( tileLevel == -1 )
@@ -846,7 +996,7 @@ void MarblePart::createPluginMenus()
 void MarblePart::updateTileZoomLevel()
 {
     const int tileZoomLevel =
-        m_controlView->marbleWidget()->map()->model()->tileZoomLevel();
+        m_controlView->marbleWidget()->tileZoomLevel();
     if ( tileZoomLevel == -1 )
         m_tileZoomLevel = NOT_AVAILABLE;
     else {
@@ -860,11 +1010,14 @@ void MarblePart::updateStatusBar()
         m_positionLabel->setText( i18n( POSITION_STRING, m_position ) );
 
     if ( m_distanceLabel )
-        m_distanceLabel->setText( i18n( DISTANCE_STRING, m_distance ) );
+        m_distanceLabel->setText( i18n( DISTANCE_STRING, m_controlView->marbleWidget()->distanceString() ) );
 
     if ( m_tileZoomLevelLabel )
         m_tileZoomLevelLabel->setText( i18n( TILEZOOMLEVEL_STRING,
                                              m_tileZoomLevel ) );
+
+    if ( m_clockLabel )
+        m_clockLabel->setText( i18n( DATETIME_STRING, m_clock ) );
 }
 
 void MarblePart::setupStatusBar()
@@ -879,17 +1032,24 @@ void MarblePart::setupStatusBar()
         QString( "%1 00.000,0 mu" ).arg(DISTANCE_STRING);
     m_distanceLabel = setupStatusBarLabel( templateDistanceString );
 
+    QString templateDateTimeString = QString( "%1 %2" ).arg( DATETIME_STRING , QLocale().toString( QDateTime::fromString ( "01:01:1000", "dd:mm:yyyy"), QLocale::ShortFormat ) );
+
+    m_clockLabel = setupStatusBarLabel( templateDateTimeString );
+
     const QString templateTileZoomLevelString = i18n( TILEZOOMLEVEL_STRING );
     m_tileZoomLevelLabel = setupStatusBarLabel( templateTileZoomLevelString );
 
     connect( m_controlView->marbleWidget(), SIGNAL( mouseMoveGeoPosition( QString ) ),
              this,                          SLOT( showPosition( QString ) ) );
     connect( m_controlView->marbleWidget(), SIGNAL( distanceChanged( QString ) ),
-             this,                          SLOT( showDistance( QString ) ) );
-    connect( m_controlView->marbleWidget()->model(), SIGNAL( tileLevelChanged( int )),
+             this,                          SLOT( updateStatusBar() ) );
+    connect( m_controlView->marbleWidget(), SIGNAL( tileLevelChanged( int )),
              SLOT( showZoomLevel( int )));
     connect( m_controlView->marbleWidget()->model(), SIGNAL( themeChanged( QString )),
              this, SLOT( mapThemeChanged( QString )), Qt::QueuedConnection );
+    connect( m_controlView->marbleWidget()->model()->clock(), SIGNAL( timeChanged() ),
+             this,                          SLOT( showDateTime() ) );
+
 
     setupDownloadProgressBar();
 
@@ -921,7 +1081,7 @@ void MarblePart::setupDownloadProgressBar()
     statusBar->addPermanentWidget( m_downloadProgressBar );
 
     HttpDownloadManager * const downloadManager =
-        m_controlView->marbleWidget()->map()->model()->downloadManager();
+        m_controlView->marbleWidget()->model()->downloadManager();
     Q_ASSERT( downloadManager );
     connect( downloadManager, SIGNAL( jobAdded() ), SLOT( downloadJobAdded() ) );
     connect( downloadManager, SIGNAL( jobRemoved() ), SLOT( downloadJobRemoved() ) );
@@ -939,6 +1099,8 @@ void MarblePart::setupStatusBarActions()
 
     m_showPositionAction = new KToggleAction( i18nc( "Action for toggling", "Show Position" ),
                                               this );
+    m_showDateTimeAction = new KToggleAction( i18nc( "Action for toggling", "Show Date and Time" ),
+                                              this );
     m_showAltitudeAction = new KToggleAction( i18nc( "Action for toggling", "Show Altitude" ),
                                               this );
     m_showTileZoomLevelAction = new KToggleAction( i18nc( "Action for toggling",
@@ -952,6 +1114,8 @@ void MarblePart::setupStatusBarActions()
              this, SLOT( showAltitudeLabel( bool ) ) );
     connect( m_showTileZoomLevelAction, SIGNAL( triggered( bool ) ),
              this, SLOT( showTileZoomLevelLabel( bool ) ) );
+    connect( m_showDateTimeAction, SIGNAL( triggered( bool ) ),
+             this, SLOT( showDateTimeLabel( bool ) ) );
     connect( m_showDownloadProgressAction, SIGNAL( triggered( bool ) ),
              this, SLOT( showDownloadProgressBar( bool ) ) );
 }
@@ -987,9 +1151,9 @@ void MarblePart::disconnectDownloadRegionDialog()
 
 void MarblePart::showDownloadRegionDialog()
 {
-    MarbleModel * const model = m_controlView->marbleWidget()->map()->model();
+    MarbleWidget * const marbleWidget = m_controlView->marbleWidget();
     if ( !m_downloadRegionDialog ) {
-        m_downloadRegionDialog = new DownloadRegionDialog( model, widget() );
+        m_downloadRegionDialog = new DownloadRegionDialog( marbleWidget, widget() );
         // it might be tempting to move the connects to DownloadRegionDialog's "accepted" and
         // "applied" signals, be aware that the "hidden" signal might be come before the "accepted"
         // signal, leading to a too early disconnect.
@@ -1002,11 +1166,10 @@ void MarblePart::showDownloadRegionDialog()
     // FIXME: get allowed range from current map theme
     m_downloadRegionDialog->setAllowedTileLevelRange( 0, 18 );
     m_downloadRegionDialog->setSelectionMethod( DownloadRegionDialog::VisibleRegionMethod );
-    ViewportParams const * const viewport =
-        m_controlView->marbleWidget()->map()->viewParams()->viewport();
+    ViewportParams const * const viewport = marbleWidget->viewport();
     m_downloadRegionDialog->setSpecifiedLatLonAltBox( viewport->viewLatLonAltBox() );
     m_downloadRegionDialog->setVisibleLatLonAltBox( viewport->viewLatLonAltBox() );
-    m_downloadRegionDialog->setVisibleTileLevel( model->tileZoomLevel() );
+    m_downloadRegionDialog->setVisibleTileLevel( marbleWidget->tileZoomLevel() );
 
     m_downloadRegionDialog->show();
     m_downloadRegionDialog->raise();
@@ -1019,8 +1182,8 @@ void MarblePart::downloadRegion()
     QString const mapThemeId = m_controlView->marbleWidget()->mapThemeId();
     QString const sourceDir = mapThemeId.left( mapThemeId.lastIndexOf( '/' ));
     kDebug() << "downloadRegion mapThemeId:" << mapThemeId << sourceDir;
-    TileCoordsPyramid const pyramid = m_downloadRegionDialog->region();
-    m_controlView->marbleWidget()->map()->model()->downloadRegion( sourceDir, pyramid );
+    QVector<TileCoordsPyramid> const pyramid = m_downloadRegionDialog->region();
+    m_controlView->marbleWidget()->downloadRegion( sourceDir, pyramid );
 }
 
 void MarblePart::showStatusBarContextMenu( const QPoint& pos )
@@ -1030,6 +1193,7 @@ void MarblePart::showStatusBarContextMenu( const QPoint& pos )
 
     KMenu statusBarContextMenu( m_controlView->marbleWidget() );
     statusBarContextMenu.addAction( m_showPositionAction );
+    statusBarContextMenu.addAction( m_showDateTimeAction );
     statusBarContextMenu.addAction( m_showAltitudeAction );
     statusBarContextMenu.addAction( m_showTileZoomLevelAction );
     statusBarContextMenu.addAction( m_showDownloadProgressAction );
@@ -1087,6 +1251,19 @@ void MarblePart::editSettings()
     connect( w_cacheSettings,               SIGNAL( clearPersistentCache() ),
 	     m_controlView->marbleWidget(), SLOT( clearPersistentTileCache() ) );
 
+    // time page
+    Ui_MarbleTimeSettingsWidget ui_timeSettings;
+    QWidget *w_timeSettings = new QWidget( 0 );
+
+    w_timeSettings->setObjectName( "time_page" );
+    ui_timeSettings.setupUi( w_timeSettings );
+    m_configDialog->addPage( w_timeSettings, i18n( "Date & Time" ), "clock" );
+
+    // routing page
+    RoutingProfilesWidget *w_routingSettings = new RoutingProfilesWidget( m_controlView->marbleWidget() );
+    w_routingSettings->setObjectName( "routing_page" );
+    m_configDialog->addPage( w_routingSettings, tr( "Routing" ) );
+
     // plugin page
     m_pluginModel = new QStandardItemModel( this );
     QStandardItem  *parentItem = m_pluginModel->invisibleRootItem();
@@ -1138,6 +1315,28 @@ void MarblePart::applyPluginState()
     for (; i != end; ++i ) {
         (*i)->applyItemState();
     }
+
+    QList<RoutingProfile>  profiles = m_controlView->marbleWidget()
+                        ->model()->routingManager()->profilesModel()->profiles();
+    KSharedConfig::Ptr sharedConfig = KSharedConfig::openConfig( KGlobal::mainComponent() );
+    KConfigGroup profilesGroup = sharedConfig->group( "Routing Profiles" );
+    profilesGroup.writeEntry( "Num", profiles.count() );
+    for ( int i = 0; i < profiles.count(); ++i ) {
+        KConfigGroup profileGroup = profilesGroup.group( QString( "Profile %0" ).arg(i) );
+        RoutingProfile profile = profiles.at( i );
+        profileGroup.writeEntry( "Name", profile.name() );
+        foreach ( const QString &pluginName, profileGroup.groupList() ) {
+            profileGroup.group( pluginName ).deleteGroup();
+        }
+        foreach ( const QString &key, profile.pluginSettings().keys() ) {
+            KConfigGroup pluginGroup = profileGroup.group( key );
+            pluginGroup.writeEntry( "Enabled", true );
+            foreach ( const QString& settingKey, profile.pluginSettings()[ key ].keys() ) {
+                Q_ASSERT( settingKey != "Enabled" );
+                pluginGroup.writeEntry( settingKey, profile.pluginSettings()[ key ][ settingKey ] );
+            }
+        }
+    }
 }
 
 void MarblePart::retrievePluginState()
@@ -1171,7 +1370,6 @@ void MarblePart::updateSettings()
     MarbleGlobal::getInstance()->locale()->
         setDistanceUnit( (DistanceUnit) MarbleSettings::distanceUnit() );
 
-    m_distance = m_controlView->marbleWidget()->distanceString();
     updateStatusBar();
 
     m_controlView->marbleWidget()->setAnimationsEnabled( MarbleSettings::animateTargetVoyage() );
@@ -1214,7 +1412,7 @@ void MarblePart::updateSettings()
     
     QNetworkProxy::setApplicationProxy(proxy);
     
-    m_controlView->marbleWidget()->updateChangedMap();
+    m_controlView->marbleWidget()->update();
 
     // Show message box
     if (    m_initialGraphicsSystem != graphicsSystem 
@@ -1226,11 +1424,22 @@ void MarblePart::updateSettings()
                                 i18n("Graphics System Change") );
     }    
     m_previousGraphicsSystem = graphicsSystem;
-}
 
-void MarblePart::reload()
-{
-    m_controlView->marbleWidget()->map()->reload();
+    // Time
+    if( MarbleSettings::systemTimezone() == true  )
+    {
+        QDateTime localTime = QDateTime::currentDateTime().toLocalTime();
+        localTime.setTimeSpec( Qt::UTC );
+        m_controlView->marbleWidget()->model()->setClockTimezone( QDateTime::currentDateTime().toUTC().secsTo( localTime ) );
+    }
+    else if( MarbleSettings::utc() == true )
+    {
+        m_controlView->marbleWidget()->model()->setClockTimezone( 0 );
+    }
+    else if( MarbleSettings::customTimezone() == true )
+    {
+        m_controlView->marbleWidget()->model()->setClockTimezone( m_timezone.value( MarbleSettings::chosenTimezone() ) );
+    }
 }
 
 void MarblePart::showPluginAboutDialog( const QString& nameId )
@@ -1341,6 +1550,87 @@ void MarblePart::downloadJobRemoved()
     m_downloadProgressBar->setUpdatesEnabled( true );
 }
 
+void MarblePart::openBookmarkInfoDialog()
+{
+    QPointer<BookmarkInfoDialog> m_bookmarkInfoDialog = new BookmarkInfoDialog( m_controlView->marbleWidget() );
+    m_bookmarkInfoDialog->exec();
+    delete m_bookmarkInfoDialog;
+}
+
+void MarblePart::removeAllBookmarks()
+{
+    QString const title = i18nc( "Application name", "Marble" );
+    QString const text = i18n( "Are you sure you want to delete all bookmarks?" );
+    if ( KMessageBox::questionYesNo( widget(), text, title ) == KMessageBox::Yes ) {
+        m_controlView->marbleWidget()->removeAllBookmarks();
+    }
+}
+
+void MarblePart::openNewBookmarkFolderDialog()
+{
+    QPointer<NewFolderInfoDialog> dialog = new NewFolderInfoDialog( m_controlView->marbleWidget());
+    dialog->exec();
+    delete dialog;
+}
+
+void MarblePart::lookAtBookmark( QAction *action)
+{
+        GeoDataLookAt temp = qvariant_cast<GeoDataLookAt>( action->data() ) ;
+        m_controlView->marbleWidget()->flyTo( temp ) ;
+                mDebug() << " looking at bookmark having longitude : "<< temp.longitude(GeoDataCoordinates::Degree)
+                         << " latitude :  "<< temp.latitude(GeoDataCoordinates::Degree)
+                         << " distance : " << temp.range();
+}
+
+
+void MarblePart::initializeCustomTimezone()
+{
+    if( m_timezone.count() == 0)
+    {
+        m_timezone.insert( 0, 0 );
+        m_timezone.insert( 1, 3600 );
+        m_timezone.insert( 2, 7200 );
+        m_timezone.insert( 3, 7200 );
+        m_timezone.insert( 4, 10800 );
+        m_timezone.insert( 5, 12600 );
+        m_timezone.insert( 6, 14400 );
+        m_timezone.insert( 7, 18000 );
+        m_timezone.insert( 8, 19800 );
+        m_timezone.insert( 9, 21600 );
+        m_timezone.insert( 10, 25200 );
+        m_timezone.insert( 11, 28800 );
+        m_timezone.insert( 12, 32400 );
+        m_timezone.insert( 13, 34200 );
+        m_timezone.insert( 14, 36000 );
+        m_timezone.insert( 15, 39600 );
+        m_timezone.insert( 16, 43200 );
+        m_timezone.insert( 17, -39600 );
+        m_timezone.insert( 18, -36000 );
+        m_timezone.insert( 19, -32400 );
+        m_timezone.insert( 20, -28800 );
+        m_timezone.insert( 21, -25200 );
+        m_timezone.insert( 22, -25200 );
+        m_timezone.insert( 23, -21600 );
+        m_timezone.insert( 24, -18000 );
+        m_timezone.insert( 25, -18000 );
+        m_timezone.insert( 26, -14400 );
+        m_timezone.insert( 27, -12600 );
+        m_timezone.insert( 28, -10800 );
+        m_timezone.insert( 29, -10800 );
+        m_timezone.insert( 30, -3600 );
+    }
+}
+
+void MarblePart::printMapScreenShot()
+{
+#ifndef QT_NO_PRINTER
+    QPrinter printer( QPrinter::HighResolution );
+    QPointer<QPrintDialog> printDialog = KdePrint::createPrintDialog(&printer, widget());
+    m_controlView->printMapScreenShot( printDialog );
+    delete printDialog;
+#endif
+}
+
 }
 
 #include "marble_part.moc"
